=== modified file '.bzrignore'
--- .bzrignore	2006-07-17 09:08:58 +0000
+++ .bzrignore	2006-08-15 13:13:27 +0000
@@ -29,3 +29,5 @@
 ./py2exe.log
 ./tools/win32/bzr.iss
 ./dist
+# performance history data file
+./.perf_history
\ No newline at end of file

=== modified file 'Makefile'
--- Makefile	2006-08-11 14:38:04 +0000
+++ Makefile	2006-08-17 13:33:50 +0000
@@ -31,12 +31,12 @@
 	ctags-exuberant -e $(tag_files)
 
 
-# make bzr.exe for win32 with py2exe
-exe:
-	@echo *** Make bzr.exe
-	python setup.py py2exe > py2exe.log
-	python tools/win32/ostools.py copytodir tools/win32/start_bzr.bat win32_bzr.exe
-	python tools/win32/ostools.py copytodir tools/win32/bazaar.url win32_bzr.exe
+# Produce HTML docs to upload on Canonical server
+HTMLDIR := html_docs
+
+html-docs: docs
+	python tools/win32/ostools.py copytodir $(htm_files) doc/default.css $(HTMLDIR)
+
 
 # translate txt docs to html
 doc_dir := doc 
@@ -44,16 +44,34 @@
 htm_files := $(patsubst %.txt, %.htm, $(txt_files)) doc/bzr_man.htm
 
 %.htm: %.txt
-	rst2html.py --link-stylesheet --stylesheet=default.css $*.txt $*.htm
+	python tools/rst2html.py --link-stylesheet --stylesheet=default.css $*.txt $*.htm
 
-doc/bzr_man.txt:
+doc/bzr_man.txt: bzrlib/builtins.py \
+		 bzrlib/bundle/commands.py \
+		 bzrlib/conflicts.py \
+		 bzrlib/sign_my_commits.py \
+		 generate_docs.py \
+		 tools/doc_generate/__init__.py \
+		 tools/doc_generate/autodoc_rstx.py
 	python generate_docs.py -o doc/bzr_man.txt rstx
 
 docs: $(htm_files)
 
-copy_docs: docs
+copy-docs: docs
 	python tools/win32/ostools.py copytodir $(htm_files) doc/default.css NEWS README  win32_bzr.exe/doc
 
+# clean produced docs
+clean-docs:
+	python tools/win32/ostools.py remove doc/bzr_man.txt $(htm_files) $(HTMLDIR)
+
+
+# make bzr.exe for win32 with py2exe
+exe:
+	@echo *** Make bzr.exe
+	python setup.py py2exe > py2exe.log
+	python tools/win32/ostools.py copytodir tools/win32/start_bzr.bat win32_bzr.exe
+	python tools/win32/ostools.py copytodir tools/win32/bazaar.url win32_bzr.exe
+
 # win32 installer for bzr.exe
 installer: exe copy_docs
 	@echo *** Make windows installer
@@ -67,7 +85,7 @@
 
 
 # clean on win32 all installer-related files and directories
-clean_win32:
+clean-win32:
 	python tools/win32/ostools.py remove build
 	python tools/win32/ostools.py remove win32_bzr.exe
 	python tools/win32/ostools.py remove py2exe.log

=== modified file 'NEWS'
--- NEWS	2006-08-15 15:57:56 +0000
+++ NEWS	2006-08-17 14:18:30 +0000
@@ -1,6 +1,8 @@
 IN DEVELOPMENT
   
   IMPROVEMENTS:
+    * 'merge' now takes --uncommitted, to apply uncommitted changes from a
+      tree.  (Aaron Bentley)
   
     * 'bzr add --file-ids-from' can be used to specify another path to use
       for creating file ids, rather than generating all new ones. Internally,
@@ -12,11 +14,28 @@
       This will cache some of the intermediate trees, and decrease the
       setup time for benchmark tests. (John Arbash Meinel)
 
+    * Inverse forms are provided for all boolean options.  For example,
+      --strict has --no-strict, --no-recurse has --recurse (Aaron Bentley)
+
+    * Serialize out Inventories directly, rather than using ElementTree.
+      Writing out a kernel sized inventory drops from 2s down to ~350ms.
+      (Robert Collins, John Arbash Meinel)
+
   BUG FIXES:
 
+    * Handle TZ=UTC properly when reading/writing revisions.
+      (John Arbash Meinel, #55783, #56290)
+
     * Use GPG_TTY to allow gpg --cl to work with gpg-agent in a pipeline,
       (passing text to sign in on stdin). (John Arbash Meinel, #54468)
 
+    * External diff does the right thing for binaries even in foreign 
+      languages. (John Arbash Meinel, #56307)
+
+    * Testament handles more cases when content is unicode. Specific bug was
+      in handling of revision properties. (John Arbash Meinel, Holger Krekel,
+      #54723)
+
   INTERNALS:
 
     * ``bzrlib.cache_utf8`` contains ``encode()`` and ``decode()`` functions
@@ -25,6 +44,10 @@
       convert revision ids to utf8 to annotate lines in storage.
       (John Arbash Meinel)
 
+    * ``setup.py`` now searches the filesystem to find all packages which
+      need to be installed. This should help make the life of packagers
+      easier. (John Arbash Meinel)
+
 bzr 0.9.0  2006-08-11
 
   SURPRISES:

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2006-08-14 13:35:31 +0000
+++ bzrlib/__init__.py	2006-08-16 02:53:40 +0000
@@ -60,23 +60,6 @@
                     'Consider using bzrlib.ignores.add_unique_user_ignores'
                     ' or bzrlib.ignores.add_runtime_ignores')
 
-
-@deprecated_function(zero_seven)
-def get_bzr_revision():
-    """If bzr is run from a branch, return (revno,revid) or None."""
-    import bzrlib.errors
-    from bzrlib.branch import Branch
-    
-    try:
-        branch = Branch.open(os.path.dirname(__path__[0]))
-        rh = branch.revision_history()
-        if rh:
-            return len(rh), rh[-1]
-        else:
-            return None
-    except bzrlib.errors.BzrError:
-        return None
-    
 def test_suite():
     import tests
     return tests.test_suite()

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2006-08-09 19:53:02 +0000
+++ bzrlib/add.py	2006-08-16 15:31:34 +0000
@@ -144,7 +144,7 @@
     """
     file_list = _prepare_file_list(file_list)
     tree = WorkingTree.open_containing(file_list[0])[0]
-    return smart_add_tree(tree, file_list, recurse, action=action)
+    return smart_add_tree(tree, file_list, recurse, action=action, save=save)
 
 
 class FastPath(object):

=== modified file 'bzrlib/benchmarks/__init__.py'
--- bzrlib/benchmarks/__init__.py	2006-08-15 13:20:18 +0000
+++ bzrlib/benchmarks/__init__.py	2006-08-18 15:21:29 +0000
@@ -17,8 +17,10 @@
 """Benchmark test suite for bzr."""
 
 from bzrlib import (
+    bzrdir,
     plugin,
     )
+import bzrlib.branch
 from bzrlib.tests.TestUtil import TestLoader
 from bzrlib.tests.blackbox import ExternalBase
 
@@ -84,6 +86,18 @@
                                                  hot_cache=hot_cache)
         return creator.create(root=root)
 
+    def make_kernel_like_inventory(self):
+        """Create an inventory with the properties of a kernel-like tree
+
+        This should be equivalent to a committed kernel like tree, not
+        just a working tree.
+        """
+        from bzrlib.benchmarks.tree_creator.kernel_like import (
+            KernelLikeInventoryCreator,
+            )
+        creator = KernelLikeInventoryCreator(self)
+        return creator.create()
+
     def make_many_commit_tree(self, directory_name='.',
                               hardlink=False):
         """Create a tree with many commits.
@@ -113,6 +127,60 @@
         creator = HeavilyMergedTreeCreator(self, link_bzr=hardlink)
         return creator.create(root=directory_name)
 
+    def create_with_commits(self, num_files, num_commits, directory_name='.'):
+        """Create a tree with many files and many commits. Every commit changes
+        exactly one file.
+        
+        :param num_files: number of files to be created
+        :param num_commits: number of commits in the newly created tree
+        """
+        files = ["%s/%s" % (directory_name, i) for i in range(num_files)]
+        for fn in files:
+            f = open(fn, "wb")
+            try:
+                f.write("some content\n")
+            finally:
+                f.close()
+        tree = bzrdir.BzrDir.create_standalone_workingtree(directory_name)
+        tree.add([str(i) for i in range(num_files)])
+        tree.lock_write()
+        try:
+            tree.commit('initial commit')
+            for i in range(num_commits):
+                fn = files[i % len(files)]
+                content = range(i) + [i, i, i, ""]
+                f = open(fn, "wb")
+                try:
+                    f.write("\n".join([str(i) for i in content]))
+                finally:
+                    f.close()
+                tree.commit("changing file %s" % fn)
+        finally:
+            tree.unlock()
+        return tree, files
+
+    def commit_some_revisions(self, tree, files, num_commits,
+                              changes_per_commit):
+        """Commit a specified number of revisions to some files in a tree,
+        makeing a specified number of changes per commit.
+
+        :param tree: The tree in which the changes happen.
+        :param files: The list of files where changes should occur.
+        :param num_commits: The number of commits
+        :param changes_per_commit: The number of files that are touched in 
+        each commit.
+        """
+        for j in range(num_commits):
+            for i in range(changes_per_commit):
+                fn = files[(i + j) % changes_per_commit]
+                content = range(i) + [i, i, i, '']
+                f = open(fn, "w")
+                try:
+                    f.write("\n".join([str(k) for k in content]))
+                finally:
+                    f.close()
+            tree.commit("new revision")
+
 
 def test_suite():
     """Build and return a TestSuite which contains benchmark tests only."""
@@ -129,6 +197,8 @@
                    'bzrlib.benchmarks.bench_status',
                    'bzrlib.benchmarks.bench_transform',
                    'bzrlib.benchmarks.bench_workingtree',
+                   'bzrlib.benchmarks.bench_sftp',
+                   'bzrlib.benchmarks.bench_xml',
                    ]
     suite = TestLoader().loadTestsFromModuleNames(testmod_names) 
 

=== modified file 'bzrlib/benchmarks/bench_commit.py'
--- bzrlib/benchmarks/bench_commit.py	2006-08-15 03:19:49 +0000
+++ bzrlib/benchmarks/bench_commit.py	2006-08-17 13:21:28 +0000
@@ -15,6 +15,7 @@
 
 """Benchmarks of bzr commit."""
 
+import os
 
 from bzrlib.benchmarks import Benchmark
 from bzrlib.transport.memory import MemoryServer
@@ -30,11 +31,49 @@
         # self.transport_server = MemoryServer
         # self.make_kernel_like_tree(self.get_url())
         tree = self.make_kernel_like_added_tree()
-        # on robertc's machine the first sample of this took 59750ms/77682ms
-        # after initial call reduction                       53922ms/73918ms
         self.time(self.run_bzr, 'commit', '-m', 'first post')
 
     def test_partial_commit_kernel_like_tree(self):
         """Commit of 1/8th of a fresh import of a clean kernel sized tree."""
         tree = self.make_kernel_like_added_tree()
         self.time(self.run_bzr, 'commit', '-m', 'first post', '1')
+
+    def test_no_op_commit_in_kernel_like_tree(self):
+        """Run commit --unchanged in a kernel sized tree"""
+        tree = self.make_kernel_like_committed_tree()
+        self.time(self.run_bzr, 'commit', '-m', 'no changes', '--unchanged')
+
+    def test_commit_one_in_kernel_like_tree(self):
+        """Time committing a single change, when not directly specified"""
+        tree = self.make_kernel_like_committed_tree()
+
+        # working-tree is hardlinked, so replace a file and commit the change
+        os.remove('4/4/4/4')
+        open('4/4/4/4', 'wb').write('new contents\n')
+        self.time(self.run_bzr, 'commit', '-m', 'second')
+
+    def test_partial_commit_one_in_kernel_like_tree(self):
+        """Time committing a single change when it is directly specified"""
+        tree = self.make_kernel_like_committed_tree()
+
+        # working-tree is hardlinked, so replace a file and commit the change
+        os.remove('4/4/4/4')
+        open('4/4/4/4', 'wb').write('new contents\n')
+        self.time(self.run_bzr, 'commit', '-m', 'second', '4/4/4/4')
+
+    def make_simple_tree(self):
+        """A small, simple tree. No caching needed"""
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a', 'b/', 'b/c'])
+        tree.add(['a', 'b', 'b/c'])
+        return tree
+
+    def test_cmd_commit(self):
+        """Test execution of simple commit"""
+        tree = self.make_simple_tree()
+        self.time(self.run_bzr, 'commit', '-m', 'init simple tree')
+
+    def test_cmd_commit_subprocess(self):
+        """Text startup and execution of a simple commit.""" 
+        tree = self.make_simple_tree()
+        self.time(self.run_bzr_subprocess, 'commit', '-m', 'init simple tree')

=== added file 'bzrlib/benchmarks/bench_sftp.py'
--- bzrlib/benchmarks/bench_sftp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/benchmarks/bench_sftp.py	2006-08-18 16:48:53 +0000
@@ -0,0 +1,98 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as published by
+# the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for bzr performance over SFTP"""
+
+import os
+
+from bzrlib import (
+    bzrdir,
+    )
+from bzrlib.benchmarks import Benchmark
+from bzrlib.tests import test_sftp_transport
+
+try:
+    import paramiko
+    paramiko_loaded = True
+except ImportError:
+    paramiko_loaded = False
+
+
+class SFTPBenchmark(Benchmark):
+    """Benchmark branch, push and pull across a local sftp connection."""
+
+    def setUp(self):
+        super(SFTPBenchmark, self).setUp()
+        if not paramiko_loaded:
+            raise TestSkipped('you must have paramiko to run this test')
+        test_sftp_transport.set_test_transport_to_sftp(self)
+         
+    def test_branch(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(100, 100, "a")
+        self.time(bzrdir.BzrDir.open(self.get_url('a')).sprout, "b")
+
+    def create_commit_and_pull(self, num_pull_revisions):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(100, 100, "a")
+        rbzrdir = bzrdir.BzrDir.open(self.get_url('a'))
+        b2 = tree.bzrdir.sprout("b") # branch
+        # change a few files and commit
+        self.commit_some_revisions(tree, files, num_pull_revisions, 20)
+        self.time(b2.open_branch().pull, rbzrdir.open_branch())
+
+    def test_pull_1(self):
+        self.create_commit_and_pull(1)
+        
+    def test_pull_10(self):
+        self.create_commit_and_pull(10)
+
+    def test_pull_100(self):
+        self.create_commit_and_pull(100)
+
+    def create_commit_and_push(self, num_push_revisions):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(100, 100, "a")
+        rbzrdir = bzrdir.BzrDir.open(self.get_url('a'))
+        b2 = tree.bzrdir.sprout("b") # branch
+        wtree = b2.open_workingtree()
+        # change a few files and commit
+        self.commit_some_revisions(
+            wtree, ["b/%i" for i in range(100)],
+            num_commits=num_push_revisions,
+            changes_per_commit=20)
+        self.time(rbzrdir.open_branch().pull, wtree.branch)
+
+    def test_initial_push(self):
+        os.mkdir('a')
+        tree, files = self.create_with_commits(100, 100, "a")
+        self.time(tree.bzrdir.clone, self.get_url('b'),
+                  revision_id=tree.last_revision())
+
+    def test_push_1(self):
+        self.create_commit_and_push(1)
+
+    def test_push_10(self):
+        self.create_commit_and_push(10)
+
+    def test_push_100(self):
+        self.create_commit_and_push(100)
+
+
+class SFTPSlowSocketBenchmark(SFTPBenchmark):
+    def setUp(self):
+        super(SFTPSlowSocketBenchmark, self).setUp()
+        self.get_server().add_latency = 0.03
+

=== added file 'bzrlib/benchmarks/bench_xml.py'
--- bzrlib/benchmarks/bench_xml.py	1970-01-01 00:00:00 +0000
+++ bzrlib/benchmarks/bench_xml.py	2006-08-16 23:38:03 +0000
@@ -0,0 +1,85 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as published by
+# the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for bzr xml serialization performance."""
+
+from bzrlib import (
+    cache_utf8,
+    xml5,
+    )
+from bzrlib.benchmarks import Benchmark
+
+
+class BenchXMLSerializer(Benchmark):
+
+    def test_write_to_string_kernel_like_inventory(self):
+        # On jam's machine, ElementTree serializer took: 2161ms/13487ms
+        #                      with Robert's serializer:  631ms/10770ms
+        #                      with Entity escaper:       487ms/11636ms
+        #           caching Entity escaper, empty cache:  448ms/ 9489ms
+        #           caching Entity escaper, full cache:   375ms/ 9489ms
+        #                      passing around function:   406ms/ 8942ms
+        #              cached, passing around function:   328ms/11248ms
+        #                      removing extra function:   354ms/ 8942ms
+        #              cached, removing extra function:   275ms/11248ms
+        #                          no cache, real utf8:   363ms/11697ms
+        #                            cached, real utf8:   272ms/12827ms
+        # Really all we want is a real inventory
+        inv = self.make_kernel_like_inventory()
+
+        xml5._clear_cache()
+        # We want a real tree with lots of file ids and sha strings, etc.
+        self.time(xml5.serializer_v5.write_inventory_to_string, inv)
+
+    def test_write_kernel_like_inventory(self):
+        # Really all we want is a real inventory
+        inv = self.make_kernel_like_inventory()
+
+        xml5._clear_cache()
+        f = open('kernel-like-inventory', 'wb')
+        try:
+            # We want a real tree with lots of file ids and sha strings, etc.
+            self.time(xml5.serializer_v5.write_inventory, inv, f)
+        finally:
+            f.close()
+
+    def test_write_to_string_cached_kernel_like_inventory(self):
+        inv = self.make_kernel_like_inventory()
+
+        xml5._clear_cache()
+        # We want a real tree with lots of file ids and sha strings, etc.
+        xml5.serializer_v5.write_inventory_to_string(inv)
+
+        self.time(xml5.serializer_v5.write_inventory_to_string, inv)
+
+    def test_read_from_string_kernel_like_inventory(self):
+        inv = self.make_kernel_like_inventory()
+        as_str = xml5.serializer_v5.write_inventory_to_string(inv)
+
+        cache_utf8.clear_encoding_cache()
+        read_inv = self.time(xml5.serializer_v5.read_inventory_from_string,
+                             as_str)
+        # TODO: make sure the final inventory is equal as a sanity check
+
+    def test_read_from_string_cached_kernel_like_inventory(self):
+        cache_utf8.clear_encoding_cache()
+        inv = self.make_kernel_like_inventory()
+        as_str = xml5.serializer_v5.write_inventory_to_string(inv)
+
+        xml5.serializer_v5.read_inventory_from_string(as_str)
+
+        read_inv = self.time(xml5.serializer_v5.read_inventory_from_string,
+                             as_str)
+        # TODO: make sure the final inventory is equal as a sanity check

=== modified file 'bzrlib/benchmarks/tree_creator/kernel_like.py'
--- bzrlib/benchmarks/tree_creator/kernel_like.py	2006-08-15 03:19:49 +0000
+++ bzrlib/benchmarks/tree_creator/kernel_like.py	2006-08-17 23:09:00 +0000
@@ -16,12 +16,15 @@
 
 """Tree creators for kernel-like trees"""
 
+import errno
 import os
 
 from bzrlib import (
     add,
     bzrdir,
     osutils,
+    workingtree,
+    xml5,
     )
 
 from bzrlib.benchmarks.tree_creator import TreeCreator
@@ -81,6 +84,11 @@
                     files.append(prefix)
                     files.extend([prefix + str(foo) for foo in range(20)])
         cwd = osutils.getcwd()
+        try:
+            os.mkdir(root)
+        except OSError, e:
+            if e.errno not in (errno.EEXIST,):
+                raise
         os.chdir(root)
         self._test.build_tree(files)
         os.chdir(cwd)
@@ -144,3 +152,66 @@
         if in_cache:
             self._protect_files(root+'/.bzr')
         return tree
+
+
+class KernelLikeInventoryCreator(TreeCreator):
+    """Return just the memory representation of a committed kernel-like tree"""
+
+    def __init__(self, test):
+        super(KernelLikeInventoryCreator, self).__init__(test,
+            tree_name='kernel_like_inventory',
+            link_working=True,
+            link_bzr=True,
+            hot_cache=True)
+
+    def ensure_cached(self):
+        """Make sure we have a cached version of the kernel-like inventory"""
+        cache_dir = self._get_cache_dir()
+        if cache_dir is None:
+            return
+
+        if self.is_cached():
+            return
+
+        committed_creator = KernelLikeCommittedTreeCreator(self._test,
+                                                           link_working=True,
+                                                           link_bzr=True,
+                                                           hot_cache=False)
+        committed_creator.ensure_cached()
+        committed_cache_dir = committed_creator._get_cache_dir()
+        committed_tree = workingtree.WorkingTree.open(committed_cache_dir)
+        rev_tree = committed_tree.basis_tree()
+        os.mkdir(cache_dir)
+        f = open(cache_dir+'/inventory', 'wb')
+        try:
+            xml5.serializer_v5.write_inventory(rev_tree.inventory, f)
+        finally:
+            f.close()
+
+    def create(self):
+        """Create a kernel like inventory
+
+        :return: An Inventory object.
+        """
+        cache_dir = self._get_cache_dir()
+        if cache_dir is None:
+            return self._create_and_return()
+
+        self.ensure_cached()
+        return self._open_cached(cache_dir)
+
+    def _create_and_return(self):
+        """Create a kernel-like tree, and return its inventory"""
+        creator = KernelLikeCommittedTreeCreator(self._test,
+                                                 link_working=True,
+                                                 link_bzr=True,
+                                                 hot_cache=False)
+        tree = creator.create('.')
+        return tree.basis_tree().inventory
+
+    def _open_cached(self, cache_dir):
+        f = open(cache_dir + '/inventory', 'rb')
+        try:
+            return xml5.serializer_v5.read_inventory(f)
+        finally:
+            f.close()

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2006-08-15 14:06:37 +0000
+++ bzrlib/branch.py	2006-08-17 07:52:09 +0000
@@ -158,23 +158,23 @@
     nick = property(_get_nick, _set_nick)
 
     def is_locked(self):
-        raise NotImplementedError('is_locked is abstract')
+        raise NotImplementedError(self.is_locked)
 
     def lock_write(self):
-        raise NotImplementedError('lock_write is abstract')
+        raise NotImplementedError(self.lock_write)
 
     def lock_read(self):
-        raise NotImplementedError('lock_read is abstract')
+        raise NotImplementedError(self.lock_read)
 
     def unlock(self):
-        raise NotImplementedError('unlock is abstract')
+        raise NotImplementedError(self.unlock)
 
     def peek_lock_mode(self):
         """Return lock mode for the Branch: 'r', 'w' or None"""
         raise NotImplementedError(self.peek_lock_mode)
 
     def get_physical_lock_status(self):
-        raise NotImplementedError('get_physical_lock_status is abstract')
+        raise NotImplementedError(self.get_physical_lock_status)
 
     def abspath(self, name):
         """Return absolute filename for something in the branch
@@ -182,7 +182,7 @@
         XXX: Robert Collins 20051017 what is this used for? why is it a branch
         method and not a tree method.
         """
-        raise NotImplementedError('abspath is abstract')
+        raise NotImplementedError(self.abspath)
 
     def bind(self, other):
         """Bind the local branch the other branch.
@@ -279,17 +279,17 @@
 
     def get_root_id(self):
         """Return the id of this branches root"""
-        raise NotImplementedError('get_root_id is abstract')
+        raise NotImplementedError(self.get_root_id)
 
     def print_file(self, file, revision_id):
         """Print `file` to stdout."""
-        raise NotImplementedError('print_file is abstract')
+        raise NotImplementedError(self.print_file)
 
     def append_revision(self, *revision_ids):
-        raise NotImplementedError('append_revision is abstract')
+        raise NotImplementedError(self.append_revision)
 
     def set_revision_history(self, rev_history):
-        raise NotImplementedError('set_revision_history is abstract')
+        raise NotImplementedError(self.set_revision_history)
 
     def revision_history(self):
         """Return sequence of revision hashes on to this branch."""
@@ -308,7 +308,7 @@
         raise errors.UpgradeRequired(self.base)
 
     def last_revision(self):
-        """Return last patch hash, or None if no history."""
+        """Return last revision id, or None"""
         ph = self.revision_history()
         if ph:
             return ph[-1]
@@ -345,7 +345,7 @@
         :param stop_revision: Updated until the given revision
         :return: None
         """
-        raise NotImplementedError('update_revisions is abstract')
+        raise NotImplementedError(self.update_revisions)
 
     def revision_id_to_revno(self, revision_id):
         """Given a revision id, return its revno"""
@@ -368,7 +368,7 @@
         return history[revno - 1]
 
     def pull(self, source, overwrite=False, stop_revision=None):
-        raise NotImplementedError('pull is abstract')
+        raise NotImplementedError(self.pull)
 
     def basis_tree(self):
         """Return `Tree` object for last revision."""
@@ -379,7 +379,7 @@
 
         This can change the directory or the filename or both.
         """
-        raise NotImplementedError('rename_one is abstract')
+        raise NotImplementedError(self.rename_one)
 
     def move(self, from_paths, to_name):
         """Rename files.
@@ -395,7 +395,7 @@
         This returns a list of (from_path, to_path) pairs for each
         entry that is moved.
         """
-        raise NotImplementedError('move is abstract')
+        raise NotImplementedError(self.move)
 
     def get_parent(self):
         """Return the parent location of the branch.
@@ -404,7 +404,7 @@
         pattern is that the user can override it by specifying a
         location.
         """
-        raise NotImplementedError('get_parent is abstract')
+        raise NotImplementedError(self.get_parent)
 
     def get_submit_branch(self):
         """Return the submit location of the branch.
@@ -426,14 +426,14 @@
 
     def get_push_location(self):
         """Return the None or the location to push this branch to."""
-        raise NotImplementedError('get_push_location is abstract')
+        raise NotImplementedError(self.get_push_location)
 
     def set_push_location(self, location):
         """Set a new push location for this branch."""
-        raise NotImplementedError('set_push_location is abstract')
+        raise NotImplementedError(self.set_push_location)
 
     def set_parent(self, url):
-        raise NotImplementedError('set_parent is abstract')
+        raise NotImplementedError(self.set_parent)
 
     @needs_write_lock
     def update(self):

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-08-15 15:57:56 +0000
+++ bzrlib/builtins.py	2006-08-16 16:33:11 +0000
@@ -1985,17 +1985,24 @@
                 test_suite_factory = benchmarks.test_suite
                 if verbose is None:
                     verbose = True
+                benchfile = open(".perf_history", "at")
             else:
                 test_suite_factory = None
                 if verbose is None:
                     verbose = False
-            result = selftest(verbose=verbose, 
-                              pattern=pattern,
-                              stop_on_failure=one, 
-                              keep_output=keep_output,
-                              transport=transport,
-                              test_suite_factory=test_suite_factory,
-                              lsprof_timed=lsprof_timed)
+                benchfile = None
+            try:
+                result = selftest(verbose=verbose, 
+                                  pattern=pattern,
+                                  stop_on_failure=one, 
+                                  keep_output=keep_output,
+                                  transport=transport,
+                                  test_suite_factory=test_suite_factory,
+                                  lsprof_timed=lsprof_timed,
+                                  bench_history=benchfile)
+            finally:
+                if benchfile is not None:
+                    benchfile.close()
             if result:
                 info('tests passed')
             else:
@@ -2005,54 +2012,12 @@
             ui.ui_factory = save_ui
 
 
-def _get_bzr_branch():
-    """If bzr is run from a branch, return Branch or None"""
-    from os.path import dirname
-    
-    try:
-        branch = Branch.open(dirname(osutils.abspath(dirname(__file__))))
-        return branch
-    except errors.BzrError:
-        return None
-    
-
-def show_version():
-    import bzrlib
-    print "Bazaar (bzr) %s" % bzrlib.__version__
-    # is bzrlib itself in a branch?
-    branch = _get_bzr_branch()
-    if branch:
-        rh = branch.revision_history()
-        revno = len(rh)
-        print "  bzr checkout, revision %d" % (revno,)
-        print "  nick: %s" % (branch.nick,)
-        if rh:
-            print "  revid: %s" % (rh[-1],)
-    print "Using python interpreter:", sys.executable
-    import site
-    print "Using python standard library:", os.path.dirname(site.__file__)
-    print "Using bzrlib:",
-    if len(bzrlib.__path__) > 1:
-        # print repr, which is a good enough way of making it clear it's
-        # more than one element (eg ['/foo/bar', '/foo/bzr'])
-        print repr(bzrlib.__path__)
-    else:
-        print bzrlib.__path__[0]
-
-    print
-    print bzrlib.__copyright__
-    print "http://bazaar-vcs.org/"
-    print
-    print "bzr comes with ABSOLUTELY NO WARRANTY.  bzr is free software, and"
-    print "you may use, modify and redistribute it under the terms of the GNU"
-    print "General Public License version 2 or later."
-
-
 class cmd_version(Command):
     """Show version of bzr."""
 
     @display_command
     def run(self):
+        from bzrlib.version import show_version
         show_version()
 
 
@@ -2528,7 +2493,8 @@
 
 class cmd_testament(Command):
     """Show testament (signing-form) of a revision."""
-    takes_options = ['revision', 'long', 
+    takes_options = ['revision', 
+                     Option('long', help='Produce long-format testament'), 
                      Option('strict', help='Produce a strict-format'
                             ' testament')]
     takes_args = ['branch?']

=== modified file 'bzrlib/bundle/commands.py'
--- bzrlib/bundle/commands.py	2006-06-29 16:24:56 +0000
+++ bzrlib/bundle/commands.py	2006-08-15 20:19:08 +0000
@@ -142,8 +142,12 @@
             fileobj = file(output, 'wb')
         else:
             fileobj = sys.stdout
-        write_bundle(target_branch.repository, target_revision, base_revision,
-                     fileobj)
+        target_branch.repository.lock_read()
+        try:
+            write_bundle(target_branch.repository, target_revision,
+                         base_revision, fileobj)
+        finally:
+            target_branch.repository.unlock()
 
 
 class cmd_verify_changeset(Command):

=== modified file 'bzrlib/bundle/serializer/__init__.py'
--- bzrlib/bundle/serializer/__init__.py	2006-08-06 23:06:44 +0000
+++ bzrlib/bundle/serializer/__init__.py	2006-08-15 20:19:08 +0000
@@ -91,11 +91,23 @@
         raise errors.BundleNotSupported(version, 'unknown bundle format')
 
     serializer = _serializers[version](version)
-    return serializer.write(source, revision_ids, forced_bases, f) 
+    source.lock_read()
+    try:
+        return serializer.write(source, revision_ids, forced_bases, f)
+    finally:
+        source.unlock()
 
 
 def write_bundle(repository, revision_id, base_revision_id, out):
     """"""
+    repository.lock_read()
+    try:
+        return _write_bundle(repository, revision_id, base_revision_id, out)
+    finally:
+        repository.unlock()
+
+
+def _write_bundle(repository, revision_id, base_revision_id, out):
     if base_revision_id is NULL_REVISION:
         base_revision_id = None
     base_ancestry = set(repository.get_ancestry(base_revision_id))

=== modified file 'bzrlib/bundle/serializer/v08.py'
--- bzrlib/bundle/serializer/v08.py	2006-08-08 18:14:25 +0000
+++ bzrlib/bundle/serializer/v08.py	2006-08-15 20:43:59 +0000
@@ -158,8 +158,7 @@
             if rev_id == last_rev_id:
                 rev_tree = last_rev_tree
             else:
-                base_tree = self.source.revision_tree(rev_id)
-            rev_tree = self.source.revision_tree(rev_id)
+                rev_tree = self.source.revision_tree(rev_id)
             if rev_id in self.forced_bases:
                 explicit_base = True
                 base_id = self.forced_bases[rev_id]

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-08-17 14:16:46 +0000
+++ bzrlib/bzrdir.py	2006-08-22 10:50:58 +0000
@@ -615,7 +615,8 @@
                 # XXX FIXME RBC 20060214 need tests for this when the basis
                 # is incomplete
                 result_repo.fetch(basis_repo, revision_id=revision_id)
-            result_repo.fetch(source_repository, revision_id=revision_id)
+            if source_repository is not None:
+                result_repo.fetch(source_repository, revision_id=revision_id)
         if source_branch is not None:
             source_branch.sprout(result, revision_id=revision_id)
         else:
@@ -733,7 +734,7 @@
         self._check_supported(format, unsupported)
         return format.open(self, _found=True)
 
-    def sprout(self, url, revision_id=None, basis=None):
+    def sprout(self, url, revision_id=None, basis=None, force_new_repo=False):
         """See BzrDir.sprout()."""
         from bzrlib.workingtree import WorkingTreeFormat2
         self._make_tail(url)
@@ -1217,8 +1218,13 @@
         result = (super(BzrDirFormat5, self).initialize_on_transport(transport))
         RepositoryFormat5().initialize(result, _internal=True)
         if not _cloning:
-            BzrBranchFormat4().initialize(result)
-            WorkingTreeFormat2().initialize(result)
+            branch = BzrBranchFormat4().initialize(result)
+            try:
+                WorkingTreeFormat2().initialize(result)
+            except errors.NotLocalUrl:
+                # Even though we can't access the working tree, we need to
+                # create its control files.
+                WorkingTreeFormat2().stub_initialize_remote(branch.control_files)
         return result
 
     def _open(self, transport):
@@ -1271,13 +1277,13 @@
         result = super(BzrDirFormat6, self).initialize_on_transport(transport)
         RepositoryFormat6().initialize(result, _internal=True)
         if not _cloning:
-            BzrBranchFormat4().initialize(result)
+            branch = BzrBranchFormat4().initialize(result)
             try:
                 WorkingTreeFormat2().initialize(result)
             except errors.NotLocalUrl:
-                # emulate pre-check behaviour for working tree and silently 
-                # fail.
-                pass
+                # Even though we can't access the working tree, we need to
+                # create its control files.
+                WorkingTreeFormat2().stub_initialize_remote(branch.control_files)
         return result
 
     def _open(self, transport):

=== modified file 'bzrlib/cache_utf8.py'
--- bzrlib/cache_utf8.py	2006-08-10 00:57:21 +0000
+++ bzrlib/cache_utf8.py	2006-08-16 22:05:26 +0000
@@ -31,6 +31,14 @@
            _uni_to_utf8=_unicode_to_utf8_map,
            _utf8_to_uni=_utf8_to_unicode_map):
     """Take this unicode revision id, and get a unicode version"""
+    # If the key is in the cache try/KeyError is 50% faster than
+    # val = dict.get(key), if val is None:
+    # On jam's machine the difference is 
+    # try/KeyError:  900ms 
+    #      if None: 1250ms 
+    # Since these are primarily used when iterating over a knit entry
+    # *most* of the time the key will already be in the cache, so use the
+    # fast path
     try:
         return _uni_to_utf8[unicode_str]
     except KeyError:

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2006-07-15 14:29:03 +0000
+++ bzrlib/commands.py	2006-08-16 16:33:11 +0000
@@ -28,11 +28,11 @@
 # TODO: "--profile=cum", to change sort order.  Is there any value in leaving
 # the profile output behind so it can be interactively examined?
 
-import sys
+import codecs
+import errno
 import os
 from warnings import warn
-import errno
-import codecs
+import sys
 
 import bzrlib
 import bzrlib.errors as errors
@@ -40,6 +40,7 @@
                            BzrCommandError,
                            BzrCheckError,
                            NotBranchError)
+from bzrlib import option
 from bzrlib.option import Option
 import bzrlib.osutils
 from bzrlib.revisionspec import RevisionSpec
@@ -221,7 +222,7 @@
         r = dict()
         r['help'] = Option.OPTIONS['help']
         for o in self.takes_options:
-            if not isinstance(o, Option):
+            if isinstance(o, basestring):
                 o = Option.OPTIONS[o]
             r[o.name] = o
         return r
@@ -256,17 +257,15 @@
 
     def run_argv_aliases(self, argv, alias_argv=None):
         """Parse the command line and run with extra aliases in alias_argv."""
+        if argv is None:
+            warn("Passing None for [] is deprecated from bzrlib 0.10", 
+                 DeprecationWarning, stacklevel=2)
+            argv = []
         args, opts = parse_args(self, argv, alias_argv)
         if 'help' in opts:  # e.g. bzr add --help
             from bzrlib.help import help_on_command
             help_on_command(self.name())
             return 0
-        # XXX: This should be handled by the parser
-        allowed_names = self.options().keys()
-        for oname in opts:
-            if oname not in allowed_names:
-                raise BzrOptionError("option '--%s' is not allowed for"
-                                " command %r" % (oname, self.name()))
         # mix arguments and options into one dictionary
         cmdargs = _match_argform(self.name(), self.takes_args, args)
         cmdopts = {}
@@ -354,106 +353,16 @@
     lookup table, something about the available options, what optargs
     they take, and which commands will accept them.
     """
-    # TODO: chop up this beast; make it a method of the Command
-    args = []
-    opts = {}
-    alias_opts = {}
-
-    cmd_options = command.options()
-    argsover = False
-    proc_aliasarg = True # Are we processing alias_argv now?
-    for proc_argv in alias_argv, argv:
-        while proc_argv:
-            a = proc_argv.pop(0)
-            if argsover:
-                args.append(a)
-                continue
-            elif a == '-':
-                args.append(a)
-                continue
-            elif a == '--':
-                # We've received a standalone -- No more flags
-                argsover = True
-                continue
-            if a[0] == '-':
-                # option names must not be unicode
-                a = str(a)
-                optarg = None
-                if a[1] == '-':
-                    mutter("  got option %r", a)
-                    if '=' in a:
-                        optname, optarg = a[2:].split('=', 1)
-                    else:
-                        optname = a[2:]
-                    if optname not in cmd_options:
-                        raise BzrCommandError('unknown option "%s"' % a)
-                else:
-                    shortopt = a[1:]
-                    if shortopt in Option.SHORT_OPTIONS:
-                        # Multi-character options must have a space to delimit
-                        # their value
-                        # ^^^ what does this mean? mbp 20051014
-                        optname = Option.SHORT_OPTIONS[shortopt].name
-                    else:
-                        # Single character short options, can be chained,
-                        # and have their value appended to their name
-                        shortopt = a[1:2]
-                        if shortopt not in Option.SHORT_OPTIONS:
-                            # We didn't find the multi-character name, and we
-                            # didn't find the single char name
-                            raise BzrCommandError('unknown option "%s"' % a)
-                        optname = Option.SHORT_OPTIONS[shortopt].name
-
-                        if a[2:]:
-                            # There are extra things on this option
-                            # see if it is the value, or if it is another
-                            # short option
-                            optargfn = Option.OPTIONS[optname].type
-                            if optargfn is None:
-                                # This option does not take an argument, so the
-                                # next entry is another short option, pack it
-                                # back into the list
-                                proc_argv.insert(0, '-' + a[2:])
-                            else:
-                                # This option takes an argument, so pack it
-                                # into the array
-                                optarg = a[2:]
-                    if optname not in cmd_options:
-                        raise BzrCommandError('unknown option "%s"' % shortopt)
-                if optname in opts:
-                    # XXX: Do we ever want to support this, e.g. for -r?
-                    if proc_aliasarg:
-                        raise BzrCommandError('repeated option %r' % a)
-                    elif optname in alias_opts:
-                        # Replace what's in the alias with what's in the real
-                        # argument
-                        del alias_opts[optname]
-                        del opts[optname]
-                        proc_argv.insert(0, a)
-                        continue
-                    else:
-                        raise BzrCommandError('repeated option %r' % a)
-                    
-                option_obj = cmd_options[optname]
-                optargfn = option_obj.type
-                if optargfn:
-                    if optarg == None:
-                        if not proc_argv:
-                            raise BzrCommandError('option %r needs an argument' % a)
-                        else:
-                            optarg = proc_argv.pop(0)
-                    opts[optname] = optargfn(optarg)
-                    if proc_aliasarg:
-                        alias_opts[optname] = optargfn(optarg)
-                else:
-                    if optarg != None:
-                        raise BzrCommandError('option %r takes no argument' % optname)
-                    opts[optname] = True
-                    if proc_aliasarg:
-                        alias_opts[optname] = True
-            else:
-                args.append(a)
-        proc_aliasarg = False # Done with alias argv
+    # TODO: make it a method of the Command?
+    parser = option.get_optparser(command.options())
+    if alias_argv is not None:
+        args = alias_argv + argv
+    else:
+        args = argv
+
+    options, args = parser.parse_args(args)
+    opts = dict([(k, v) for k, v in options.__dict__.iteritems() if 
+                 v is not option.OptionParser.DEFAULT_VALUE])
     return args, opts
 
 
@@ -621,7 +530,7 @@
         return 0
 
     if argv[0] == '--version':
-        from bzrlib.builtins import show_version
+        from bzrlib.version import show_version
         show_version()
         return 0
         

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2006-08-17 14:16:46 +0000
+++ bzrlib/commit.py	2006-08-22 10:50:58 +0000
@@ -66,7 +66,6 @@
 import re
 import sys
 import time
-import warnings
 
 from cStringIO import StringIO
 
@@ -83,25 +82,13 @@
 from bzrlib.trace import mutter, note, warning
 from bzrlib.xml5 import serializer_v5
 from bzrlib.inventory import Inventory, ROOT_ID, InventoryEntry
+from bzrlib import symbol_versioning
 from bzrlib.symbol_versioning import (deprecated_passed,
         deprecated_function,
-        zero_seven,
         DEPRECATED_PARAMETER)
 from bzrlib.workingtree import WorkingTree
 
 
-@deprecated_function(zero_seven)
-def commit(*args, **kwargs):
-    """Commit a new revision to a branch.
-
-    Function-style interface for convenience of old callers.
-
-    New code should use the Commit class instead.
-    """
-    ## XXX: Remove this in favor of WorkingTree.commit?
-    Commit().commit(*args, **kwargs)
-
-
 class NullCommitReporter(object):
     """I report on progress of a commit."""
 
@@ -221,7 +208,7 @@
         mutter('preparing to commit')
 
         if deprecated_passed(branch):
-            warnings.warn("Commit.commit (branch, ...): The branch parameter is "
+            symbol_versioning.warn("Commit.commit (branch, ...): The branch parameter is "
                  "deprecated as of bzr 0.8. Please use working_tree= instead.",
                  DeprecationWarning, stacklevel=2)
             self.branch = branch
@@ -504,8 +491,9 @@
         mutter("Selecting files for commit with filter %s", self.specific_files)
         entries = self.work_inv.iter_entries()
         if not self.builder.record_root_entry:
-            warnings.warn('CommitBuilders should support recording the root'
-                ' entry as of bzr 0.10.', DeprecationWarning, stacklevel=1)
+            symbol_versioning.warn('CommitBuilders should support recording'
+                ' the root entry as of bzr 0.10.', DeprecationWarning, 
+                stacklevel=1)
             self.builder.new_inventory.add(self.basis_inv.root.copy())
             entries.next()
             self._emit_progress_update()

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2006-07-31 19:15:36 +0000
+++ bzrlib/diff.py	2006-08-14 22:38:25 +0000
@@ -88,6 +88,41 @@
     print >>to_file
 
 
+def _set_lang_C():
+    """Set the env var LANG=C"""
+    os.environ['LANG'] = 'C'
+
+
+def _spawn_external_diff(diffcmd, capture_errors=True):
+    """Spawn the externall diff process, and return the child handle.
+
+    :param diffcmd: The command list to spawn
+    :param capture_errors: Capture stderr as well as setting LANG=C.
+        This lets us read and understand the output of diff, and respond 
+        to any errors.
+    :return: A Popen object.
+    """
+    if capture_errors:
+        preexec_fn = _set_lang_C
+        stderr = subprocess.PIPE
+    else:
+        preexec_fn = None
+        stderr = None
+
+    try:
+        pipe = subprocess.Popen(diffcmd,
+                                stdin=subprocess.PIPE,
+                                stdout=subprocess.PIPE,
+                                stderr=stderr,
+                                preexec_fn=preexec_fn)
+    except OSError, e:
+        if e.errno == errno.ENOENT:
+            raise errors.NoDiff(str(e))
+        raise
+
+    return pipe
+
+
 def external_diff(old_filename, oldlines, new_filename, newlines, to_file,
                   diff_opts):
     """Display a diff by calling out to the external diff program."""
@@ -147,31 +182,43 @@
         if diff_opts:
             diffcmd.extend(diff_opts)
 
-        try:
-            pipe = subprocess.Popen(diffcmd,
-                                    stdin=subprocess.PIPE,
-                                    stdout=subprocess.PIPE)
-        except OSError, e:
-            if e.errno == errno.ENOENT:
-                raise errors.NoDiff(str(e))
-            raise
-        pipe.stdin.close()
-
-        first_line = pipe.stdout.readline()
-        to_file.write(first_line)
-        bzrlib.osutils.pumpfile(pipe.stdout, to_file)
-        rc = pipe.wait()
+        pipe = _spawn_external_diff(diffcmd, capture_errors=True)
+        out,err = pipe.communicate()
+        rc = pipe.returncode
         
+        # internal_diff() adds a trailing newline, add one here for consistency
+        out += '\n'
         if rc == 2:
             # 'diff' gives retcode == 2 for all sorts of errors
             # one of those is 'Binary files differ'.
             # Bad options could also be the problem.
             # 'Binary files' is not a real error, so we suppress that error
+            lang_c_out = out
+
+            # Since we got here, we want to make sure to give an i18n error
+            pipe = _spawn_external_diff(diffcmd, capture_errors=False)
+            out, err = pipe.communicate()
+
+            # Write out the new i18n diff response
+            to_file.write(out+'\n')
+            if pipe.returncode != 2:
+                raise BzrError('external diff failed with exit code 2'
+                               ' when run with LANG=C, but not when run'
+                               ' natively: %r' % (diffcmd,))
+
+            first_line = lang_c_out.split('\n', 1)[0]
             m = re.match('^binary files.*differ$', first_line, re.I)
-            if not m:
+            if m is None:
                 raise BzrError('external diff failed with exit code 2;'
                                ' command: %r' % (diffcmd,))
-        elif rc not in (0, 1):
+            else:
+                # Binary files differ, just return
+                return
+
+        # If we got to here, we haven't written out the output of diff
+        # do so now
+        to_file.write(out)
+        if rc not in (0, 1):
             # returns 1 if files differ; that's OK
             if rc < 0:
                 msg = 'signal %d' % (-rc)
@@ -181,8 +228,6 @@
             raise BzrError('external diff failed with %s; command: %r' 
                            % (rc, diffcmd))
 
-        # internal_diff() adds a trailing newline, add one here for consistency
-        to_file.write('\n')
 
     finally:
         oldtmpf.close()                 # and delete

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-08-17 04:28:45 +0000
+++ bzrlib/errors.py	2006-08-22 05:52:51 +0000
@@ -1105,8 +1105,8 @@
         BzrNewError.__init__(self)
 
 
-class GhostRevision(BzrNewError):
-    """Revision {%(revision_id)s} is a ghost."""
+class GhostRevisionUnusableHere(BzrNewError):
+    """Ghost revision {%(revision_id)s} cannot be used here."""
 
     def __init__(self, revision_id):
         BzrNewError.__init__(self)

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2006-07-30 07:02:22 +0000
+++ bzrlib/help.py	2006-08-02 13:02:47 +0000
@@ -128,26 +128,12 @@
 
 
 def help_on_command_options(cmd, outfile=None):
-    from bzrlib.option import Option
+    from bzrlib.option import Option, get_optparser
+    if outfile is None:
+        outfile = sys.stdout
     options = cmd.options()
-    if not options:
-        return
-    if outfile == None:
-        outfile = sys.stdout
-    outfile.write('\noptions:\n')
-    for option_name, option in sorted(options.items()):
-        l = '    --' + option_name
-        if option.type is not None:
-            l += ' ' + option.argname.upper()
-        short_name = option.short_name()
-        if short_name:
-            assert len(short_name) == 1
-            l += ', -' + short_name
-        l += (30 - len(l)) * ' ' + option.help
-        # TODO: split help over multiple lines with correct indenting and 
-        # wrapping
-        wrapped = textwrap.fill(l, initial_indent='', subsequent_indent=30*' ')
-        outfile.write(wrapped + '\n')
+    outfile.write('\n')
+    outfile.write(get_optparser(options).format_option_help())
 
 
 def help_commands(outfile=None):

=== modified file 'bzrlib/option.py'
--- bzrlib/option.py	2006-07-28 16:05:23 +0000
+++ bzrlib/option.py	2006-08-16 13:13:39 +0000
@@ -17,6 +17,7 @@
 # TODO: For things like --diff-prefix, we want a way to customize the display
 # of the option argument.
 
+import optparse
 import re
 
 from bzrlib.trace import warning
@@ -153,6 +154,68 @@
             if option is self:
                 return short
 
+    def get_negation_name(self):
+        if self.name.startswith('no-'):
+            return self.name[3:]
+        else:
+            return 'no-' + self.name
+
+    def add_option(self, parser, short_name):
+        """Add this option to an Optparse parser"""
+        option_strings = ['--%s' % self.name]
+        if short_name is not None:
+            option_strings.append('-%s' % short_name)
+        optargfn = self.type
+        if optargfn is None:
+            parser.add_option(action='store_true', dest=self.name, 
+                              help=self.help,
+                              default=OptionParser.DEFAULT_VALUE,
+                              *option_strings)
+            negation_strings = ['--%s' % self.get_negation_name()]
+            parser.add_option(action='store_false', dest=self.name, 
+                              help=optparse.SUPPRESS_HELP, *negation_strings)
+        else:
+            parser.add_option(action='callback', 
+                              callback=self._optparse_callback, 
+                              type='string', metavar=self.argname.upper(),
+                              help=self.help,
+                              default=OptionParser.DEFAULT_VALUE, 
+                              *option_strings)
+
+    def _optparse_callback(self, option, opt, value, parser):
+        setattr(parser.values, self.name, self.type(value))
+
+    def iter_switches(self):
+        """Iterate through the list of switches provided by the option
+        
+        :return: an iterator of (name, short_name, argname, help)
+        """
+        argname =  self.argname
+        if argname is not None:
+            argname = argname.upper()
+        yield self.name, self.short_name(), argname, self.help
+
+
+class OptionParser(optparse.OptionParser):
+    """OptionParser that raises exceptions instead of exiting"""
+
+    DEFAULT_VALUE = object()
+
+    def error(self, message):
+        raise BzrCommandError(message)
+
+
+def get_optparser(options):
+    """Generate an optparse parser for bzrlib-style options"""
+
+    parser = OptionParser()
+    parser.remove_option('--help')
+    short_options = dict((k.name, v) for v, k in 
+                         Option.SHORT_OPTIONS.iteritems())
+    for option in options.itervalues():
+        option.add_option(parser, short_options.get(option.name))
+    return parser
+
 
 def _global_option(name, **kwargs):
     """Register o as a global option."""

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-08-15 03:18:20 +0000
+++ bzrlib/repository.py	2006-08-16 21:10:33 +0000
@@ -20,7 +20,6 @@
 import re
 import time
 from unittest import TestSuite
-from warnings import warn
 
 from bzrlib import bzrdir, check, delta, gpg, errors, xml5, ui, transactions, osutils
 from bzrlib.decorators import needs_read_lock, needs_write_lock
@@ -37,6 +36,7 @@
 from bzrlib.revisiontree import RevisionTree
 from bzrlib.store.versioned import VersionedFileStore, WeaveStore
 from bzrlib.store.text import TextStore
+from bzrlib import symbol_versioning
 from bzrlib.symbol_versioning import (deprecated_method,
         zero_nine, 
         )
@@ -2038,8 +2038,8 @@
     def finish_inventory(self):
         """Tell the builder that the inventory is finished."""
         if self.new_inventory.root is None:
-            warn('Root entry should be supplied to record_entry_contents, as'
-                 ' of bzr 0.10.',
+            symbol_versioning.warn('Root entry should be supplied to'
+                ' record_entry_contents, as of bzr 0.10.',
                  DeprecationWarning, stacklevel=2)
             self.new_inventory.add(InventoryDirectory(ROOT_ID, '', None))
         self.new_inventory.revision_id = self._new_revision_id
@@ -2080,6 +2080,12 @@
         :param tree: The tree which contains this entry and should be used to 
         obtain content.
         """
+        if self.new_inventory.root is None and ie.parent_id is not None:
+            symbol_versioning.warn('Root entry should be supplied to'
+                ' record_entry_contents, as of bzr 0.10.',
+                 DeprecationWarning, stacklevel=2)
+            self.record_entry_contents(tree.inventory.root.copy(), parent_invs,
+                                       '', tree)
         self.new_inventory.add(ie)
 
         # ie.revision is always None if the InventoryEntry is considered

=== modified file 'bzrlib/testament.py'
--- bzrlib/testament.py	2006-07-21 01:37:47 +0000
+++ bzrlib/testament.py	2006-08-16 16:55:54 +0000
@@ -70,12 +70,11 @@
 # revisions can be serialized.
 
 from copy import copy
-from cStringIO import StringIO
-import string
 from sha import sha
 
 from bzrlib.osutils import contains_whitespace, contains_linebreaks
 
+
 class Testament(object):
     """Reduced summary of a revision.
 
@@ -145,23 +144,28 @@
 
     def _escape_path(self, path):
         assert not contains_linebreaks(path)
-        return unicode(path.replace('\\', '/').replace(' ', '\ ')).encode('utf-8')
+        return unicode(path.replace('\\', '/').replace(' ', '\ '))
 
     def _entry_to_line(self, path, ie):
         """Turn an inventory entry into a testament line"""
-        l = '  ' + str(ie.kind)
-        l += ' ' + self._escape_path(path)
         assert not contains_whitespace(ie.file_id)
-        l += ' ' + unicode(ie.file_id).encode('utf-8')
+
+        content = ''
+        content_spacer=''
         if ie.kind == 'file':
             # TODO: avoid switching on kind
             assert ie.text_sha1
-            l += ' ' + ie.text_sha1
+            content = ie.text_sha1
+            content_spacer = ' '
         elif ie.kind == 'symlink':
             assert ie.symlink_target
-            l += ' ' + self._escape_path(ie.symlink_target)
-        l += '\n'
-        return l.decode('utf-8')
+            content = self._escape_path(ie.symlink_target)
+            content_spacer = ' '
+
+        l = u'  %s %s %s%s%s\n' % (ie.kind, self._escape_path(path),
+                                   unicode(ie.file_id),
+                                   content_spacer, content)
+        return l
 
     def as_text(self):
         return ''.join(self.as_text_lines())
@@ -183,9 +187,7 @@
             assert not contains_whitespace(name)
             r.append('  %s:\n' % name)
             for line in value.splitlines():
-                if not isinstance(line, str):
-                    line = line.encode('utf-8')
-                r.append('    %s\n' % line)
+                r.append(u'    %s\n' % line)
         return r
 
     def as_sha1(self):
@@ -201,6 +203,6 @@
     short_header = 'bazaar-ng testament short form 2.1\n'
     def _entry_to_line(self, path, ie):
         l = Testament._entry_to_line(self, path, ie)[:-1]
-        l += ' ' + ie.revision.decode('utf-8')
+        l += ' ' + ie.revision
         l += {True: ' yes\n', False: ' no\n'}[ie.executable]
         return l

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-08-15 13:20:18 +0000
+++ bzrlib/tests/__init__.py	2006-08-16 21:10:33 +0000
@@ -63,7 +63,9 @@
 import bzrlib.plugin
 import bzrlib.progress as progress
 from bzrlib.revision import common_ancestor
+from bzrlib.revisionspec import RevisionSpec
 import bzrlib.store
+from bzrlib import symbol_versioning
 import bzrlib.trace
 from bzrlib.transport import get_transport
 import bzrlib.transport
@@ -136,9 +138,25 @@
     """
     stop_early = False
     
-    def __init__(self, stream, descriptions, verbosity, pb=None):
+    def __init__(self, stream, descriptions, verbosity, pb=None,
+                 bench_history=None):
+        """Construct new TestResult.
+
+        :param bench_history: Optionally, a writable file object to accumulate
+            benchmark results.
+        """
         unittest._TextTestResult.__init__(self, stream, descriptions, verbosity)
         self.pb = pb
+        if bench_history is not None:
+            from bzrlib.version import _get_bzr_source_tree
+            src_tree = _get_bzr_source_tree()
+            if src_tree:
+                revision_id = src_tree.last_revision()
+            else:
+                # XXX: If there's no branch, what should we do?
+                revision_id = ''
+            bench_history.write("--date %s %s\n" % (time.time(), revision_id))
+        self._bench_history = bench_history
     
     def extractBenchmarkTime(self, testCase):
         """Add a benchmark time for the current test case."""
@@ -248,6 +266,11 @@
 
     def addSuccess(self, test):
         self.extractBenchmarkTime(test)
+        if self._bench_history is not None:
+            if self._benchmarkTime is not None:
+                self._bench_history.write("%s %s\n" % (
+                    self._formatTime(self._benchmarkTime),
+                    test.id()))
         if self.showAll:
             self.stream.writeln('   OK %s' % self._testTimeString())
             for bench_called, stats in getattr(test, '_benchcalls', []):
@@ -304,18 +327,21 @@
                  descriptions=0,
                  verbosity=1,
                  keep_output=False,
-                 pb=None):
+                 pb=None,
+                 bench_history=None):
         self.stream = unittest._WritelnDecorator(stream)
         self.descriptions = descriptions
         self.verbosity = verbosity
         self.keep_output = keep_output
         self.pb = pb
+        self._bench_history = bench_history
 
     def _makeResult(self):
         result = _MyResult(self.stream,
                            self.descriptions,
                            self.verbosity,
-                           pb=self.pb)
+                           pb=self.pb,
+                           bench_history=self._bench_history)
         result.stop_early = self.stop_on_failure
         return result
 
@@ -395,7 +421,6 @@
 
 class TestSkipped(Exception):
     """Indicates that a test was intentionally skipped, rather than failing."""
-    # XXX: Not used yet
 
 
 class CommandFailed(Exception):
@@ -552,6 +577,27 @@
             self.fail("%r is an instance of %s rather than %s" % (
                 obj, obj.__class__, kls))
 
+    def assertDeprecated(self, expected, callable, *args, **kwargs):
+        """Assert that a callable is deprecated in a particular way.
+
+        :param expected: a list of the deprecation warnings expected, in order
+        :param callable: The callable to call
+        :param args: The positional arguments for the callable
+        :param kwargs: The keyword arguments for the callable
+        """
+        local_warnings = []
+        def capture_warnings(msg, cls, stacklevel=None):
+            self.assertEqual(cls, DeprecationWarning)
+            local_warnings.append(msg)
+        method = symbol_versioning.warn
+        symbol_versioning.set_warning_method(capture_warnings)
+        try:
+            callable(*args, **kwargs)
+        finally:
+            result = symbol_versioning.set_warning_method(method)
+        self.assertEqual(expected, local_warnings)
+        return result
+
     def _startLogFile(self):
         """Send bzr and test log messages to a temporary file.
 
@@ -1209,7 +1255,7 @@
 
 def run_suite(suite, name='test', verbose=False, pattern=".*",
               stop_on_failure=False, keep_output=False,
-              transport=None, lsprof_timed=None):
+              transport=None, lsprof_timed=None, bench_history=None):
     TestCaseInTempDir._TEST_NAME = name
     TestCase._gather_lsprof_in_benchmarks = lsprof_timed
     if verbose:
@@ -1222,7 +1268,8 @@
                             descriptions=0,
                             verbosity=verbosity,
                             keep_output=keep_output,
-                            pb=pb)
+                            pb=pb,
+                            bench_history=bench_history)
     runner.stop_on_failure=stop_on_failure
     if pattern != '.*':
         suite = filter_suite_by_re(suite, pattern)
@@ -1234,7 +1281,8 @@
              keep_output=False,
              transport=None,
              test_suite_factory=None,
-             lsprof_timed=None):
+             lsprof_timed=None,
+             bench_history=None):
     """Run the whole test suite under the enhanced runner"""
     # XXX: Very ugly way to do this...
     # Disable warning about old formats because we don't want it to disturb
@@ -1255,7 +1303,8 @@
         return run_suite(suite, 'testbzr', verbose=verbose, pattern=pattern,
                      stop_on_failure=stop_on_failure, keep_output=keep_output,
                      transport=transport,
-                     lsprof_timed=lsprof_timed)
+                     lsprof_timed=lsprof_timed,
+                     bench_history=bench_history)
     finally:
         default_transport = old_transport
 
@@ -1340,6 +1389,7 @@
                    'bzrlib.tests.test_upgrade',
                    'bzrlib.tests.test_urlutils',
                    'bzrlib.tests.test_versionedfile',
+                   'bzrlib.tests.test_version',
                    'bzrlib.tests.test_weave',
                    'bzrlib.tests.test_whitebox',
                    'bzrlib.tests.test_workingtree',

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2006-07-31 15:12:38 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-08-16 16:27:58 +0000
@@ -81,6 +81,7 @@
                      'bzrlib.tests.blackbox.test_shared_repository',
                      'bzrlib.tests.blackbox.test_sign_my_commits',
                      'bzrlib.tests.blackbox.test_status',
+                     'bzrlib.tests.blackbox.test_testament',
                      'bzrlib.tests.blackbox.test_too_much',
                      'bzrlib.tests.blackbox.test_uncommit',
                      'bzrlib.tests.blackbox.test_update',

=== modified file 'bzrlib/tests/blackbox/test_add.py'
--- bzrlib/tests/blackbox/test_add.py	2006-08-09 19:53:02 +0000
+++ bzrlib/tests/blackbox/test_add.py	2006-08-16 08:03:47 +0000
@@ -178,3 +178,12 @@
         new_tree.read_working_inventory()
         self.assertEqual(base_tree.path2id('b/c'), new_tree.path2id('c'))
         self.assertEqual(base_tree.path2id('b/d'), new_tree.path2id('d'))
+
+    def test_add_dry_run(self):
+        # ensure that --dry-run actually don't add anything
+        base_tree = self.make_branch_and_tree('.')
+        self.build_tree(['spam'])
+        out = self.run_bzr_captured(['add', '--dry-run'], retcode=0)[0]
+        self.assertEquals('added spam\n', out)
+        out = self.run_bzr_captured(['added'], retcode=0)[0]
+        self.assertEquals('', out)

=== modified file 'bzrlib/tests/blackbox/test_selftest.py'
--- bzrlib/tests/blackbox/test_selftest.py	2006-07-20 04:04:58 +0000
+++ bzrlib/tests/blackbox/test_selftest.py	2006-08-16 16:33:11 +0000
@@ -22,6 +22,7 @@
 from bzrlib.tests import (
                           TestCase,
                           TestCaseInTempDir,
+                          TestCaseWithTransport,
                           TestSkipped,
                           )
 from bzrlib.tests.blackbox import ExternalBase
@@ -77,16 +78,6 @@
             TestOptions.current_test = None
             TestCaseInTempDir.TEST_ROOT = old_root
 
-    def test_benchmark_runs_benchmark_tests(self):
-        """bzr selftest --benchmark should not run the default test suite."""
-        # We test this by passing a regression test name to --benchmark, which
-        # should result in 0 rests run.
-        out, err = self.run_bzr('selftest', '--benchmark', 'workingtree_implementations')
-        self.assertContainsRe(out, 'Ran 0 tests.*\n\nOK')
-        self.assertEqual(
-            'running tests...\ntests passed\n',
-            err)
-        
 
 class TestRunBzr(ExternalBase):
 
@@ -105,6 +96,31 @@
         self.assertEqual('zippy', self.stdin)
 
 
+class TestBenchmarkTests(TestCaseWithTransport):
+
+    def test_benchmark_runs_benchmark_tests(self):
+        """bzr selftest --benchmark should not run the default test suite."""
+        # We test this by passing a regression test name to --benchmark, which
+        # should result in 0 rests run.
+        old_root = TestCaseInTempDir.TEST_ROOT
+        try:
+            TestCaseInTempDir.TEST_ROOT = None
+            out, err = self.run_bzr('selftest', '--benchmark', 'workingtree_implementations')
+        finally:
+            TestCaseInTempDir.TEST_ROOT = old_root
+        self.assertContainsRe(out, 'Ran 0 tests.*\n\nOK')
+        self.assertEqual(
+            'running tests...\ntests passed\n',
+            err)
+        benchfile = open(".perf_history", "rt")
+        try:
+            lines = benchfile.readlines()
+        finally:
+            benchfile.close()
+        self.assertEqual(1, len(lines))
+        self.assertContainsRe(lines[0], "--date [0-9.]+")
+
+
 class TestRunBzrCaptured(ExternalBase):
 
     def apply_redirected(self, stdin=None, stdout=None, stderr=None,

=== modified file 'bzrlib/tests/blackbox/test_status.py'
--- bzrlib/tests/blackbox/test_status.py	2006-08-17 03:35:26 +0000
+++ bzrlib/tests/blackbox/test_status.py	2006-08-22 05:40:35 +0000
@@ -40,78 +40,89 @@
 
 class BranchStatus(TestCaseWithTransport):
     
+    def assertStatus(self, output_lines, working_tree,
+        revision=None):
+        """Run status in working_tree and look for output.
+        
+        :param output_lines: The lines to look for.
+        :param working_tree: The tree to run status in.
+        """
+        output_string = self.status_string(working_tree, revision)
+        self.assertEqual(output_lines, output_string.splitlines(True))
+    
+    def status_string(self, wt, revision=None):
+        # use a real file rather than StringIO because it doesn't handle
+        # Unicode very well.
+        tof = codecs.getwriter('utf-8')(TemporaryFile())
+        show_tree_status(wt, to_file=tof, revision=revision)
+        tof.seek(0)
+        return tof.read().decode('utf-8')
+
     def test_branch_status(self):
         """Test basic branch status"""
         wt = self.make_branch_and_tree('.')
+
+        ignores._set_user_ignores(['./.bazaar'])
+
+        # status with no commits or files - it must
+        # work and show no output. We do this with no
+        # commits to be sure that it's not going to fail
+        # as a corner case.
+        self.assertStatus([], wt)
+
+        self.build_tree(['hello.c', 'bye.c'])
+        self.assertStatus([
+                'unknown:\n',
+                '  bye.c\n',
+                '  hello.c\n',
+            ],
+            wt)
+
+        # add a commit to allow showing pending merges.
         wt.commit('create a parent to allow testing merge output')
 
-        ignores._set_user_ignores(['./.bazaar'])
-
-        # status with nothing
-        tof = StringIO()
-        show_tree_status(wt, to_file=tof)
-        self.assertEquals(tof.getvalue(), "")
-
-        tof = StringIO()
-        self.build_tree(['hello.c', 'bye.c'])
         wt.add_pending_merge('pending@pending-0-0')
-        show_tree_status(wt, to_file=tof)
-        tof.seek(0)
-        self.assertEquals(tof.readlines(),
-                          ['unknown:\n',
-                           '  bye.c\n',
-                           '  hello.c\n',
-                           'pending merges:\n',
-                           '  pending@pending-0-0\n'
-                           ])
+        self.assertStatus([
+                'unknown:\n',
+                '  bye.c\n',
+                '  hello.c\n',
+                'pending merges:\n',
+                '  pending@pending-0-0\n',
+            ],
+            wt)
 
     def test_branch_status_revisions(self):
         """Tests branch status with revisions"""
         wt = self.make_branch_and_tree('.')
-        b = wt.branch
 
         ignores._set_user_ignores(['./.bazaar'])
 
-        tof = StringIO()
         self.build_tree(['hello.c', 'bye.c'])
         wt.add('hello.c')
         wt.add('bye.c')
         wt.commit('Test message')
 
-        tof = StringIO()
-        revs =[]
-        revs.append(RevisionSpec(0))
-        
-        show_tree_status(wt, to_file=tof, revision=revs)
-        
-        tof.seek(0)
-        self.assertEquals(tof.readlines(),
-                          ['added:\n',
-                           '  bye.c\n',
-                           '  hello.c\n'])
+        revs = [RevisionSpec(0)]
+        self.assertStatus([
+                'added:\n',
+                '  bye.c\n',
+                '  hello.c\n'
+            ],
+            wt,
+            revision=revs)
 
         self.build_tree(['more.c'])
         wt.add('more.c')
         wt.commit('Another test message')
         
-        tof = StringIO()
         revs.append(RevisionSpec(1))
-        
-        show_tree_status(wt, to_file=tof, revision=revs)
-        
-        tof.seek(0)
-        self.assertEquals(tof.readlines(),
-                          ['added:\n',
-                           '  bye.c\n',
-                           '  hello.c\n'])
-
-    def status_string(self, wt):
-        # use a real file rather than StringIO because it doesn't handle
-        # Unicode very well.
-        tof = codecs.getwriter('utf-8')(TemporaryFile())
-        show_tree_status(wt, to_file=tof)
-        tof.seek(0)
-        return tof.read().decode('utf-8')
+        self.assertStatus([
+                'added:\n',
+                '  bye.c\n',
+                '  hello.c\n',
+            ],
+            wt,
+            revision=revs)
 
     def test_pending(self):
         """Pending merges display works, including Unicode"""

=== added file 'bzrlib/tests/blackbox/test_testament.py'
--- bzrlib/tests/blackbox/test_testament.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_testament.py	2006-08-16 16:27:58 +0000
@@ -0,0 +1,48 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Blackbox tests for the 'bzr testament' command"""
+
+
+from bzrlib.tests.test_testament import (
+    REV_1_SHORT,
+    REV_1_SHORT_STRICT,
+    REV_2_TESTAMENT,
+    TestamentSetup,
+    )
+
+
+class TestTestament(TestamentSetup):
+    """Run blackbox tests on 'bzr testament'"""
+
+    def test_testament_command(self):
+        """Testament containing a file and a directory."""
+        out, err = self.run_bzr('testament', '--long')
+        self.assertEqualDiff(err, '')
+        self.assertEqualDiff(out, REV_2_TESTAMENT)
+
+    def test_testament_command_2(self):
+        """Command getting short testament of previous version."""
+        out, err = self.run_bzr('testament', '-r1')
+        self.assertEqualDiff(err, '')
+        self.assertEqualDiff(out, REV_1_SHORT)
+
+    def test_testament_command_3(self):
+        """Command getting short testament of previous version."""
+        out, err = self.run_bzr('testament', '-r1', '--strict')
+        self.assertEqualDiff(err, '')
+        self.assertEqualDiff(out, REV_1_SHORT_STRICT)
+

=== modified file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-08-17 14:16:46 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-08-22 10:50:58 +0000
@@ -104,6 +104,44 @@
                                          target.get(path).read(),
                                          "text for file %r differs:\n" % path)
 
+    def skipIfNoWorkingTree(self, a_bzrdir):
+        """Raises TestSkipped if a_bzrdir doesn't have a working tree.
+        
+        If the bzrdir does have a workingtree, this is a no-op.
+        """
+        try:
+            a_bzrdir.open_workingtree()
+        except (errors.NotLocalUrl, errors.NoWorkingTree):
+            raise TestSkipped("bzrdir on transport %r has no working tree"
+                              % a_bzrdir.transport)
+
+    def createWorkingTreeOrSkip(self, a_bzrdir):
+        """Create a working tree on a_bzrdir, or raise TestSkipped.
+        
+        A simple wrapper for create_workingtree that translates NotLocalUrl into
+        TestSkipped.  Returns the newly created working tree.
+        """
+        try:
+            return a_bzrdir.create_workingtree()
+        except errors.NotLocalUrl:
+            raise TestSkipped("cannot make working tree with transport %r"
+                              % a_bzrdir.transport)
+
+    def sproutOrSkip(self, from_bzrdir, to_url, revision_id=None, basis=None,
+                     force_new_repo=False):
+        """Sprout from_bzrdir into to_url, or raise TestSkipped.
+        
+        A simple wrapper for from_bzrdir.sprout that translates NotLocalUrl into
+        TestSkipped.  Returns the newly sprouted bzrdir.
+        """
+        try:
+            target = from_bzrdir.sprout(to_url, revision_id=revision_id,
+                                        basis=basis,
+                                        force_new_repo=force_new_repo)
+        except errors.NotLocalUrl:
+            raise TestSkipped('Cannot sprout to remote bzrdirs.')
+        return target
+
     def test_clone_bzrdir_empty(self):
         dir = self.make_bzrdir('source')
         target = dir.clone(self.get_url('target'))
@@ -332,6 +370,7 @@
         tree.commit('revision 1')
         dir = tree.bzrdir
         target = dir.clone(self.get_url('target'))
+        self.skipIfNoWorkingTree(target)
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
                                     ['./.bzr/stat-cache',
@@ -348,6 +387,7 @@
         tree.commit('revision 1')
         dir = tree.bzrdir
         target = dir.clone(self.get_url('target'))
+        self.skipIfNoWorkingTree(target)
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
                                     ['./.bzr/stat-cache',
                                      './.bzr/checkout/stat-cache',
@@ -373,8 +413,9 @@
         except errors.IncompatibleFormat:
             # this is ok too, not all formats have to support references.
             return
-        dir.create_workingtree()
+        self.createWorkingTreeOrSkip(dir)
         target = dir.clone(self.get_url('target'))
+        self.skipIfNoWorkingTree(target)
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
                                     ['./.bzr/stat-cache',
@@ -382,7 +423,6 @@
                                      './.bzr/repository/inventory.knit',
                                      ])
 
-
     def test_clone_bzrdir_tree_revision(self):
         # test for revision limiting, [smoke test, not corner case checks].
         # make a tree with a revision with a last-revision
@@ -396,6 +436,7 @@
         tree.commit('revision 2', rev_id='2', allow_pointless=True)
         dir = tree.bzrdir
         target = dir.clone(self.get_url('target'), revision_id='1')
+        self.skipIfNoWorkingTree(target)
         self.assertEqual('1', target.open_workingtree().last_revision())
 
     def test_clone_bzrdir_incomplete_source_with_basis(self):
@@ -414,12 +455,17 @@
         dir = source.bzrdir
         target = dir.clone(self.get_url('target'), basis=tree.bzrdir)
         self.assertEqual('2', target.open_branch().last_revision())
-        self.assertEqual('2', target.open_workingtree().last_revision())
+        try:
+            self.assertEqual('2', target.open_workingtree().last_revision())
+        except errors.NoWorkingTree:
+            # It should have a working tree if it's able to have one, so if
+            # we're here make sure it really can't have one.
+            self.assertRaises(errors.NotLocalUrl, target.create_workingtree)
         self.assertTrue(target.open_branch().repository.has_revision('2'))
 
     def test_sprout_bzrdir_empty(self):
         dir = self.make_bzrdir('source')
-        target = dir.sprout(self.get_url('target'))
+        target = self.sproutOrSkip(dir, self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # creates a new repository branch and tree
         target.open_repository()
@@ -433,12 +479,12 @@
             self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
             return
-        target = dir.sprout(self.get_url('target/child'))
+        target = self.sproutOrSkip(dir, self.get_url('target/child'))
         self.assertRaises(errors.NoRepositoryPresent, target.open_repository)
         target.open_branch()
         target.open_workingtree()
 
-    def test_sprout_bzrdir_empty_under_shared_repo(self):
+    def test_sprout_bzrdir_empty_under_shared_repo_force_new(self):
         # the force_new_repo parameter should force use of a new repo in an empty
         # bzrdir's sprout logic
         dir = self.make_bzrdir('source')
@@ -446,7 +492,8 @@
             self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
             return
-        target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
+        target = self.sproutOrSkip(dir, self.get_url('target/child'),
+                                   force_new_repo=True)
         target.open_repository()
         target.open_branch()
         target.open_workingtree()
@@ -460,7 +507,7 @@
         repo = dir.create_repository()
         repo.fetch(tree.branch.repository)
         self.assertTrue(repo.has_revision('1'))
-        target = dir.sprout(self.get_url('target'))
+        target = self.sproutOrSkip(dir, self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
                                     ['./.bzr/repository/inventory.knit',
@@ -481,7 +528,7 @@
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
             return
-        target = dir.sprout(self.get_url('target/child'))
+        target = self.sproutOrSkip(dir, self.get_url('target/child'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertTrue(shared_repo.has_revision('1'))
 
@@ -500,7 +547,7 @@
         tree.bzrdir.open_repository().copy_content_into(shared_repo)
         dir = self.make_bzrdir('shared/source')
         dir.create_branch()
-        target = dir.sprout(self.get_url('shared/target'))
+        target = self.sproutOrSkip(dir, self.get_url('shared/target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertNotEqual(dir.transport.base, shared_repo.bzrdir.transport.base)
         self.assertTrue(shared_repo.has_revision('1'))
@@ -523,7 +570,7 @@
         self.assertTrue(shared_repo.has_revision('1'))
         dir = self.make_bzrdir('shared/source')
         dir.create_branch()
-        target = dir.sprout(self.get_url('target'))
+        target = self.sproutOrSkip(dir, self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertNotEqual(dir.transport.base, shared_repo.bzrdir.transport.base)
         branch = target.open_branch()
@@ -546,7 +593,8 @@
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
             return
-        target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
+        target = self.sproutOrSkip(dir, self.get_url('target/child'),
+                                   force_new_repo=True)
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertFalse(shared_repo.has_revision('1'))
 
@@ -565,7 +613,7 @@
         source = self.make_repository('source')
         tree.bzrdir.open_repository().copy_content_into(source)
         dir = source.bzrdir
-        target = dir.sprout(self.get_url('target'), revision_id='2')
+        target = self.sproutOrSkip(dir, self.get_url('target'), revision_id='2')
         raise TestSkipped('revision limiting not strict yet')
 
     def test_sprout_bzrdir_branch_and_repo(self):
@@ -577,7 +625,7 @@
         tree.bzrdir.open_repository().copy_content_into(source.repository)
         tree.bzrdir.open_branch().copy_content_into(source)
         dir = source.bzrdir
-        target = dir.sprout(self.get_url('target'))
+        target = self.sproutOrSkip(dir, self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
                                     ['./.bzr/stat-cache',
@@ -602,7 +650,7 @@
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
             return
-        target = dir.sprout(self.get_url('target/child'))
+        target = self.sproutOrSkip(dir, self.get_url('target/child'))
         self.assertTrue(shared_repo.has_revision('1'))
 
     def test_sprout_bzrdir_branch_and_repo_shared_force_new_repo(self):
@@ -620,7 +668,8 @@
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
             return
-        target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
+        target = self.sproutOrSkip(dir, self.get_url('target/child'),
+                                   force_new_repo=True)
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertFalse(shared_repo.has_revision('1'))
 
@@ -635,7 +684,7 @@
             # this is ok too, not all formats have to support references.
             return
         self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
-        target = dir.sprout(self.get_url('target'))
+        target = self.sproutOrSkip(dir, self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # we want target to have a branch that is in-place.
         self.assertEqual(target, target.open_branch().bzrdir)
@@ -659,7 +708,7 @@
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
             return
-        target = dir.sprout(self.get_url('target/child'))
+        target = self.sproutOrSkip(dir, self.get_url('target/child'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # we want target to have a branch that is in-place.
         self.assertEqual(target, target.open_branch().bzrdir)
@@ -685,7 +734,8 @@
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
             return
-        target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
+        target = self.sproutOrSkip(dir, self.get_url('target/child'),
+                                   force_new_repo=True)
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # we want target to have a branch that is in-place.
         self.assertEqual(target, target.open_branch().bzrdir)
@@ -708,7 +758,7 @@
         tree.bzrdir.open_repository().copy_content_into(source.repository)
         tree.bzrdir.open_branch().copy_content_into(source)
         dir = source.bzrdir
-        target = dir.sprout(self.get_url('target'), revision_id='1')
+        target = self.sproutOrSkip(dir, self.get_url('target'), revision_id='1')
         self.assertEqual('1', target.open_branch().last_revision())
         
     def test_sprout_bzrdir_tree_branch_repo(self):
@@ -717,7 +767,7 @@
         tree.add('foo')
         tree.commit('revision 1')
         dir = tree.bzrdir
-        target = dir.sprout(self.get_url('target'))
+        target = self.sproutOrSkip(dir, self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
                                     ['./.bzr/stat-cache',
@@ -739,10 +789,10 @@
             # this is ok too, not all formats have to support references.
             return
         self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
-        tree = dir.create_workingtree()
+        tree = self.createWorkingTreeOrSkip(dir)
         tree.bzrdir.root_transport.mkdir('subdir')
         tree.add('subdir')
-        target = dir.sprout(self.get_url('target'))
+        target = self.sproutOrSkip(dir, self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # we want target to have a branch that is in-place.
         self.assertEqual(target, target.open_branch().bzrdir)
@@ -765,12 +815,13 @@
             # this is ok too, not all formats have to support references.
             return
         self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
-        tree = dir.create_workingtree()
+        tree = self.createWorkingTreeOrSkip(dir)
         self.build_tree(['foo'], transport=dir.root_transport)
         tree.add('foo')
         tree.commit('revision 1', rev_id='1')
         tree.commit('revision 2', rev_id='2', allow_pointless=True)
         target = dir.sprout(self.get_url('target'), revision_id='1')
+        self.skipIfNoWorkingTree(target)
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # we want target to have a branch that is in-place.
         self.assertEqual(target, target.open_branch().bzrdir)
@@ -793,7 +844,7 @@
         tree.commit('revision 1', rev_id='1')
         tree.commit('revision 2', rev_id='2', allow_pointless=True)
         dir = tree.bzrdir
-        target = dir.sprout(self.get_url('target'), revision_id='1')
+        target = self.sproutOrSkip(dir, self.get_url('target'), revision_id='1')
         self.assertEqual('1', target.open_workingtree().last_revision())
 
     def test_sprout_bzrdir_incomplete_source_with_basis(self):
@@ -810,7 +861,8 @@
         tree.copy_content_into(source)
         self.assertFalse(source.branch.repository.has_revision('2'))
         dir = source.bzrdir
-        target = dir.sprout(self.get_url('target'), basis=tree.bzrdir)
+        target = self.sproutOrSkip(dir, self.get_url('target'),
+                                   basis=tree.bzrdir)
         self.assertEqual('2', target.open_branch().last_revision())
         self.assertEqual('2', target.open_workingtree().last_revision())
         self.assertTrue(target.open_branch().repository.has_revision('2'))
@@ -936,26 +988,29 @@
             # because the default open will not open them and
             # they may not be initializable.
             return
-        # this has to be tested with local access as we still support creating 
-        # format 6 bzrdirs
-        t = get_transport('.')
+        t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
         made_branch = made_control.create_branch()
-        made_tree = made_control.create_workingtree()
+        made_tree = self.createWorkingTreeOrSkip(made_control)
         self.failUnless(isinstance(made_tree, workingtree.WorkingTree))
         self.assertEqual(made_control, made_tree.bzrdir)
         
     def test_create_workingtree_revision(self):
         # a bzrdir can construct a working tree for itself @ a specific revision.
+        t = self.get_transport()
         source = self.make_branch_and_tree('source')
         source.commit('a', rev_id='a', allow_pointless=True)
         source.commit('b', rev_id='b', allow_pointless=True)
         self.build_tree(['new/'])
-        made_control = self.bzrdir_format.initialize('new')
+        t_new = t.clone('new')
+        made_control = self.bzrdir_format.initialize_on_transport(t_new)
         source.branch.repository.clone(made_control)
         source.branch.clone(made_control)
-        made_tree = made_control.create_workingtree(revision_id='a')
+        try:
+            made_tree = made_control.create_workingtree(revision_id='a')
+        except errors.NotLocalUrl:
+            raise TestSkipped("Can't make working tree on transport %r" % t)
         self.assertEqual('a', made_tree.last_revision())
         
     def test_open_workingtree(self):
@@ -994,8 +1049,8 @@
         except errors.IncompatibleFormat:
             found_transport = dir.get_branch_transport(identifiable_format)
         self.assertTrue(isinstance(found_transport, transport.Transport))
-        # and the dir which has been initialized for us must be statable.
-        found_transport.stat('.')
+        # and the dir which has been initialized for us must exist.
+        found_transport.list_dir('.')
 
     def test_get_repository_transport(self):
         dir = self.make_bzrdir('.')
@@ -1015,8 +1070,8 @@
         except errors.IncompatibleFormat:
             found_transport = dir.get_repository_transport(identifiable_format)
         self.assertTrue(isinstance(found_transport, transport.Transport))
-        # and the dir which has been initialized for us must be statable.
-        found_transport.stat('.')
+        # and the dir which has been initialized for us must exist.
+        found_transport.list_dir('.')
 
     def test_get_workingtree_transport(self):
         dir = self.make_bzrdir('.')
@@ -1036,8 +1091,8 @@
         except errors.IncompatibleFormat:
             found_transport = dir.get_workingtree_transport(identifiable_format)
         self.assertTrue(isinstance(found_transport, transport.Transport))
-        # and the dir which has been initialized for us must be statable.
-        found_transport.stat('.')
+        # and the dir which has been initialized for us must exist.
+        found_transport.list_dir('.')
 
     def test_root_transport(self):
         dir = self.make_bzrdir('.')
@@ -1171,12 +1226,12 @@
         dir.needs_format_conversion(None)
 
     def test_upgrade_new_instance(self):
-        """Does an available updater work ?."""
+        """Does an available updater work?"""
         dir = self.make_bzrdir('.')
-        # for now, check is not ready for partial bzrdirs.
+        # for now, upgrade is not ready for partial bzrdirs.
         dir.create_repository()
         dir.create_branch()
-        dir.create_workingtree()
+        self.createWorkingTreeOrSkip(dir)
         if dir.can_convert_format():
             # if its default updatable there must be an updater 
             # (we change the default to match the lastest known format

=== modified file 'bzrlib/tests/repository_implementations/__init__.py'
--- bzrlib/tests/repository_implementations/__init__.py	2006-08-11 10:03:44 +0000
+++ bzrlib/tests/repository_implementations/__init__.py	2006-08-14 15:07:33 +0000
@@ -45,7 +45,7 @@
         'bzrlib.tests.repository_implementations.test_fileid_involved',
         'bzrlib.tests.repository_implementations.test_reconcile',
         'bzrlib.tests.repository_implementations.test_repository',
-        'bzrlib.tests.repository_implementations.test_revprops',
+        'bzrlib.tests.repository_implementations.test_revision',
         ]
     adapter = RepositoryTestProviderAdapter(
         default_transport,

=== modified file 'bzrlib/tests/repository_implementations/test_commit_builder.py'
--- bzrlib/tests/repository_implementations/test_commit_builder.py	2006-08-15 03:09:14 +0000
+++ bzrlib/tests/repository_implementations/test_commit_builder.py	2006-08-16 21:10:33 +0000
@@ -67,6 +67,19 @@
         # but thats all the current contract guarantees anyway.
         self.assertEqual('foo', tree.branch.repository.get_inventory('foo').revision_id)
 
+    def test_commit_without_root(self):
+        """This should cause a deprecation warning, not an assertion failure"""
+        tree = self.make_branch_and_tree(".")
+        self.build_tree(['foo'])
+        tree.add('foo', 'foo-id')
+        entry = tree.inventory['foo-id']
+        builder = tree.branch.get_commit_builder([])
+        self.assertDeprecated(['Root entry should be supplied to'
+            ' record_entry_contents, as of bzr 0.10.'], 
+            builder.record_entry_contents, entry, [], 'foo', tree)
+        builder.finish_inventory()
+        rev_id = builder.commit('foo bar')
+
     def test_commit(self):
         tree = self.make_branch_and_tree(".")
         builder = tree.branch.get_commit_builder([])

=== renamed file 'bzrlib/tests/repository_implementations/test_revprops.py' => 'bzrlib/tests/repository_implementations/test_revision.py'
--- bzrlib/tests/repository_implementations/test_revprops.py	2006-07-31 15:46:16 +0000
+++ bzrlib/tests/repository_implementations/test_revision.py	2006-08-15 17:01:22 +0000
@@ -57,3 +57,52 @@
                           wt.commit, 
                           message='invalid',
                           revprops=dict(number=13))
+
+
+class TestRevisionAttributes(TestCaseWithRepository):
+    """Test that revision attributes are correct."""
+
+    def test_revision_accessors(self):
+        """Make sure the values that come out of a revision are the 
+        same as the ones that go in.
+        """
+        tree1 = self.make_branch_and_tree("br1")
+
+        # create a revision
+        tree1.commit(message="quux", allow_pointless=True, committer="jaq",
+                     revprops={'empty':'',
+                               'value':'one',
+                               'unicode':'\xb5',
+                               'multiline':'foo\nbar\n\n'
+                              })
+        assert len(tree1.branch.revision_history()) > 0
+        rev_a = tree1.branch.repository.get_revision(
+                            tree1.branch.last_revision())
+
+        tree2 = self.make_branch_and_tree("br2")
+        tree2.commit(message=rev_a.message,
+                     timestamp=rev_a.timestamp,
+                     timezone=rev_a.timezone,
+                     committer=rev_a.committer,
+                     rev_id=rev_a.revision_id,
+                     revprops=rev_a.properties,
+                     allow_pointless=True, # there's nothing in this commit
+                     strict=True,
+                     verbose=True)
+        rev_b = tree2.branch.repository.get_revision(
+                            tree2.branch.last_revision())
+        
+        self.assertEqual(rev_a.message, rev_b.message)
+        self.assertEqual(rev_a.timestamp, rev_b.timestamp)
+        self.assertEqual(rev_a.timezone, rev_b.timezone)
+        self.assertEqual(rev_a.committer, rev_b.committer)
+        self.assertEqual(rev_a.revision_id, rev_b.revision_id)
+        self.assertEqual(rev_a.properties, rev_b.properties)
+
+    def test_zero_timezone(self):
+        tree1 = self.make_branch_and_tree("br1")
+
+        # create a revision
+        tree1.commit(message="quux", timezone=0, rev_id='r1')
+        rev_a = tree1.branch.repository.get_revision('r1')
+        self.assertEqual(0, rev_a.timezone)

=== modified file 'bzrlib/tests/revisionstore_implementations/test_all.py'
--- bzrlib/tests/revisionstore_implementations/test_all.py	2006-07-24 03:28:53 +0000
+++ bzrlib/tests/revisionstore_implementations/test_all.py	2006-08-16 14:10:56 +0000
@@ -46,7 +46,7 @@
 
     def add_sample_rev(self):
         rev = Revision(timestamp=0,
-                       timezone=None,
+                       timezone=0,
                        committer="Foo Bar <foo@example.com>",
                        message="Message",
                        inventory_sha1='aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa',

=== modified file 'bzrlib/tests/test_diff.py'
--- bzrlib/tests/test_diff.py	2006-07-31 19:15:36 +0000
+++ bzrlib/tests/test_diff.py	2006-08-16 16:03:04 +0000
@@ -17,6 +17,7 @@
 import os
 from cStringIO import StringIO
 import errno
+import subprocess
 from tempfile import TemporaryFile
 
 from bzrlib.diff import internal_diff, external_diff, show_diff_trees
@@ -109,9 +110,17 @@
                                      use_stringio=True)
         self.check_patch(lines)
 
-    def test_external_diff_binary(self):
-        lines = external_udiff_lines(['\x00foobar\n'], ['foo\x00bar\n'])
-        self.assertEqual(['Binary files old and new differ\n', '\n'], lines)
+    def test_external_diff_binary_lang_c(self):
+        orig_lang = os.environ.get('LANG')
+        try:
+            os.environ['LANG'] = 'C'
+            lines = external_udiff_lines(['\x00foobar\n'], ['foo\x00bar\n'])
+            self.assertEqual(['Binary files old and new differ\n', '\n'], lines)
+        finally:
+            if orig_lang is None:
+                del os.environ['LANG']
+            else:
+                os.environ['LANG'] = orig_lang
 
     def test_no_external_diff(self):
         """Check that NoDiff is raised when diff is not available"""
@@ -182,6 +191,25 @@
             'internal_diff should return bytestrings')
 
 
+class TestDiffFiles(TestCaseInTempDir):
+
+    def test_external_diff_binary(self):
+        """The output when using external diff should use diff's i18n error"""
+        # Make sure external_diff doesn't fail in the current LANG
+        lines = external_udiff_lines(['\x00foobar\n'], ['foo\x00bar\n'])
+
+        cmd = ['diff', '-u', 'old', 'new']
+        open('old', 'wb').write('\x00foobar\n')
+        open('new', 'wb').write('foo\x00bar\n')
+        pipe = subprocess.Popen(cmd, stdout=subprocess.PIPE,
+                                     stdin=subprocess.PIPE)
+        out, err = pipe.communicate()
+        # Diff returns '2' on Binary files.
+        self.assertEqual(2, pipe.returncode)
+        # We should output whatever diff tells us, plus a trailing newline
+        self.assertEqual(out.splitlines(True) + ['\n'], lines)
+
+
 class TestDiffDates(TestCaseWithTransport):
 
     def setUp(self):

=== modified file 'bzrlib/tests/test_options.py'
--- bzrlib/tests/test_options.py	2006-07-11 21:46:11 +0000
+++ bzrlib/tests/test_options.py	2006-08-16 13:13:39 +0000
@@ -1,8 +1,10 @@
 # Copyright (C) 2005, 2006 Canonical Ltd
 
+from bzrlib.builtins import cmd_commit, cmd_log, cmd_status
+from bzrlib.commands import Command, parse_args
+from bzrlib import errors
+from bzrlib import option
 from bzrlib.tests import TestCase
-from bzrlib.commands import Command, parse_args
-from bzrlib.builtins import cmd_commit, cmd_log, cmd_status
 
 # TODO: might be nice to just parse them into a structured form and test
 # against that, rather than running the whole command.
@@ -28,8 +30,9 @@
     def test_option_help(self):
         """Options have help strings."""
         out, err = self.run_bzr_captured(['commit', '--help'])
-        self.assertContainsRe(out, r'--file.*file containing commit message')
-        self.assertContainsRe(out, r'--help.*-h')
+        self.assertContainsRe(out, r'--file(.|\n)*file containing commit'
+                                   ' message')
+        self.assertContainsRe(out, r'-h.*--help')
 
     def test_option_help_global(self):
         """Global options have help strings."""
@@ -44,12 +47,40 @@
 
     def test_unknown_short_opt(self):
         out, err = self.run_bzr_captured(['help', '-r'], retcode=3)
-        self.assertContainsRe(err, r'unknown option')
+        self.assertContainsRe(err, r'no such option')
 
     def test_allow_dash(self):
         """Test that we can pass a plain '-' as an argument."""
         self.assertEqual((['-'], {}), parse_args(cmd_commit(), ['-']))
 
+    def test_conversion(self):
+        def parse(options, args):
+            parser = option.get_optparser(dict((o.name, o) for o in options))
+            return parser.parse_args(args)
+        options = [option.Option('hello')]
+        opts, args = parse(options, ['--no-hello', '--hello'])
+        self.assertEqual(True, opts.hello)
+        opts, args = parse(options, [])
+        self.assertEqual(option.OptionParser.DEFAULT_VALUE, opts.hello)
+        opts, args = parse(options, ['--hello', '--no-hello'])
+        self.assertEqual(False, opts.hello)
+        options = [option.Option('number', type=int)]
+        opts, args = parse(options, ['--number', '6'])
+        self.assertEqual(6, opts.number)
+        self.assertRaises(errors.BzrCommandError, parse, options, ['--number'])
+        self.assertRaises(errors.BzrCommandError, parse, options, 
+                          ['--no-number'])
+
+    def test_iter_switches(self):
+        opt = option.Option('hello', help='fg')
+        self.assertEqual(list(opt.iter_switches()),
+                         [('hello', None, None, 'fg')])
+        opt = option.Option('hello', help='fg', type=int)
+        self.assertEqual(list(opt.iter_switches()),
+                         [('hello', None, 'ARG', 'fg')])
+        opt = option.Option('hello', help='fg', type=int, argname='gar')
+        self.assertEqual(list(opt.iter_switches()),
+                         [('hello', None, 'GAR', 'fg')])
 
 #     >>> parse_args('log -r 500'.split())
 #     (['log'], {'revision': [<RevisionSpec_int 500>]})

=== modified file 'bzrlib/tests/test_revision.py'
--- bzrlib/tests/test_revision.py	2006-08-10 00:43:37 +0000
+++ bzrlib/tests/test_revision.py	2006-08-15 16:59:36 +0000
@@ -300,40 +300,3 @@
         self.assertEqual({'B':['A'],
                           'A':[]},
                          source.get_revision_graph('B'))
-
-class TestRevisionAttributes(TestCaseWithTransport):
-    """Test that revision attributes are correct."""
-
-    def test_revision_accessors(self):
-        """Make sure the values that come out of a revision are the same as the ones that go in.
-        """
-        tree1 = self.make_branch_and_tree("br1")
-
-        # create a revision
-        tree1.commit(message="quux", allow_pointless=True, committer="jaq",
-                     revprops={'empty':'',
-                               'value':'one',
-                               'unicode':'\xb5',
-                               'multiline':'foo\nbar\n\n'
-                              })
-        assert len(tree1.branch.revision_history()) > 0
-        rev_a = tree1.branch.repository.get_revision(tree1.branch.last_revision())
-
-        tree2 = self.make_branch_and_tree("br2")
-        tree2.commit(message=rev_a.message,
-                     timestamp=rev_a.timestamp,
-                     timezone=rev_a.timezone,
-                     committer=rev_a.committer,
-                     rev_id=rev_a.revision_id,
-                     revprops=rev_a.properties,
-                     allow_pointless=True, # there's nothing in this commit
-                     strict=True,
-                     verbose=True)
-        rev_b = tree2.branch.repository.get_revision(tree2.branch.last_revision())
-        
-        self.assertEqual(rev_a.message, rev_b.message)
-        self.assertEqual(rev_a.timestamp, rev_b.timestamp)
-        self.assertEqual(rev_a.timezone, rev_b.timezone)
-        self.assertEqual(rev_a.committer, rev_b.committer)
-        self.assertEqual(rev_a.revision_id, rev_b.revision_id)
-        self.assertEqual(rev_a.properties, rev_b.properties)

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2006-07-24 11:26:40 +0000
+++ bzrlib/tests/test_selftest.py	2006-08-17 01:31:39 +0000
@@ -36,6 +36,7 @@
                           )
 from bzrlib.tests.TestUtil import _load_module_by_name
 import bzrlib.errors as errors
+from bzrlib import symbol_versioning
 from bzrlib.trace import note
 
 
@@ -576,6 +577,38 @@
         # cheat. Yes, wash thy mouth out with soap.
         self._benchtime = None
 
+    def test_assigned_benchmark_file_stores_date(self):
+        output = StringIO()
+        result = bzrlib.tests._MyResult(self._log_file,
+                                        descriptions=0,
+                                        verbosity=1,
+                                        bench_history=output
+                                        )
+        output_string = output.getvalue()
+        # if you are wondering about the regexp please read the comment in
+        # test_bench_history (bzrlib.tests.test_selftest.TestRunner)
+        self.assertContainsRe(output_string, "--date [0-9.]+ \S")
+
+    def test_benchhistory_records_test_times(self):
+        result_stream = StringIO()
+        result = bzrlib.tests._MyResult(
+            self._log_file,
+            descriptions=0,
+            verbosity=1,
+            bench_history=result_stream
+            )
+
+        # we want profile a call and check that its test duration is recorded
+        # make a new test instance that when run will generate a benchmark
+        example_test_case = TestTestResult("_time_hello_world_encoding")
+        # execute the test, which should succeed and record times
+        example_test_case.run(result)
+        lines = result_stream.getvalue().splitlines()
+        self.assertEqual(2, len(lines))
+        self.assertContainsRe(lines[1],
+            " *[0-9]+ms bzrlib.tests.test_selftest.TestTestResult"
+            "._time_hello_world_encoding")
+ 
     def _time_hello_world_encoding(self):
         """Profile two sleep calls
         
@@ -671,6 +704,21 @@
         result = self.run_test_runner(runner, test)
         self.assertTrue(result.wasSuccessful())
 
+    def test_bench_history(self):
+        import bzrlib.branch
+        import bzrlib.revisionspec
+        test = TestRunner('dummy_test')
+        output = StringIO()
+        runner = TextTestRunner(stream=self._log_file, bench_history=output)
+        result = self.run_test_runner(runner, test)
+        output_string = output.getvalue()
+        # does anyone know a good regexp for revision ids?
+        # here we are using \S instead and checking the revision id afterwards
+        self.assertContainsRe(output_string, "--date [0-9.]+ \S")
+        branch = bzrlib.branch.Branch.open_containing('.')[0]
+        revision_id = bzrlib.revisionspec.RevisionSpec(branch.revno()).in_history(branch).rev_id
+        self.assert_(output_string.rstrip().endswith(revision_id))
+
 
 class TestTestCase(TestCase):
     """Tests that test the core bzrlib TestCase."""
@@ -761,6 +809,17 @@
         self.assertEndsWith('foo', 'oo')
         self.assertRaises(AssertionError, self.assertEndsWith, 'o', 'oo')
 
+    def test_assertDeprecated(self):
+        def testfunc(be_deprecated):
+            if be_deprecated is True:
+                symbol_versioning.warn('i am deprecated', DeprecationWarning, 
+                                       stacklevel=1)
+        self.assertDeprecated(['i am deprecated'], testfunc, True)
+        self.assertDeprecated([], testfunc, False)
+        self.assertDeprecated(['i am deprecated'], testfunc, 
+                              be_deprecated=True)
+        self.assertDeprecated([], testfunc, be_deprecated=False)
+
 
 class TestConvenienceMakers(TestCaseWithTransport):
     """Test for the make_* convenience functions."""

=== modified file 'bzrlib/tests/test_sftp_transport.py'
--- bzrlib/tests/test_sftp_transport.py	2006-07-31 23:04:57 +0000
+++ bzrlib/tests/test_sftp_transport.py	2006-08-18 16:09:14 +0000
@@ -17,12 +17,14 @@
 import os
 import socket
 import threading
+import time
 
 import bzrlib.bzrdir as bzrdir
 import bzrlib.errors as errors
 from bzrlib.osutils import pathjoin, lexists
 from bzrlib.tests import TestCaseWithTransport, TestCase, TestSkipped
 import bzrlib.transport
+import bzrlib.transport.http
 from bzrlib.workingtree import WorkingTree
 
 try:
@@ -32,21 +34,26 @@
     paramiko_loaded = False
 
 
+def set_test_transport_to_sftp(testcase):
+    """A helper to set transports on test case instances."""
+    from bzrlib.transport.sftp import SFTPAbsoluteServer, SFTPHomeDirServer
+    if getattr(testcase, '_get_remote_is_absolute', None) is None:
+        testcase._get_remote_is_absolute = True
+    if testcase._get_remote_is_absolute:
+        testcase.transport_server = SFTPAbsoluteServer
+    else:
+        testcase.transport_server = SFTPHomeDirServer
+    testcase.transport_readonly_server = bzrlib.transport.http.HttpServer
+
+
 class TestCaseWithSFTPServer(TestCaseWithTransport):
     """A test case base class that provides a sftp server on localhost."""
 
     def setUp(self):
-        if not paramiko_loaded:
-            raise TestSkipped('you must have paramiko to run this test')
         super(TestCaseWithSFTPServer, self).setUp()
-        from bzrlib.transport.sftp import SFTPAbsoluteServer, SFTPHomeDirServer
-        if getattr(self, '_get_remote_is_absolute', None) is None:
-            self._get_remote_is_absolute = True
-        if self._get_remote_is_absolute:
-            self.transport_server = SFTPAbsoluteServer
-        else:
-            self.transport_server = SFTPHomeDirServer
-        self.transport_readonly_server = bzrlib.transport.http.HttpServer
+        if not paramiko_loaded:
+            raise TestSkipped('you must have paramiko to run this test')
+        set_test_transport_to_sftp(self) 
 
     def get_transport(self, path=None):
         """Return a transport relative to self._test_root."""
@@ -342,3 +349,99 @@
             raise TestSkipped('Known NameError bug with paramiko-1.6.1')
         self.assertContainsRe(err, 'Connection error')
 
+
+class SFTPLatencyKnob(TestCaseWithSFTPServer):
+    """Test that the testing SFTPServer's latency knob works."""
+
+    def test_latency_knob_slows_transport(self):
+        # change the latency knob to 500ms. We take about 40ms for a 
+        # loopback connection ordinarily.
+        start_time = time.time()
+        self.get_server().add_latency = 0.5
+        transport = self.get_transport()
+        with_latency_knob_time = time.time() - start_time
+        print with_latency_knob_time
+        self.assertTrue(with_latency_knob_time > 0.4)
+
+    def test_default(self):
+        # This test is potentially brittle: under extremely high machine load
+        # it could fail, but that is quite unlikely
+        start_time = time.time()
+        transport = self.get_transport()
+        regular_time = time.time() - start_time
+        self.assertTrue(regular_time < 0.5)
+
+
+class FakeSocket(object):
+    """Fake socket object used to test the SocketDelay wrapper without
+    using a real socket.
+    """
+
+    def __init__(self):
+        self._data = ""
+
+    def send(self, data, flags=0):
+        self._data += data
+        return len(data)
+
+    def sendall(self, data, flags=0):
+        self._data += data
+        return len(data)
+
+    def recv(self, size, flags=0):
+        if size < len(self._data):
+            result = self._data[:size]
+            self._data = self._data[size:]
+            return result
+        else:
+            result = self._data
+            self._data = ""
+            return result
+
+
+class TestSocketDelay(TestCase):
+
+    def setUp(self):
+        TestCase.setUp(self)
+
+    def test_delay(self):
+        from bzrlib.transport.sftp import SocketDelay
+        sending = FakeSocket()
+        receiving = SocketDelay(sending, 0.1, bandwidth=1000000,
+                                really_sleep=False)
+        # check that simulated time is charged only per round-trip:
+        t1 = SocketDelay.simulated_time
+        receiving.send("connect1")
+        self.assertEqual(sending.recv(1024), "connect1")
+        t2 = SocketDelay.simulated_time
+        self.assertAlmostEqual(t2 - t1, 0.1)
+        receiving.send("connect2")
+        self.assertEqual(sending.recv(1024), "connect2")
+        sending.send("hello")
+        self.assertEqual(receiving.recv(1024), "hello")
+        t3 = SocketDelay.simulated_time
+        self.assertAlmostEqual(t3 - t2, 0.1)
+        sending.send("hello")
+        self.assertEqual(receiving.recv(1024), "hello")
+        sending.send("hello")
+        self.assertEqual(receiving.recv(1024), "hello")
+        sending.send("hello")
+        self.assertEqual(receiving.recv(1024), "hello")
+        t4 = SocketDelay.simulated_time
+        self.assertAlmostEqual(t4, t3)
+
+    def test_bandwidth(self):
+        from bzrlib.transport.sftp import SocketDelay
+        sending = FakeSocket()
+        receiving = SocketDelay(sending, 0, bandwidth=8.0/(1024*1024),
+                                really_sleep=False)
+        # check that simulated time is charged only per round-trip:
+        t1 = SocketDelay.simulated_time
+        receiving.send("connect")
+        self.assertEqual(sending.recv(1024), "connect")
+        sending.send("a" * 100)
+        self.assertEqual(receiving.recv(1024), "a" * 100)
+        t2 = SocketDelay.simulated_time
+        self.assertAlmostEqual(t2 - t1, 100 + 7)
+
+

=== modified file 'bzrlib/tests/test_testament.py'
--- bzrlib/tests/test_testament.py	2006-07-31 16:05:20 +0000
+++ bzrlib/tests/test_testament.py	2006-08-16 16:55:54 +0000
@@ -20,20 +20,17 @@
 
 import os
 from sha import sha
-import sys
 
 from bzrlib.tests import TestCaseWithTransport
-from bzrlib.branch import Branch
 from bzrlib.testament import Testament, StrictTestament
-from bzrlib.trace import mutter
 from bzrlib.transform import TreeTransform
 from bzrlib.osutils import has_symlinks
 
 
-class TestamentTests(TestCaseWithTransport):
+class TestamentSetup(TestCaseWithTransport):
 
     def setUp(self):
-        super(TestamentTests, self).setUp()
+        super(TestamentSetup, self).setUp()
         self.wt = self.make_branch_and_tree('.')
         b = self.b = self.wt.branch
         b.nick = "test branch"
@@ -57,6 +54,9 @@
                  rev_id='test@user-2',
                  committer='test@user')
 
+
+class TestamentTests(TestamentSetup):
+
     def test_null_testament(self):
         """Testament for a revision with no contents."""
         t = Testament.from_revision(self.b.repository, 'test@user-1')
@@ -100,24 +100,6 @@
         actual_short = t.as_short_text()
         self.assertEqualDiff(actual_short, REV_2_SHORT_STRICT)
 
-    def test_testament_command(self):
-        """Testament containing a file and a directory."""
-        out, err = self.run_bzr_captured(['testament', '--long'])
-        self.assertEqualDiff(err, '')
-        self.assertEqualDiff(out, REV_2_TESTAMENT)
-
-    def test_testament_command_2(self):
-        """Command getting short testament of previous version."""
-        out, err = self.run_bzr_captured(['testament', '-r1'])
-        self.assertEqualDiff(err, '')
-        self.assertEqualDiff(out, REV_1_SHORT)
-
-    def test_testament_command_3(self):
-        """Command getting short testament of previous version."""
-        out, err = self.run_bzr_captured(['testament', '-r1', '--strict'])
-        self.assertEqualDiff(err, '')
-        self.assertEqualDiff(out, REV_1_SHORT_STRICT)
-
     def test_testament_symlinks(self):
         """Testament containing symlink (where possible)"""
         if not has_symlinks():
@@ -153,7 +135,9 @@
             timestamp=1129025493,
             timezone=36000,
             rev_id='test@user-3',
-            committer='test@user')
+            committer='Erik B\xe5gfors <test@user>',
+            revprops={'uni':u'\xb5'}
+            )
         t = Testament.from_revision(self.b.repository, 'test@user-3')
         self.assertEqualDiff(
             SAMPLE_UNICODE_TESTAMENT.encode('utf-8'), t.as_text())
@@ -316,7 +300,7 @@
 SAMPLE_UNICODE_TESTAMENT = u"""\
 bazaar-ng testament version 1
 revision-id: test@user-3
-committer: test@user
+committer: Erik B\xe5gfors <test@user>
 timestamp: 1129025493
 timezone: 36000
 parents:
@@ -330,4 +314,6 @@
 properties:
   branch-nick:
     test branch
+  uni:
+    \xb5
 """

=== added file 'bzrlib/tests/test_version.py'
--- bzrlib/tests/test_version.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_version.py	2006-08-16 02:53:40 +0000
@@ -0,0 +1,35 @@
+# Copyright (C) 2004, 2005, 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests for versioning of bzrlib."""
+
+from bzrlib import version
+from bzrlib.tests import TestCase, TestSkipped
+
+class TestBzrlibVersioning(TestCase):
+
+    def test_get_bzr_source_tree(self):
+        """Get tree for bzr source, if any."""
+        # We don't know if these tests are being run from a checkout or branch
+        # of bzr, from an installed copy, or from source unpacked from a
+        # tarball.  We don't construct a branch just for testing this, so we
+        # just assert that it must either return None or the tree.
+        src_tree = version._get_bzr_source_tree()
+        if src_tree is None:
+            raise TestSkipped("bzr tests aren't run from a bzr working tree")
+        else:
+            source_version = src_tree.last_revision()

=== modified file 'bzrlib/tests/test_xml.py'
--- bzrlib/tests/test_xml.py	2006-07-31 16:29:15 +0000
+++ bzrlib/tests/test_xml.py	2006-08-16 20:55:23 +0000
@@ -57,6 +57,21 @@
 </revision>
 """
 
+_revision_v5_utc = """\
+<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
+    inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
+    revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    timestamp="1125907235.211783886"
+    timezone="0">
+<message>- start splitting code for xml (de)serialization away from objects
+  preparatory to supporting multiple formats by a single library
+</message>
+<parents>
+<revision_ref revision_id="mbp@sourcefrog.net-20050905063503-43948f59fa127d92"/>
+</parents>
+</revision>
+"""
+
 _committed_inv_v5 = """<inventory>
 <file file_id="bar-20050901064931-73b4b1138abc9cd2" 
       name="bar" parent_id="TREE_ROOT" 
@@ -65,7 +80,7 @@
            file_id="foo-20050801201819-4139aa4a272f4250"
            parent_id="TREE_ROOT" 
            revision="mbp@foo-00"/>
-<file file_id="bar-20050824000535-6bc48cfad47ed134" 
+<file executable="yes" file_id="bar-20050824000535-6bc48cfad47ed134" 
       name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" 
       revision="mbp@foo-00"/>
 </inventory>
@@ -86,6 +101,35 @@
 """
 
 
+# DO NOT REFLOW THIS. Its the exact revision we want.
+_expected_rev_v5 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;" format="5" inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41" revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9" timestamp="1125907235.211783886" timezone="36000">
+<message>- start splitting code for xml (de)serialization away from objects
+  preparatory to supporting multiple formats by a single library
+</message>
+<parents>
+<revision_ref revision_id="mbp@sourcefrog.net-20050905063503-43948f59fa127d92" />
+</parents>
+</revision>
+"""
+
+
+# DO NOT REFLOW THIS. Its the exact inventory we want.
+_expected_inv_v5 = """<inventory format="5">
+<file file_id="bar-20050901064931-73b4b1138abc9cd2" name="bar" revision="mbp@foo-123123" />
+<directory file_id="foo-20050801201819-4139aa4a272f4250" name="subdir" revision="mbp@foo-00" />
+<file executable="yes" file_id="bar-20050824000535-6bc48cfad47ed134" name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" revision="mbp@foo-00" />
+</inventory>
+"""
+
+
+_expected_inv_v5_root = """<inventory file_id="f&lt;" format="5" revision_id="mother!">
+<file file_id="bar-20050901064931-73b4b1138abc9cd2" name="bar" parent_id="f&lt;" revision="mbp@foo-123123" />
+<directory file_id="foo-20050801201819-4139aa4a272f4250" name="subdir" parent_id="f&lt;" revision="mbp@foo-00" />
+<file executable="yes" file_id="bar-20050824000535-6bc48cfad47ed134" name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" revision="mbp@foo-00" />
+</inventory>
+"""
+
+
 class TestSerializer(TestCase):
     """Test XML serialization"""
     def test_canned_inventory(self):
@@ -120,6 +164,17 @@
         eq(rev.parent_ids[0],
            "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
 
+    def test_unpack_revision_5_utc(self):
+        inp = StringIO(_revision_v5_utc)
+        rev = bzrlib.xml5.serializer_v5.read_revision(inp)
+        eq = self.assertEqual
+        eq(rev.committer,
+           "Martin Pool <mbp@sourcefrog.net>")
+        eq(len(rev.parent_ids), 1)
+        eq(rev.timezone, 0)
+        eq(rev.parent_ids[0],
+           "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
+
     def test_unpack_inventory_5(self):
         """Unpack canned new-style inventory"""
         inp = StringIO(_committed_inv_v5)
@@ -150,18 +205,38 @@
         inv = bzrlib.xml5.serializer_v5.read_inventory(inp)
         outp = StringIO()
         bzrlib.xml5.serializer_v5.write_inventory(inv, outp)
-        inv2 = bzrlib.xml5.serializer_v5.read_inventory(StringIO(outp.getvalue()))
-        self.assertEqual(inv, inv2)
+        self.assertEqualDiff(_expected_inv_v5, outp.getvalue())
+        inv2 = bzrlib.xml5.serializer_v5.read_inventory(StringIO(outp.getvalue()))
+        self.assertEqual(inv, inv2)
+    
+    def assertRoundTrips(self, xml_string):
+        inp = StringIO(xml_string)
+        inv = bzrlib.xml5.serializer_v5.read_inventory(inp)
+        outp = StringIO()
+        bzrlib.xml5.serializer_v5.write_inventory(inv, outp)
+        self.assertEqualDiff(xml_string, outp.getvalue())
+        inv2 = bzrlib.xml5.serializer_v5.read_inventory(StringIO(outp.getvalue()))
+        self.assertEqual(inv, inv2)
+
+    def tests_serialize_inventory_v5_with_root(self):
+        self.assertRoundTrips(_expected_inv_v5_root)
+
+    def check_repack_revision(self, txt):
+        """Check that repacking a revision yields the same information"""
+        inp = StringIO(txt)
+        rev = bzrlib.xml5.serializer_v5.read_revision(inp)
+        outp = StringIO()
+        bzrlib.xml5.serializer_v5.write_revision(rev, outp)
+        outfile_contents = outp.getvalue()
+        rev2 = bzrlib.xml5.serializer_v5.read_revision(StringIO(outfile_contents))
+        self.assertEqual(rev, rev2)
 
     def test_repack_revision_5(self):
         """Round-trip revision to XML v5"""
-        inp = StringIO(_revision_v5)
-        rev = bzrlib.xml5.serializer_v5.read_revision(inp)
-        outp = StringIO()
-        bzrlib.xml5.serializer_v5.write_revision(rev, outp)
-        outfile_contents = outp.getvalue()
-        rev2 = bzrlib.xml5.serializer_v5.read_revision(StringIO(outfile_contents))
-        self.assertEqual(rev, rev2)
+        self.check_repack_revision(_revision_v5)
+
+    def test_repack_revision_5_utc(self):
+        self.check_repack_revision(_revision_v5_utc)
 
     def test_pack_revision_5(self):
         """Pack revision to XML v5"""
@@ -172,6 +247,7 @@
         outfile_contents = outp.getvalue()
         self.assertEqual(outfile_contents[-1], '\n')
         self.assertEqualDiff(outfile_contents, bzrlib.xml5.serializer_v5.write_revision_to_string(rev))
+        self.assertEqualDiff(outfile_contents, _expected_rev_v5)
 
     def test_empty_property_value(self):
         """Create an empty property value check that it serializes correctly"""

=== modified file 'bzrlib/tests/workingtree_implementations/test_parents.py'
--- bzrlib/tests/workingtree_implementations/test_parents.py	2006-08-17 04:29:31 +0000
+++ bzrlib/tests/workingtree_implementations/test_parents.py	2006-08-22 05:52:51 +0000
@@ -47,7 +47,7 @@
 
     def test_set_one_ghost_parent_rejects(self):
         t = self.make_branch_and_tree('.')
-        self.assertRaises(errors.GhostRevision,
+        self.assertRaises(errors.GhostRevisionUnusableHere,
             t.set_parent_trees, [('missing-revision-id', None)])
 
     def test_set_one_ghost_parent_force(self):
@@ -94,7 +94,7 @@
 
     def test_set_one_ghost_parent_ids_rejects(self):
         t = self.make_branch_and_tree('.')
-        self.assertRaises(errors.GhostRevision,
+        self.assertRaises(errors.GhostRevisionUnusableHere,
             t.set_parent_ids, ['missing-revision-id'])
 
     def test_set_one_ghost_parent_ids_force(self):
@@ -141,7 +141,7 @@
     def test_add_first_parent_id_ghost_rejects(self):
         """Test adding the first parent id - as a ghost"""
         tree = self.make_branch_and_tree('.')
-        self.assertRaises(errors.GhostRevision,
+        self.assertRaises(errors.GhostRevisionUnusableHere,
             tree.add_parent_tree_id, 'first-revision')
         
     def test_add_first_parent_id_ghost_force(self):
@@ -178,7 +178,7 @@
     def test_add_first_parent_tree_ghost_rejects(self):
         """Test adding the first parent id - as a ghost"""
         tree = self.make_branch_and_tree('.')
-        self.assertRaises(errors.GhostRevision,
+        self.assertRaises(errors.GhostRevisionUnusableHere,
             tree.add_parent_tree, ('first-revision', None))
         
     def test_add_first_parent_tree_ghost_force(self):

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2006-07-31 23:04:57 +0000
+++ bzrlib/transport/sftp.py	2006-08-17 11:42:33 +0000
@@ -311,8 +311,8 @@
             pass
 
 
-class SFTPTransport (Transport):
-    """Transport implementation for SFTP access"""
+class SFTPTransport(Transport):
+    """Transport implementation for SFTP access."""
 
     _do_prefetch = _default_do_prefetch
     # TODO: jam 20060717 Conceivably these could be configurable, either
@@ -1065,6 +1065,81 @@
                         x)
 
 
+class SocketDelay(object):
+    """A socket decorator to make TCP appear slower.
+
+    This changes recv, send, and sendall to add a fixed latency to each python
+    call if a new roundtrip is detected. That is, when a recv is called and the
+    flag new_roundtrip is set, latency is charged. Every send and send_all
+    sets this flag.
+
+    In addition every send, sendall and recv sleeps a bit per character send to
+    simulate bandwidth.
+
+    Not all methods are implemented, this is deliberate as this class is not a
+    replacement for the builtin sockets layer. fileno is not implemented to
+    prevent the proxy being bypassed. 
+    """
+
+    simulated_time = 0
+    _proxied_arguments = dict.fromkeys([
+        "close", "getpeername", "getsockname", "getsockopt", "gettimeout",
+        "setblocking", "setsockopt", "settimeout", "shutdown"])
+
+    def __init__(self, sock, latency, bandwidth=1.0, 
+                 really_sleep=True):
+        """ 
+        :param bandwith: simulated bandwith (MegaBit)
+        :param really_sleep: If set to false, the SocketDelay will just
+        increase a counter, instead of calling time.sleep. This is useful for
+        unittesting the SocketDelay.
+        """
+        self.sock = sock
+        self.latency = latency
+        self.really_sleep = really_sleep
+        self.time_per_byte = 1 / (bandwidth / 8.0 * 1024 * 1024) 
+        self.new_roundtrip = False
+
+    def sleep(self, s):
+        if self.really_sleep:
+            time.sleep(s)
+        else:
+            SocketDelay.simulated_time += s
+
+    def __getattr__(self, attr):
+        if attr in SocketDelay._proxied_arguments:
+            return getattr(self.sock, attr)
+        raise AttributeError("'SocketDelay' object has no attribute %r" %
+                             attr)
+
+    def dup(self):
+        return SocketDelay(self.sock.dup(), self.latency, self.time_per_byte,
+                           self._sleep)
+
+    def recv(self, *args):
+        data = self.sock.recv(*args)
+        if data and self.new_roundtrip:
+            self.new_roundtrip = False
+            self.sleep(self.latency)
+        self.sleep(len(data) * self.time_per_byte)
+        return data
+
+    def sendall(self, data, flags=0):
+        if not self.new_roundtrip:
+            self.new_roundtrip = True
+            self.sleep(self.latency)
+        self.sleep(len(data) * self.time_per_byte)
+        return self.sock.sendall(data, flags)
+
+    def send(self, data, flags=0):
+        if not self.new_roundtrip:
+            self.new_roundtrip = True
+            self.sleep(self.latency)
+        bytes_sent = self.sock.send(data, flags)
+        self.sleep(bytes_sent * self.time_per_byte)
+        return bytes_sent
+
+
 class SFTPServer(Server):
     """Common code for SFTP server facilities."""
 
@@ -1077,6 +1152,7 @@
         self._vendor = 'none'
         # sftp server logs
         self.logs = []
+        self.add_latency = 0
 
     def _get_sftp_url(self, path):
         """Calculate an sftp url to this server for path."""
@@ -1086,6 +1162,16 @@
         """StubServer uses this to log when a new server is created."""
         self.logs.append(message)
 
+    def _run_server_entry(self, sock):
+        """Entry point for all implementations of _run_server.
+        
+        If self.add_latency is > 0.000001 then sock is given a latency adding
+        decorator.
+        """
+        if self.add_latency > 0.000001:
+            sock = SocketDelay(sock, self.add_latency)
+        return self._run_server(sock)
+
     def _run_server(self, s):
         ssh_server = paramiko.Transport(s)
         key_file = pathjoin(self._homedir, 'test_rsa.key')
@@ -1117,7 +1203,7 @@
         self._root = '/'
         if sys.platform == 'win32':
             self._root = ''
-        self._listener = SocketListener(self._run_server)
+        self._listener = SocketListener(self._run_server_entry)
         self._listener.setDaemon(True)
         self._listener.start()
 

=== modified file 'bzrlib/tuned_gzip.py'
--- bzrlib/tuned_gzip.py	2006-06-12 01:53:19 +0000
+++ bzrlib/tuned_gzip.py	2006-08-17 13:25:05 +0000
@@ -17,6 +17,8 @@
 
 """Bzrlib specific gzip tunings. We plan to feed these to the upstream gzip."""
 
+from cStringIO import StringIO
+
 # make GzipFile faster:
 import gzip
 from gzip import U32, LOWU32, FEXTRA, FCOMMENT, FNAME, FHCRC
@@ -63,6 +65,30 @@
         self.extrasize += len_data
         self.size += len_data
 
+    def _write_gzip_header(self):
+        """A tuned version of gzip._write_gzip_header
+
+        We have some extra constrains that plain Gzip does not.
+        1) We want to write the whole blob at once. rather than multiple 
+           calls to fileobj.write().
+        2) We never have a filename
+        3) We don't care about the time
+        """
+        self.fileobj.write(
+           '\037\213'   # self.fileobj.write('\037\213')  # magic header
+            '\010'      # self.fileobj.write('\010')      # compression method
+                        # fname = self.filename[:-3]
+                        # flags = 0
+                        # if fname:
+                        #     flags = FNAME
+            '\x00'      # self.fileobj.write(chr(flags))
+            '\0\0\0\0'  # write32u(self.fileobj, long(time.time()))
+            '\002'      # self.fileobj.write('\002')
+            '\377'      # self.fileobj.write('\377')
+                        # if fname:
+            ''          #     self.fileobj.write(fname + '\000')
+            )
+
     def _read(self, size=1024):
         # various optimisations:
         # reduces lsprof count from 2500 to 
@@ -255,10 +281,16 @@
         # to :
         # 4168 calls in 417.
         # Negative numbers result in reading all the lines
-        if sizehint <= 0:
-            sizehint = -1
-        content = self.read(sizehint)
-        return bzrlib.osutils.split_lines(content)
+        
+        # python's gzip routine uses sizehint. This is a more efficient way
+        # than python uses to honor it. But it is even more efficient to
+        # just read the entire thing and use cStringIO to split into lines.
+        # if sizehint <= 0:
+        #     sizehint = -1
+        # content = self.read(sizehint)
+        # return bzrlib.osutils.split_lines(content)
+        content = StringIO(self.read(-1))
+        return content.readlines()
 
     def _unread(self, buf, len_buf=None):
         """tuned to remove unneeded len calls.

=== removed directory 'bzrlib/util/urlgrabber'
=== removed file 'bzrlib/util/urlgrabber/__init__.py'
--- bzrlib/util/urlgrabber/__init__.py	2005-09-19 06:05:19 +0000
+++ bzrlib/util/urlgrabber/__init__.py	1970-01-01 00:00:00 +0000
@@ -1,53 +0,0 @@
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Library General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-
-# Copyright 2002-2004 Michael D. Stenner, Ryan Tomayko
-
-# $Id: __init__.py,v 1.14 2005/03/08 22:54:25 mstenner Exp $
-
-"""A high-level cross-protocol url-grabber.
-
-Using urlgrabber, data can be fetched in three basic ways:
-
-  urlgrab(url) copy the file to the local filesystem
-  urlopen(url) open the remote file and return a file object
-     (like urllib2.urlopen)
-  urlread(url) return the contents of the file as a string
-
-When using these functions (or methods), urlgrabber supports the
-following features:
-
-  * identical behavior for http://, ftp://, and file:// urls
-  * http keepalive - faster downloads of many files by using
-    only a single connection
-  * byte ranges - fetch only a portion of the file
-  * reget - for a urlgrab, resume a partial download
-  * progress meters - the ability to report download progress
-    automatically, even when using urlopen!
-  * throttling - restrict bandwidth usage
-  * retries - automatically retry a download if it fails. The
-    number of retries and failure types are configurable.
-  * authenticated server access for http and ftp
-  * proxy support - support for authenticated http and ftp proxies
-  * mirror groups - treat a list of mirrors as a single source,
-    automatically switching mirrors if there is a failure.
-"""
-
-__version__ = '2.9.6'
-__date__    = '2005/03/08'
-__author__  = 'Michael D. Stenner <mstenner@linux.duke.edu>, ' \
-              'Ryan Tomayko <rtomayko@naeblis.cx>'
-__url__     = 'http://linux.duke.edu/projects/urlgrabber/'
-
-from grabber import urlgrab, urlopen, urlread

=== removed file 'bzrlib/util/urlgrabber/byterange.py'
--- bzrlib/util/urlgrabber/byterange.py	2005-09-19 06:05:19 +0000
+++ bzrlib/util/urlgrabber/byterange.py	1970-01-01 00:00:00 +0000
@@ -1,452 +0,0 @@
-#   This library is free software; you can redistribute it and/or
-#   modify it under the terms of the GNU Lesser General Public
-#   License as published by the Free Software Foundation; either
-#   version 2.1 of the License, or (at your option) any later version.
-#
-#   This library is distributed in the hope that it will be useful,
-#   but WITHOUT ANY WARRANTY; without even the implied warranty of
-#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#   Lesser General Public License for more details.
-#
-#   You should have received a copy of the GNU Lesser General Public
-#   License along with this library; if not, write to the 
-#      Free Software Foundation, Inc., 
-#      59 Temple Place, Suite 330, 
-#      Boston, MA  02111-1307  USA
-
-# This file is part of urlgrabber, a high-level cross-protocol url-grabber
-# Copyright 2002-2004 Michael D. Stenner, Ryan Tomayko
-
-# $Id: byterange.py,v 1.9 2005/02/14 21:55:07 mstenner Exp $
-
-import os
-import stat
-import urllib
-import urllib2
-import rfc822
-
-try:    
-    from cStringIO import StringIO
-except ImportError, msg: 
-    from StringIO import StringIO
-
-class RangeError(IOError):
-    """Error raised when an unsatisfiable range is requested."""
-    pass
-    
-class HTTPRangeHandler(urllib2.BaseHandler):
-    """Handler that enables HTTP Range headers.
-    
-    This was extremely simple. The Range header is a HTTP feature to
-    begin with so all this class does is tell urllib2 that the 
-    "206 Partial Content" reponse from the HTTP server is what we 
-    expected.
-    
-    Example:
-        import urllib2
-        import byterange
-        
-        range_handler = range.HTTPRangeHandler()
-        opener = urllib2.build_opener(range_handler)
-        
-        # install it
-        urllib2.install_opener(opener)
-        
-        # create Request and set Range header
-        req = urllib2.Request('http://www.python.org/')
-        req.header['Range'] = 'bytes=30-50'
-        f = urllib2.urlopen(req)
-    """
-    
-    def http_error_206(self, req, fp, code, msg, hdrs):
-        # 206 Partial Content Response
-        r = urllib.addinfourl(fp, hdrs, req.get_full_url())
-        r.code = code
-        r.msg = msg
-        return r
-    
-    def http_error_416(self, req, fp, code, msg, hdrs):
-        # HTTP's Range Not Satisfiable error
-        raise RangeError('Requested Range Not Satisfiable')
-
-class RangeableFileObject:
-    """File object wrapper to enable raw range handling.
-    This was implemented primarilary for handling range 
-    specifications for file:// urls. This object effectively makes 
-    a file object look like it consists only of a range of bytes in 
-    the stream.
-    
-    Examples:
-        # expose 10 bytes, starting at byte position 20, from 
-        # /etc/aliases.
-        >>> fo = RangeableFileObject(file('/etc/passwd', 'r'), (20,30))
-        # seek seeks within the range (to position 23 in this case)
-        >>> fo.seek(3)
-        # tell tells where your at _within the range_ (position 3 in
-        # this case)
-        >>> fo.tell()
-        # read EOFs if an attempt is made to read past the last
-        # byte in the range. the following will return only 7 bytes.
-        >>> fo.read(30)
-    """
-    
-    def __init__(self, fo, rangetup):
-        """Create a RangeableFileObject.
-        fo       -- a file like object. only the read() method need be 
-                    supported but supporting an optimized seek() is 
-                    preferable.
-        rangetup -- a (firstbyte,lastbyte) tuple specifying the range
-                    to work over.
-        The file object provided is assumed to be at byte offset 0.
-        """
-        self.fo = fo
-        (self.firstbyte, self.lastbyte) = range_tuple_normalize(rangetup)
-        self.realpos = 0
-        self._do_seek(self.firstbyte)
-        
-    def __getattr__(self, name):
-        """This effectively allows us to wrap at the instance level.
-        Any attribute not found in _this_ object will be searched for
-        in self.fo.  This includes methods."""
-        if hasattr(self.fo, name):
-            return getattr(self.fo, name)
-        raise AttributeError, name
-    
-    def tell(self):
-        """Return the position within the range.
-        This is different from fo.seek in that position 0 is the 
-        first byte position of the range tuple. For example, if
-        this object was created with a range tuple of (500,899),
-        tell() will return 0 when at byte position 500 of the file.
-        """
-        return (self.realpos - self.firstbyte)
-    
-    def seek(self,offset,whence=0):
-        """Seek within the byte range.
-        Positioning is identical to that described under tell().
-        """
-        assert whence in (0, 1, 2)
-        if whence == 0:   # absolute seek
-            realoffset = self.firstbyte + offset
-        elif whence == 1: # relative seek
-            realoffset = self.realpos + offset
-        elif whence == 2: # absolute from end of file
-            # XXX: are we raising the right Error here?
-            raise IOError('seek from end of file not supported.')
-        
-        # do not allow seek past lastbyte in range
-        if self.lastbyte and (realoffset >= self.lastbyte):
-            realoffset = self.lastbyte
-        
-        self._do_seek(realoffset - self.realpos)
-        
-    def read(self, size=-1):
-        """Read within the range.
-        This method will limit the size read based on the range.
-        """
-        size = self._calc_read_size(size)
-        rslt = self.fo.read(size)
-        self.realpos += len(rslt)
-        return rslt
-    
-    def readline(self, size=-1):
-        """Read lines within the range.
-        This method will limit the size read based on the range.
-        """
-        size = self._calc_read_size(size)
-        rslt = self.fo.readline(size)
-        self.realpos += len(rslt)
-        return rslt
-    
-    def _calc_read_size(self, size):
-        """Handles calculating the amount of data to read based on
-        the range.
-        """
-        if self.lastbyte:
-            if size > -1:
-                if ((self.realpos + size) >= self.lastbyte):
-                    size = (self.lastbyte - self.realpos)
-            else:
-                size = (self.lastbyte - self.realpos)
-        return size
-        
-    def _do_seek(self,offset):
-        """Seek based on whether wrapped object supports seek().
-        offset is relative to the current position (self.realpos).
-        """
-        assert offset >= 0
-        if not hasattr(self.fo, 'seek'):
-            self._poor_mans_seek(offset)
-        else:
-            self.fo.seek(self.realpos + offset)
-        self.realpos+= offset
-        
-    def _poor_mans_seek(self,offset):
-        """Seek by calling the wrapped file objects read() method.
-        This is used for file like objects that do not have native
-        seek support. The wrapped objects read() method is called
-        to manually seek to the desired position.
-        offset -- read this number of bytes from the wrapped
-                  file object.
-        raise RangeError if we encounter EOF before reaching the 
-        specified offset.
-        """
-        pos = 0
-        bufsize = 1024
-        while pos < offset:
-            if (pos + bufsize) > offset:
-                bufsize = offset - pos
-            buf = self.fo.read(bufsize)
-            if len(buf) != bufsize:
-                raise RangeError('Requested Range Not Satisfiable')
-            pos+= bufsize
-
-class FileRangeHandler(urllib2.FileHandler):
-    """FileHandler subclass that adds Range support.
-    This class handles Range headers exactly like an HTTP
-    server would.
-    """
-    def open_local_file(self, req):
-        import mimetypes
-        import mimetools
-        host = req.get_host()
-        file = req.get_selector()
-        localfile = urllib.url2pathname(file)
-        stats = os.stat(localfile)
-        size = stats[stat.ST_SIZE]
-        modified = rfc822.formatdate(stats[stat.ST_MTIME])
-        mtype = mimetypes.guess_type(file)[0]
-        if host:
-            host, port = urllib.splitport(host)
-            if port or socket.gethostbyname(host) not in self.get_names():
-                raise URLError('file not on local host')
-        fo = open(localfile,'rb')
-        brange = req.headers.get('Range',None)
-        brange = range_header_to_tuple(brange)
-        assert brange != ()
-        if brange:
-            (fb,lb) = brange
-            if lb == '': lb = size
-            if fb < 0 or fb > size or lb > size:
-                raise RangeError('Requested Range Not Satisfiable')
-            size = (lb - fb)
-            fo = RangeableFileObject(fo, (fb,lb))
-        headers = mimetools.Message(StringIO(
-            'Content-Type: %s\nContent-Length: %d\nLast-modified: %s\n' %
-            (mtype or 'text/plain', size, modified)))
-        return urllib.addinfourl(fo, headers, 'file:'+file)
-
-
-# FTP Range Support 
-# Unfortunately, a large amount of base FTP code had to be copied
-# from urllib and urllib2 in order to insert the FTP REST command.
-# Code modifications for range support have been commented as 
-# follows:
-# -- range support modifications start/end here
-
-from urllib import splitport, splituser, splitpasswd, splitattr, \
-                   unquote, addclosehook, addinfourl
-import ftplib
-import socket
-import sys
-import ftplib
-import mimetypes
-import mimetools
-
-class FTPRangeHandler(urllib2.FTPHandler):
-    def ftp_open(self, req):
-        host = req.get_host()
-        if not host:
-            raise IOError, ('ftp error', 'no host given')
-        host, port = splitport(host)
-        if port is None:
-            port = ftplib.FTP_PORT
-
-        # username/password handling
-        user, host = splituser(host)
-        if user:
-            user, passwd = splitpasswd(user)
-        else:
-            passwd = None
-        host = unquote(host)
-        user = unquote(user or '')
-        passwd = unquote(passwd or '')
-        
-        try:
-            host = socket.gethostbyname(host)
-        except socket.error, msg:
-            raise URLError(msg)
-        path, attrs = splitattr(req.get_selector())
-        dirs = path.split('/')
-        dirs = map(unquote, dirs)
-        dirs, file = dirs[:-1], dirs[-1]
-        if dirs and not dirs[0]:
-            dirs = dirs[1:]
-        try:
-            fw = self.connect_ftp(user, passwd, host, port, dirs)
-            type = file and 'I' or 'D'
-            for attr in attrs:
-                attr, value = splitattr(attr)
-                if attr.lower() == 'type' and \
-                   value in ('a', 'A', 'i', 'I', 'd', 'D'):
-                    type = value.upper()
-            
-            # -- range support modifications start here
-            rest = None
-            range_tup = range_header_to_tuple(req.headers.get('Range',None))    
-            assert range_tup != ()
-            if range_tup:
-                (fb,lb) = range_tup
-                if fb > 0: rest = fb
-            # -- range support modifications end here
-            
-            fp, retrlen = fw.retrfile(file, type, rest)
-            
-            # -- range support modifications start here
-            if range_tup:
-                (fb,lb) = range_tup
-                if lb == '': 
-                    if retrlen is None or retrlen == 0:
-                        raise RangeError('Requested Range Not Satisfiable due to unobtainable file length.')
-                    lb = retrlen
-                    retrlen = lb - fb
-                    if retrlen < 0:
-                        # beginning of range is larger than file
-                        raise RangeError('Requested Range Not Satisfiable')
-                else:
-                    retrlen = lb - fb
-                    fp = RangeableFileObject(fp, (0,retrlen))
-            # -- range support modifications end here
-            
-            headers = ""
-            mtype = mimetypes.guess_type(req.get_full_url())[0]
-            if mtype:
-                headers += "Content-Type: %s\n" % mtype
-            if retrlen is not None and retrlen >= 0:
-                headers += "Content-Length: %d\n" % retrlen
-            sf = StringIO(headers)
-            headers = mimetools.Message(sf)
-            return addinfourl(fp, headers, req.get_full_url())
-        except ftplib.all_errors, msg:
-            raise IOError, ('ftp error', msg), sys.exc_info()[2]
-
-    def connect_ftp(self, user, passwd, host, port, dirs):
-        fw = ftpwrapper(user, passwd, host, port, dirs)
-        return fw
-
-class ftpwrapper(urllib.ftpwrapper):
-    # range support note:
-    # this ftpwrapper code is copied directly from
-    # urllib. The only enhancement is to add the rest
-    # argument and pass it on to ftp.ntransfercmd
-    def retrfile(self, file, type, rest=None):
-        self.endtransfer()
-        if type in ('d', 'D'): cmd = 'TYPE A'; isdir = 1
-        else: cmd = 'TYPE ' + type; isdir = 0
-        try:
-            self.ftp.voidcmd(cmd)
-        except ftplib.all_errors:
-            self.init()
-            self.ftp.voidcmd(cmd)
-        conn = None
-        if file and not isdir:
-            # Use nlst to see if the file exists at all
-            try:
-                self.ftp.nlst(file)
-            except ftplib.error_perm, reason:
-                raise IOError, ('ftp error', reason), sys.exc_info()[2]
-            # Restore the transfer mode!
-            self.ftp.voidcmd(cmd)
-            # Try to retrieve as a file
-            try:
-                cmd = 'RETR ' + file
-                conn = self.ftp.ntransfercmd(cmd, rest)
-            except ftplib.error_perm, reason:
-                if str(reason)[:3] == '501':
-                    # workaround for REST not supported error
-                    fp, retrlen = self.retrfile(file, type)
-                    fp = RangeableFileObject(fp, (rest,''))
-                    return (fp, retrlen)
-                elif str(reason)[:3] != '550':
-                    raise IOError, ('ftp error', reason), sys.exc_info()[2]
-        if not conn:
-            # Set transfer mode to ASCII!
-            self.ftp.voidcmd('TYPE A')
-            # Try a directory listing
-            if file: cmd = 'LIST ' + file
-            else: cmd = 'LIST'
-            conn = self.ftp.ntransfercmd(cmd)
-        self.busy = 1
-        # Pass back both a suitably decorated object and a retrieval length
-        return (addclosehook(conn[0].makefile('rb'),
-                            self.endtransfer), conn[1])
-
-
-####################################################################
-# Range Tuple Functions
-# XXX: These range tuple functions might go better in a class.
-
-_rangere = None
-def range_header_to_tuple(range_header):
-    """Get a (firstbyte,lastbyte) tuple from a Range header value.
-    
-    Range headers have the form "bytes=<firstbyte>-<lastbyte>". This
-    function pulls the firstbyte and lastbyte values and returns
-    a (firstbyte,lastbyte) tuple. If lastbyte is not specified in
-    the header value, it is returned as an empty string in the
-    tuple.
-    
-    Return None if range_header is None
-    Return () if range_header does not conform to the range spec 
-    pattern.
-    
-    """
-    global _rangere
-    if range_header is None: return None
-    if _rangere is None:
-        import re
-        _rangere = re.compile(r'^bytes=(\d{1,})-(\d*)')
-    match = _rangere.match(range_header)
-    if match: 
-        tup = range_tuple_normalize(match.group(1,2))
-        if tup and tup[1]: 
-            tup = (tup[0],tup[1]+1)
-        return tup
-    return ()
-
-def range_tuple_to_header(range_tup):
-    """Convert a range tuple to a Range header value.
-    Return a string of the form "bytes=<firstbyte>-<lastbyte>" or None
-    if no range is needed.
-    """
-    if range_tup is None: return None
-    range_tup = range_tuple_normalize(range_tup)
-    if range_tup:
-        if range_tup[1]: 
-            range_tup = (range_tup[0],range_tup[1] - 1)
-        return 'bytes=%s-%s' % range_tup
-    
-def range_tuple_normalize(range_tup):
-    """Normalize a (first_byte,last_byte) range tuple.
-    Return a tuple whose first element is guaranteed to be an int
-    and whose second element will be '' (meaning: the last byte) or 
-    an int. Finally, return None if the normalized tuple == (0,'')
-    as that is equivelant to retrieving the entire file.
-    """
-    if range_tup is None: return None
-    # handle first byte
-    fb = range_tup[0]
-    if fb in (None,''): fb = 0
-    else: fb = int(fb)
-    # handle last byte
-    try: lb = range_tup[1]
-    except IndexError: lb = ''
-    else:  
-        if lb is None: lb = ''
-        elif lb != '': lb = int(lb)
-    # check if range is over the entire file
-    if (fb,lb) == (0,''): return None
-    # check that the range is valid
-    if lb < fb: raise RangeError('Invalid byte range: %s-%s' % (fb,lb))
-    return (fb,lb)
-

=== removed file 'bzrlib/util/urlgrabber/grabber.py'
--- bzrlib/util/urlgrabber/grabber.py	2005-09-19 06:05:19 +0000
+++ bzrlib/util/urlgrabber/grabber.py	1970-01-01 00:00:00 +0000
@@ -1,1177 +0,0 @@
-#   This library is free software; you can redistribute it and/or
-#   modify it under the terms of the GNU Lesser General Public
-#   License as published by the Free Software Foundation; either
-#   version 2.1 of the License, or (at your option) any later version.
-#
-#   This library is distributed in the hope that it will be useful,
-#   but WITHOUT ANY WARRANTY; without even the implied warranty of
-#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#   Lesser General Public License for more details.
-#
-#   You should have received a copy of the GNU Lesser General Public
-#   License along with this library; if not, write to the 
-#      Free Software Foundation, Inc., 
-#      59 Temple Place, Suite 330, 
-#      Boston, MA  02111-1307  USA
-
-# This file is part of urlgrabber, a high-level cross-protocol url-grabber
-# Copyright 2002-2004 Michael D. Stenner, Ryan Tomayko
-
-"""A high-level cross-protocol url-grabber.
-
-GENERAL ARGUMENTS (kwargs)
-
-  Where possible, the module-level default is indicated, and legal
-  values are provided.
-
-  copy_local = 0   [0|1]
-
-    ignored except for file:// urls, in which case it specifies
-    whether urlgrab should still make a copy of the file, or simply
-    point to the existing copy. The module level default for this
-    option is 0.
-
-  close_connection = 0   [0|1]
-
-    tells URLGrabber to close the connection after a file has been
-    transfered. This is ignored unless the download happens with the
-    http keepalive handler (keepalive=1).  Otherwise, the connection
-    is left open for further use. The module level default for this
-    option is 0 (keepalive connections will not be closed).
-
-  keepalive = 1   [0|1]
-
-    specifies whether keepalive should be used for HTTP/1.1 servers
-    that support it. The module level default for this option is 1
-    (keepalive is enabled).
-
-  progress_obj = None
-
-    a class instance that supports the following methods:
-      po.start(filename, url, basename, length, text)
-      # length will be None if unknown
-      po.update(read) # read == bytes read so far
-      po.end()
-
-  text = None
-  
-    specifies an alternativ text item in the beginning of the progress
-    bar line. If not given, the basename of the file is used.
-
-  throttle = 1.0
-
-    a number - if it's an int, it's the bytes/second throttle limit.
-    If it's a float, it is first multiplied by bandwidth.  If throttle
-    == 0, throttling is disabled.  If None, the module-level default
-    (which can be set on default_grabber.throttle) is used. See
-    BANDWIDTH THROTTLING for more information.
-
-  timeout = None
-
-    a positive float expressing the number of seconds to wait for socket
-    operations. If the value is None or 0.0, socket operations will block
-    forever. Setting this option causes urlgrabber to call the settimeout
-    method on the Socket object used for the request. See the Python
-    documentation on settimeout for more information.
-    http://www.python.org/doc/current/lib/socket-objects.html
-
-  bandwidth = 0
-
-    the nominal max bandwidth in bytes/second.  If throttle is a float
-    and bandwidth == 0, throttling is disabled.  If None, the
-    module-level default (which can be set on
-    default_grabber.bandwidth) is used. See BANDWIDTH THROTTLING for
-    more information.
-
-  range = None
-
-    a tuple of the form (first_byte, last_byte) describing a byte
-    range to retrieve. Either or both of the values may set to
-    None. If first_byte is None, byte offset 0 is assumed. If
-    last_byte is None, the last byte available is assumed. Note that
-    the range specification is python-like in that (0,10) will yeild
-    the first 10 bytes of the file.
-
-    If set to None, no range will be used.
-    
-  reget = None   [None|'simple'|'check_timestamp']
-
-    whether to attempt to reget a partially-downloaded file.  Reget
-    only applies to .urlgrab and (obviously) only if there is a
-    partially downloaded file.  Reget has two modes:
-
-      'simple' -- the local file will always be trusted.  If there
-        are 100 bytes in the local file, then the download will always
-        begin 100 bytes into the requested file.
-
-      'check_timestamp' -- the timestamp of the server file will be
-        compared to the timestamp of the local file.  ONLY if the
-        local file is newer than or the same age as the server file
-        will reget be used.  If the server file is newer, or the
-        timestamp is not returned, the entire file will be fetched.
-
-    NOTE: urlgrabber can do very little to verify that the partial
-    file on disk is identical to the beginning of the remote file.
-    You may want to either employ a custom "checkfunc" or simply avoid
-    using reget in situations where corruption is a concern.
-
-  user_agent = 'urlgrabber/VERSION'
-
-    a string, usually of the form 'AGENT/VERSION' that is provided to
-    HTTP servers in the User-agent header. The module level default
-    for this option is "urlgrabber/VERSION".
-
-  http_headers = None
-
-    a tuple of 2-tuples, each containing a header and value.  These
-    will be used for http and https requests only.  For example, you
-    can do
-      http_headers = (('Pragma', 'no-cache'),)
-
-  ftp_headers = None
-
-    this is just like http_headers, but will be used for ftp requests.
-
-  proxies = None
-
-    a dictionary that maps protocol schemes to proxy hosts. For
-    example, to use a proxy server on host "foo" port 3128 for http
-    and https URLs:
-      proxies={ 'http' : 'http://foo:3128', 'https' : 'http://foo:3128' }
-    note that proxy authentication information may be provided using
-    normal URL constructs:
-      proxies={ 'http' : 'http://user:host@foo:3128' }
-    Lastly, if proxies is None, the default environment settings will
-    be used.
-
-  prefix = None
-
-    a url prefix that will be prepended to all requested urls.  For
-    example:
-      g = URLGrabber(prefix='http://foo.com/mirror/')
-      g.urlgrab('some/file.txt')
-      ## this will fetch 'http://foo.com/mirror/some/file.txt'
-    This option exists primarily to allow identical behavior to
-    MirrorGroup (and derived) instances.  Note: a '/' will be inserted
-    if necessary, so you cannot specify a prefix that ends with a
-    partial file or directory name.
-
-  opener = None
-  
-    Overrides the default urllib2.OpenerDirector provided to urllib2
-    when making requests.  This option exists so that the urllib2
-    handler chain may be customized.  Note that the range, reget,
-    proxy, and keepalive features require that custom handlers be
-    provided to urllib2 in order to function properly.  If an opener
-    option is provided, no attempt is made by urlgrabber to ensure
-    chain integrity.  You are responsible for ensuring that any
-    extension handlers are present if said features are required.
-    
-RETRY RELATED ARGUMENTS
-
-  retry = None
-
-    the number of times to retry the grab before bailing.  If this is
-    zero, it will retry forever. This was intentional... really, it
-    was :). If this value is not supplied or is supplied but is None
-    retrying does not occur.
-
-  retrycodes = [-1,2,4,5,6,7]
-
-    a sequence of errorcodes (values of e.errno) for which it should
-    retry. See the doc on URLGrabError for more details on
-    this. retrycodes defaults to [-1,2,4,5,6,7] if not specified
-    explicitly.
-
-  checkfunc = None
-
-    a function to do additional checks. This defaults to None, which
-    means no additional checking.  The function should simply return
-    on a successful check.  It should raise URLGrabError on an
-    unsuccessful check.  Raising of any other exception will be
-    considered immediate failure and no retries will occur.
-
-    If it raises URLGrabError, the error code will determine the retry
-    behavior.  Negative error numbers are reserved for use by these
-    passed in functions, so you can use many negative numbers for
-    different types of failure.  By default, -1 results in a retry,
-    but this can be customized with retrycodes.
-
-    If you simply pass in a function, it will be given exactly one
-    argument: a CallbackObject instance with the .url attribute
-    defined and either .filename (for urlgrab) or .data (for urlread).
-    For urlgrab, .filename is the name of the local file.  For
-    urlread, .data is the actual string data.  If you need other
-    arguments passed to the callback (program state of some sort), you
-    can do so like this:
-
-      checkfunc=(function, ('arg1', 2), {'kwarg': 3})
-
-    if the downloaded file has filename /tmp/stuff, then this will
-    result in this call (for urlgrab):
-
-      function(obj, 'arg1', 2, kwarg=3)
-      # obj.filename = '/tmp/stuff'
-      # obj.url = 'http://foo.com/stuff'
-      
-    NOTE: both the "args" tuple and "kwargs" dict must be present if
-    you use this syntax, but either (or both) can be empty.
-
-  failure_callback = None
-
-    The callback that gets called during retries when an attempt to
-    fetch a file fails.  The syntax for specifying the callback is
-    identical to checkfunc, except for the attributes defined in the
-    CallbackObject instance.  In this case, it will have .exception
-    and .url defined.  As you might suspect, .exception is the
-    exception that was raised.
-
-    The callback is present primarily to inform the calling program of
-    the failure, but if it raises an exception (including the one it's
-    passed) that exception will NOT be caught and will therefore cause
-    future retries to be aborted.
-
-BANDWIDTH THROTTLING
-
-  urlgrabber supports throttling via two values: throttle and
-  bandwidth Between the two, you can either specify and absolute
-  throttle threshold or specify a theshold as a fraction of maximum
-  available bandwidth.
-
-  throttle is a number - if it's an int, it's the bytes/second
-  throttle limit.  If it's a float, it is first multiplied by
-  bandwidth.  If throttle == 0, throttling is disabled.  If None, the
-  module-level default (which can be set with set_throttle) is used.
-
-  bandwidth is the nominal max bandwidth in bytes/second.  If throttle
-  is a float and bandwidth == 0, throttling is disabled.  If None, the
-  module-level default (which can be set with set_bandwidth) is used.
-
-  THROTTLING EXAMPLES:
-
-  Lets say you have a 100 Mbps connection.  This is (about) 10^8 bits
-  per second, or 12,500,000 Bytes per second.  You have a number of
-  throttling options:
-
-  *) set_bandwidth(12500000); set_throttle(0.5) # throttle is a float
-
-     This will limit urlgrab to use half of your available bandwidth.
-
-  *) set_throttle(6250000) # throttle is an int
-
-     This will also limit urlgrab to use half of your available
-     bandwidth, regardless of what bandwidth is set to.
-
-  *) set_throttle(6250000); set_throttle(1.0) # float
-
-     Use half your bandwidth
-
-  *) set_throttle(6250000); set_throttle(2.0) # float
-
-    Use up to 12,500,000 Bytes per second (your nominal max bandwidth)
-
-  *) set_throttle(6250000); set_throttle(0) # throttle = 0
-
-     Disable throttling - this is more efficient than a very large
-     throttle setting.
-
-  *) set_throttle(0); set_throttle(1.0) # throttle is float, bandwidth = 0
-
-     Disable throttling - this is the default when the module is loaded.
-
-  SUGGESTED AUTHOR IMPLEMENTATION (THROTTLING)
-
-  While this is flexible, it's not extremely obvious to the user.  I
-  suggest you implement a float throttle as a percent to make the
-  distinction between absolute and relative throttling very explicit.
-
-  Also, you may want to convert the units to something more convenient
-  than bytes/second, such as kbps or kB/s, etc.
-
-"""
-
-# $Id: grabber.py,v 1.39 2005/03/03 00:54:23 mstenner Exp $
-
-import os
-import os.path
-import urlparse
-import rfc822
-import time
-import string
-import urllib
-import urllib2
-from stat import *  # S_* and ST_*
-
-try:
-    exec('from ' + (__name__.split('.'))[0] + ' import __version__')
-except:
-    __version__ = '???'
-
-auth_handler = urllib2.HTTPBasicAuthHandler( \
-     urllib2.HTTPPasswordMgrWithDefaultRealm())
-
-DEBUG=0
-
-try:
-    from i18n import _
-except ImportError, msg:
-    def _(st): return st
-
-try:
-    from httplib import HTTPException
-except ImportError, msg:
-    HTTPException = None
-
-try:
-    # This is a convenient way to make keepalive optional.
-    # Just rename the module so it can't be imported.
-    from keepalive import HTTPHandler
-except ImportError, msg:
-    keepalive_handler = None
-else:
-    keepalive_handler = HTTPHandler()
-
-try:
-    # add in range support conditionally too
-    from urlgrabber.byterange import HTTPRangeHandler, FileRangeHandler, \
-         FTPRangeHandler, range_tuple_normalize, range_tuple_to_header, \
-         RangeError
-except ImportError, msg:
-    range_handlers = ()
-    RangeError = None
-    have_range = 0
-else:
-    range_handlers = (HTTPRangeHandler(), FileRangeHandler(), FTPRangeHandler())
-    have_range = 1
-
-
-# check whether socket timeout support is available (Python >= 2.3)
-import socket
-try:
-    TimeoutError = socket.timeout
-    have_socket_timeout = True
-except AttributeError:
-    TimeoutError = None
-    have_socket_timeout = False
-
-class URLGrabError(IOError):
-    """
-    URLGrabError error codes:
-
-      URLGrabber error codes (0 -- 255)
-        0    - everything looks good (you should never see this)
-        1    - malformed url
-        2    - local file doesn't exist
-        3    - request for non-file local file (dir, etc)
-        4    - IOError on fetch
-        5    - OSError on fetch
-        6    - no content length header when we expected one
-        7    - HTTPException
-        8    - Exceeded read limit (for urlread)
-        9    - Requested byte range not satisfiable.
-        10   - Byte range requested, but range support unavailable
-        11   - Illegal reget mode
-        12   - Socket timeout.
-
-      MirrorGroup error codes (256 -- 511)
-        256  - No more mirrors left to try
-
-      Custom (non-builtin) classes derived from MirrorGroup (512 -- 767)
-        [ this range reserved for application-specific error codes ]
-
-      Retry codes (< 0)
-        -1   - retry the download, unknown reason
-
-    Note: to test which group a code is in, you can simply do integer
-    division by 256: e.errno / 256
-
-    Negative codes are reserved for use by functions passed in to
-    retrygrab with checkfunc.  The value -1 is built in as a generic
-    retry code and is already included in the retrycodes list.
-    Therefore, you can create a custom check function that simply
-    returns -1 and the fetch will be re-tried.  For more customized
-    retries, you can use other negative number and include them in
-    retry-codes.  This is nice for outputting useful messages about
-    what failed.
-
-    You can use these error codes like so:
-      try: urlgrab(url)
-      except URLGrabError, e:
-         if e.errno == 3: ...
-           # or
-         print e.strerror
-           # or simply
-         print e  #### print '[Errno %i] %s' % (e.errno, e.strerror)
-    """
-    pass
-
-class CallbackObject:
-    """Container for returned callback data.
-
-    This is currently a dummy class into which urlgrabber can stuff
-    information for passing to callbacks.  This way, the prototype for
-    all callbacks is the same, regardless of the data that will be
-    passed back.  Any function that accepts a callback function as an
-    argument SHOULD document what it will define in this object.
-
-    It is possible that this class will have some greater
-    functionality in the future.
-    """
-    pass
-
-def close_all():
-    """close any open keepalive connections"""
-    if keepalive_handler: keepalive_handler.close_all()
-
-def urlgrab(url, filename=None, **kwargs):
-    """grab the file at <url> and make a local copy at <filename>
-    If filename is none, the basename of the url is used.
-    urlgrab returns the filename of the local file, which may be different
-    from the passed-in filename if the copy_local kwarg == 0.
-    
-    See module documentation for a description of possible kwargs.
-    """
-    return default_grabber.urlgrab(url, filename, **kwargs)
-
-def urlopen(url, **kwargs):
-    """open the url and return a file object
-    If a progress object or throttle specifications exist, then
-    a special file object will be returned that supports them.
-    The file object can be treated like any other file object.
-    
-    See module documentation for a description of possible kwargs.
-    """
-    return default_grabber.urlopen(url, **kwargs)
-
-def urlread(url, limit=None, **kwargs):
-    """read the url into a string, up to 'limit' bytes
-    If the limit is exceeded, an exception will be thrown.  Note that urlread
-    is NOT intended to be used as a way of saying "I want the first N bytes"
-    but rather 'read the whole file into memory, but don't use too much'
-    
-    See module documentation for a description of possible kwargs.
-    """
-    return default_grabber.urlread(url, limit, **kwargs)
-
-
-class URLGrabberOptions:
-    """Class to ease kwargs handling."""
-
-    def __init__(self, delegate=None, **kwargs):
-        """Initialize URLGrabberOptions object.
-        Set default values for all options and then update options specified
-        in kwargs.
-        """
-        self.delegate = delegate
-        if delegate is None:
-            self._set_defaults()
-        self._set_attributes(**kwargs)
-    
-    def __getattr__(self, name):
-        if self.delegate and hasattr(self.delegate, name):
-            return getattr(self.delegate, name)
-        raise AttributeError, name
-    
-    def raw_throttle(self):
-        """Calculate raw throttle value from throttle and bandwidth 
-        values.
-        """
-        if self.throttle <= 0:  
-            return 0
-        elif type(self.throttle) == type(0): 
-            return float(self.throttle)
-        else: # throttle is a float
-            return self.bandwidth * self.throttle
-        
-    def derive(self, **kwargs):
-        """Create a derived URLGrabberOptions instance.
-        This method creates a new instance and overrides the
-        options specified in kwargs.
-        """
-        return URLGrabberOptions(delegate=self, **kwargs)
-        
-    def _set_attributes(self, **kwargs):
-        """Update object attributes with those provided in kwargs."""
-        self.__dict__.update(kwargs)
-        if have_range and kwargs.has_key('range'):
-            # normalize the supplied range value
-            self.range = range_tuple_normalize(self.range)
-        if not self.reget in [None, 'simple', 'check_timestamp']:
-            raise URLGrabError(11, _('Illegal reget mode: %s') \
-                               % (self.reget, ))
-
-    def _set_defaults(self):
-        """Set all options to their default values. 
-        When adding new options, make sure a default is
-        provided here.
-        """
-        self.progress_obj = None
-        self.throttle = 1.0
-        self.bandwidth = 0
-        self.retry = None
-        self.retrycodes = [-1,2,4,5,6,7]
-        self.checkfunc = None
-        self.copy_local = 0
-        self.close_connection = 0
-        self.range = None
-        self.user_agent = 'urlgrabber/%s' % __version__
-        self.keepalive = 1
-        self.proxies = None
-        self.reget = None
-        self.failure_callback = None
-        self.prefix = None
-        self.opener = None
-        self.cache_openers = True
-        self.timeout = None
-        self.text = None
-        self.http_headers = None
-        self.ftp_headers = None
-
-class URLGrabber:
-    """Provides easy opening of URLs with a variety of options.
-    
-    All options are specified as kwargs. Options may be specified when
-    the class is created and may be overridden on a per request basis.
-    
-    New objects inherit default values from default_grabber.
-    """
-    
-    def __init__(self, **kwargs):
-        self.opts = URLGrabberOptions(**kwargs)
-    
-    def _retry(self, opts, func, *args):
-        tries = 0
-        while 1:
-            tries = tries + 1
-            try:
-                return apply(func, (opts,) + args, {})
-            except URLGrabError, e:
-                if DEBUG: print 'EXCEPTION: %s' % e
-                if (opts.retry is None) \
-                    or (tries == opts.retry) \
-                    or (e.errno not in opts.retrycodes): raise
-                if opts.failure_callback:
-                    cb_func, cb_args, cb_kwargs = \
-                          self._make_callback(opts.failure_callback)
-                    # this is a little icky - for now, the first element
-                    # of args is the url.  we might consider a way to tidy
-                    # that up, though
-                    obj = CallbackObject()
-                    obj.exception = e
-                    obj.url = args[0]
-                    cb_func(obj, *cb_args, **cb_kwargs)
-    
-    def urlopen(self, url, **kwargs):
-        """open the url and return a file object
-        If a progress object or throttle value specified when this 
-        object was created, then  a special file object will be 
-        returned that supports them. The file object can be treated 
-        like any other file object.
-        """
-        opts = self.opts.derive(**kwargs)
-        (url,parts) = self._parse_url(url) 
-        def retryfunc(opts, url):
-            return URLGrabberFileObject(url, filename=None, opts=opts)
-        return self._retry(opts, retryfunc, url)
-    
-    def urlgrab(self, url, filename=None, **kwargs):
-        """grab the file at <url> and make a local copy at <filename>
-        If filename is none, the basename of the url is used.
-        urlgrab returns the filename of the local file, which may be 
-        different from the passed-in filename if copy_local == 0.
-        """
-        opts = self.opts.derive(**kwargs)
-        (url, parts) = self._parse_url(url)
-        (scheme, host, path, parm, query, frag) = parts
-        if filename is None:
-            if scheme in [ 'http', 'https' ]:
-                filename = os.path.basename( urllib.unquote(path) )
-            else:
-                filename = os.path.basename( path )
-        if scheme == 'file' and not opts.copy_local:
-            # just return the name of the local file - don't make a 
-            # copy currently
-            if not os.path.exists(path):
-                raise URLGrabError(2, 
-                      _('Local file does not exist: %s') % (path, ))
-            elif not os.path.isfile(path):
-                raise URLGrabError(3, 
-                              _('Not a normal file: %s') % (path, ))
-            elif not opts.range:
-                return path
-        
-        def retryfunc(opts, url, filename):
-            fo = URLGrabberFileObject(url, filename, opts)
-            try:
-                fo._do_grab()
-                if not opts.checkfunc is None:
-                    cb_func, cb_args, cb_kwargs = \
-                             self._make_callback(opts.checkfunc)
-                    obj = CallbackObject()
-                    obj.filename = filename
-                    obj.url = url
-                    apply(cb_func, (obj, )+cb_args, cb_kwargs)
-            finally:
-                fo.close()
-            return filename
-        
-        return self._retry(opts, retryfunc, url, filename)
-    
-    def urlread(self, url, limit=None, **kwargs):
-        """read the url into a string, up to 'limit' bytes
-        If the limit is exceeded, an exception will be thrown.  Note
-        that urlread is NOT intended to be used as a way of saying 
-        "I want the first N bytes" but rather 'read the whole file 
-        into memory, but don't use too much'
-        """
-        opts = self.opts.derive(**kwargs)
-        (url, parts) = self._parse_url(url)
-        if limit is not None:
-            limit = limit + 1
-            
-        def retryfunc(opts, url, limit):
-            fo = URLGrabberFileObject(url, filename=None, opts=opts)
-            s = ''
-            try:
-                # this is an unfortunate thing.  Some file-like objects
-                # have a default "limit" of None, while the built-in (real)
-                # file objects have -1.  They each break the other, so for
-                # now, we just force the default if necessary.
-                if limit is None: s = fo.read()
-                else: s = fo.read(limit)
-
-                if not opts.checkfunc is None:
-                    cb_func, cb_args, cb_kwargs = \
-                             self._make_callback(opts.checkfunc)
-                    obj = CallbackObject()
-                    obj.data = s
-                    obj.url = url
-                    apply(cb_func, (obj, )+cb_args, cb_kwargs)
-            finally:
-                fo.close()
-            return s
-            
-        s = self._retry(opts, retryfunc, url, limit)
-        if limit and len(s) > limit:
-            raise URLGrabError(8, 
-                        _('Exceeded limit (%i): %s') % (limit, url))
-        return s
-        
-    def _parse_url(self,url):
-        """break up the url into its component parts
-
-        This function disassembles a url and
-        1) "normalizes" it, tidying it up a bit
-        2) does any authentication stuff it needs to do
-
-        it returns the (cleaned) url and a tuple of component parts
-        """
-        if self.opts.prefix:
-            p = self.opts.prefix
-            if p[-1] == '/' or url[0] == '/': url = p + url
-            else: url = p + '/' + url
-            
-        (scheme, host, path, parm, query, frag) = \
-                                             urlparse.urlparse(url)
-        if not scheme:
-            if not url[0] == '/': url = os.path.abspath(url)
-            url = 'file:' + url
-            (scheme, host, path, parm, query, frag) = \
-                                             urlparse.urlparse(url)
-        path = os.path.normpath(path)
-        if scheme in ['http', 'https']: path = urllib.quote(path)
-        if '@' in host and auth_handler and scheme in ['http', 'https']:
-            try:
-                user_pass, host = host.split('@', 1)
-                if ':' in user_pass: user, password = user_pass.split(':', 1)
-            except ValueError, e:
-                raise URLGrabError(1, _('Bad URL: %s') % url)
-            if DEBUG: print 'adding HTTP auth: %s, %s' % (user, password)
-            auth_handler.add_password(None, host, user, password)
-        parts = (scheme, host, path, parm, query, frag)
-        url = urlparse.urlunparse(parts)
-        return url, parts
-        
-    def _make_callback(self, callback_obj):
-        if callable(callback_obj):
-            return callback_obj, (), {}
-        else:
-            return callback_obj
-
-# create the default URLGrabber used by urlXXX functions.
-# NOTE: actual defaults are set in URLGrabberOptions
-default_grabber = URLGrabber()
-
-class URLGrabberFileObject:
-    """This is a file-object wrapper that supports progress objects 
-    and throttling.
-
-    This exists to solve the following problem: lets say you want to
-    drop-in replace a normal open with urlopen.  You want to use a
-    progress meter and/or throttling, but how do you do that without
-    rewriting your code?  Answer: urlopen will return a wrapped file
-    object that does the progress meter and-or throttling internally.
-    """
-
-    def __init__(self, url, filename, opts):
-        self.url = url
-        self.filename = filename
-        self.opts = opts
-        self.fo = None
-        self._rbuf = ''
-        self._rbufsize = 1024*8
-        self._ttime = time.time()
-        self._tsize = 0
-        self._amount_read = 0
-        self._opener = None
-        self._do_open()
-        
-    def __getattr__(self, name):
-        """This effectively allows us to wrap at the instance level.
-        Any attribute not found in _this_ object will be searched for
-        in self.fo.  This includes methods."""
-        if hasattr(self.fo, name):
-            return getattr(self.fo, name)
-        raise AttributeError, name
-    
-    def _get_opener(self):
-        """Build a urllib2 OpenerDirector based on request options."""
-        if self.opts.opener:
-            return self.opts.opener
-        elif self._opener is None:
-            handlers = []
-            need_keepalive_handler = (keepalive_handler and self.opts.keepalive)
-            need_range_handler = (range_handlers and \
-                                  (self.opts.range or self.opts.reget))
-            # if you specify a ProxyHandler when creating the opener
-            # it _must_ come before all other handlers in the list or urllib2
-            # chokes.
-            if self.opts.proxies:
-                handlers.append( CachedProxyHandler(self.opts.proxies) )
-
-                # -------------------------------------------------------
-                # OK, these next few lines are a serious kludge to get
-                # around what I think is a bug in python 2.2's
-                # urllib2.  The basic idea is that default handlers
-                # get applied first.  If you override one (like a
-                # proxy handler), then the default gets pulled, but
-                # the replacement goes on the end.  In the case of
-                # proxies, this means the normal handler picks it up
-                # first and the proxy isn't used.  Now, this probably
-                # only happened with ftp or non-keepalive http, so not
-                # many folks saw it.  The simple approach to fixing it
-                # is just to make sure you override the other
-                # conflicting defaults as well.  I would LOVE to see
-                # these go way or be dealt with more elegantly.  The
-                # problem isn't there after 2.2.  -MDS 2005/02/24
-                if not need_keepalive_handler:
-                    handlers.append( urllib2.HTTPHandler() )
-                if not need_range_handler:
-                    handlers.append( urllib2.FTPHandler() )
-                # -------------------------------------------------------
-                    
-            if need_keepalive_handler:
-                handlers.append( keepalive_handler )
-            if need_range_handler:
-                handlers.extend( range_handlers )
-            handlers.append( auth_handler )
-            if self.opts.cache_openers:
-              self._opener = CachedOpenerDirector(*handlers)
-            else:
-              self._opener = urllib2.build_opener(*handlers)
-            # OK, I don't like to do this, but otherwise, we end up with
-            # TWO user-agent headers.
-            self._opener.addheaders = []
-        return self._opener
-        
-    def _do_open(self):
-        opener = self._get_opener()
-
-        req = urllib2.Request(self.url) # build request object
-        self._add_headers(req) # add misc headers that we need
-        self._build_range(req) # take care of reget and byterange stuff
-
-        fo, hdr = self._make_request(req, opener)
-        if self.reget_time and self.opts.reget == 'check_timestamp':
-            # do this if we have a local file with known timestamp AND
-            # we're in check_timestamp reget mode.
-            fetch_again = 0
-            try:
-                modified_tuple  = hdr.getdate_tz('last-modified')
-                modified_stamp  = rfc822.mktime_tz(modified_tuple)
-                if modified_stamp > self.reget_time: fetch_again = 1
-            except (TypeError,):
-                fetch_again = 1
-            
-            if fetch_again:
-                # the server version is newer than the (incomplete) local
-                # version, so we should abandon the version we're getting
-                # and fetch the whole thing again.
-                fo.close()
-                self.opts.reget = None
-                del req.headers['Range']
-                self._build_range(req)
-                fo, hdr = self._make_request(req, opener)
-
-        (scheme, host, path, parm, query, frag) = urlparse.urlparse(self.url)
-        if not (self.opts.progress_obj or self.opts.raw_throttle() \
-                or self.opts.timeout):
-            # if we're not using the progress_obj, throttling, or timeout
-            # we can get a performance boost by going directly to
-            # the underlying fileobject for reads.
-            self.read = fo.read
-            if hasattr(fo, 'readline'):
-                self.readline = fo.readline
-        elif self.opts.progress_obj:
-            try:    length = int(hdr['Content-Length'])
-            except: length = None
-            self.opts.progress_obj.start(str(self.filename), self.url, 
-                                         os.path.basename(path), 
-                                         length,
-                                         text=self.opts.text)
-            self.opts.progress_obj.update(0)
-        (self.fo, self.hdr) = (fo, hdr)
-    
-    def _add_headers(self, req):
-        if self.opts.user_agent:
-            req.add_header('User-agent', self.opts.user_agent)
-        try: req_type = req.get_type()
-        except ValueError: req_type = None
-        if self.opts.http_headers and req_type in ('http', 'https'):
-            for h, v in self.opts.http_headers:
-                req.add_header(h, v)
-        if self.opts.ftp_headers and req_type == 'ftp':
-            for h, v in self.opts.ftp_headers:
-                req.add_header(h, v)
-
-    def _build_range(self, req):
-        self.reget_time = None
-        self.append = 0
-        reget_length = 0
-        rt = None
-        if have_range and self.opts.reget and type(self.filename) == type(''):
-            # we have reget turned on and we're dumping to a file
-            try:
-                s = os.stat(self.filename)
-            except OSError:
-                pass
-            else:
-                self.reget_time = s[ST_MTIME]
-                reget_length = s[ST_SIZE]
-                rt = reget_length, ''
-                self.append = 1
-                
-        if self.opts.range:
-            if not have_range:
-                raise URLGrabError(10, _('Byte range requested but range '\
-                                         'support unavailable'))
-            rt = self.opts.range
-            if rt[0]: rt = (rt[0] + reget_length, rt[1])
-
-        if rt:
-            header = range_tuple_to_header(rt)
-            if header: req.add_header('Range', header)
-
-    def _make_request(self, req, opener):
-        try:
-            if have_socket_timeout and self.opts.timeout:
-                old_to = socket.getdefaulttimeout()
-                socket.setdefaulttimeout(self.opts.timeout)
-                try:
-                    fo = opener.open(req)
-                finally:
-                    socket.setdefaulttimeout(old_to)
-            else:
-                fo = opener.open(req)
-            hdr = fo.info()
-        except ValueError, e:
-            raise URLGrabError(1, _('Bad URL: %s') % (e, ))
-        except RangeError, e:
-            raise URLGrabError(9, _('%s') % (e, ))
-        except IOError, e:
-            if hasattr(e, 'reason') and have_socket_timeout and \
-                   isinstance(e.reason, TimeoutError):
-                raise URLGrabError(12, _('Timeout: %s') % (e, ))
-            else:
-                raise URLGrabError(4, _('IOError: %s') % (e, ))
-        except OSError, e:
-            raise URLGrabError(5, _('OSError: %s') % (e, ))
-        except HTTPException, e:
-            raise URLGrabError(7, _('HTTP Error (%s): %s') % \
-                            (e.__class__.__name__, e))
-        else:
-            return (fo, hdr)
-        
-    def _do_grab(self):
-        """dump the file to self.filename."""
-        if self.append: new_fo = open(self.filename, 'ab')
-        else: new_fo = open(self.filename, 'wb')
-        bs = 1024*8
-        size = 0
-
-        block = self.read(bs)
-        size = size + len(block)
-        while block:
-            new_fo.write(block)
-            block = self.read(bs)
-            size = size + len(block)
-
-        new_fo.close()
-        try:
-            modified_tuple  = self.hdr.getdate_tz('last-modified')
-            modified_stamp  = rfc822.mktime_tz(modified_tuple)
-            os.utime(self.filename, (modified_stamp, modified_stamp))
-        except (TypeError,), e: pass
-
-        return size
-    
-    def _fill_buffer(self, amt=None):
-        """fill the buffer to contain at least 'amt' bytes by reading
-        from the underlying file object.  If amt is None, then it will
-        read until it gets nothing more.  It updates the progress meter
-        and throttles after every self._rbufsize bytes."""
-        # the _rbuf test is only in this first 'if' for speed.  It's not
-        # logically necessary
-        if self._rbuf and not amt is None:
-            L = len(self._rbuf)
-            if amt > L:
-                amt = amt - L
-            else:
-                return
-
-        # if we've made it here, then we don't have enough in the buffer
-        # and we need to read more.
-
-        buf = [self._rbuf]
-        bufsize = len(self._rbuf)
-        while amt is None or amt:
-            # first, delay if necessary for throttling reasons
-            if self.opts.raw_throttle():
-                diff = self._tsize/self.opts.raw_throttle() - \
-                       (time.time() - self._ttime)
-                if diff > 0: time.sleep(diff)
-                self._ttime = time.time()
-                
-            # now read some data, up to self._rbufsize
-            if amt is None: readamount = self._rbufsize
-            else:           readamount = min(amt, self._rbufsize)
-            try:
-                new = self.fo.read(readamount)
-            except socket.error, e:
-                raise URLGrabError(4, _('Socket Error: %s') % (e, ))
-            except TimeoutError, e:
-                raise URLGrabError(12, _('Timeout: %s') % (e, ))
-            newsize = len(new)
-            if not newsize: break # no more to read
-
-            if amt: amt = amt - newsize
-            buf.append(new)
-            bufsize = bufsize + newsize
-            self._tsize = newsize
-            self._amount_read = self._amount_read + newsize
-            if self.opts.progress_obj:
-                self.opts.progress_obj.update(self._amount_read)
-
-        self._rbuf = string.join(buf, '')
-        return
-
-    def read(self, amt=None):
-        self._fill_buffer(amt)
-        if amt is None:
-            s, self._rbuf = self._rbuf, ''
-        else:
-            s, self._rbuf = self._rbuf[:amt], self._rbuf[amt:]
-        return s
-
-    def readline(self, limit=-1):
-        i = string.find(self._rbuf, '\n')
-        while i < 0 and not (0 < limit <= len(self._rbuf)):
-            L = len(self._rbuf)
-            self._fill_buffer(L + self._rbufsize)
-            if not len(self._rbuf) > L: break
-            i = string.find(self._rbuf, '\n', L)
-
-        if i < 0: i = len(self._rbuf)
-        else: i = i+1
-        if 0 <= limit < len(self._rbuf): i = limit
-
-        s, self._rbuf = self._rbuf[:i], self._rbuf[i:]
-        return s
-
-    def close(self):
-        if self.opts.progress_obj:
-            self.opts.progress_obj.end(self._amount_read)
-        self.fo.close()
-        if self.opts.close_connection:
-            try: self.fo.close_connection()
-            except: pass
-
-_handler_cache = []
-def CachedOpenerDirector(*handlers):
-    for (cached_handlers, opener) in _handler_cache:
-        if cached_handlers == handlers:
-            for handler in opener.handlers:
-                handler.add_parent(opener)
-            return opener
-    opener = urllib2.build_opener(*handlers)
-    _handler_cache.append( (handlers, opener) )
-    return opener
-
-_proxy_cache = []
-def CachedProxyHandler(proxies):
-    for (pdict, handler) in _proxy_cache:
-        if pdict == proxies:
-            break
-    else:
-        handler = urllib2.ProxyHandler(proxies)
-        _proxy_cache.append( (proxies, handler) )
-    return handler
-
-#####################################################################
-# DEPRECATED FUNCTIONS
-def set_throttle(new_throttle):
-    """Deprecated. Use: default_grabber.throttle = new_throttle"""
-    default_grabber.throttle = new_throttle
-
-def set_bandwidth(new_bandwidth):
-    """Deprecated. Use: default_grabber.bandwidth = new_bandwidth"""
-    default_grabber.bandwidth = new_bandwidth
-
-def set_progress_obj(new_progress_obj):
-    """Deprecated. Use: default_grabber.progress_obj = new_progress_obj"""
-    default_grabber.progress_obj = new_progress_obj
-
-def set_user_agent(new_user_agent):
-    """Deprecated. Use: default_grabber.user_agent = new_user_agent"""
-    default_grabber.user_agent = new_user_agent
-    
-def retrygrab(url, filename=None, copy_local=0, close_connection=0,
-              progress_obj=None, throttle=None, bandwidth=None,
-              numtries=3, retrycodes=[-1,2,4,5,6,7], checkfunc=None):
-    """Deprecated. Use: urlgrab() with the retry arg instead"""
-    kwargs = {'copy_local' :  copy_local, 
-              'close_connection' : close_connection,
-              'progress_obj' : progress_obj, 
-              'throttle' : throttle, 
-              'bandwidth' : bandwidth,
-              'retry' : numtries,
-              'retrycodes' : retrycodes,
-              'checkfunc' : checkfunc 
-              }
-    return urlgrab(url, filename, **kwargs)
-
-        
-#####################################################################
-#  TESTING
-def _main_test():
-    import sys
-    try: url, filename = sys.argv[1:3]
-    except ValueError:
-        print 'usage:', sys.argv[0], \
-              '<url> <filename> [copy_local=0|1] [close_connection=0|1]'
-        sys.exit()
-
-    kwargs = {}
-    for a in sys.argv[3:]:
-        k, v = string.split(a, '=', 1)
-        kwargs[k] = int(v)
-
-    set_throttle(1.0)
-    set_bandwidth(32 * 1024)
-    print "throttle: %s,  throttle bandwidth: %s B/s" % (default_grabber.throttle, 
-                                                        default_grabber.bandwidth)
-
-    try: from progress import text_progress_meter
-    except ImportError, e: pass
-    else: kwargs['progress_obj'] = text_progress_meter()
-
-    try: name = apply(urlgrab, (url, filename), kwargs)
-    except URLGrabError, e: print e
-    else: print 'LOCAL FILE:', name
-
-
-def _retry_test():
-    import sys
-    try: url, filename = sys.argv[1:3]
-    except ValueError:
-        print 'usage:', sys.argv[0], \
-              '<url> <filename> [copy_local=0|1] [close_connection=0|1]'
-        sys.exit()
-
-    kwargs = {}
-    for a in sys.argv[3:]:
-        k, v = string.split(a, '=', 1)
-        kwargs[k] = int(v)
-
-    try: from progress import text_progress_meter
-    except ImportError, e: pass
-    else: kwargs['progress_obj'] = text_progress_meter()
-
-    global DEBUG
-    #DEBUG = 1
-    def cfunc(filename, hello, there='foo'):
-        print hello, there
-        import random
-        rnum = random.random()
-        if rnum < .5:
-            print 'forcing retry'
-            raise URLGrabError(-1, 'forcing retry')
-        if rnum < .75:
-            print 'forcing failure'
-            raise URLGrabError(-2, 'forcing immediate failure')
-        print 'success'
-        return
-        
-    close_all()
-    kwargs['checkfunc'] = (cfunc, ('hello',), {'there':'there'})
-    try: name = apply(retrygrab, (url, filename), kwargs)
-    except URLGrabError, e: print e
-    else: print 'LOCAL FILE:', name
-
-def _file_object_test(filename=None):
-    import random, cStringIO, sys
-    if filename is None:
-        filename = __file__
-    print 'using file "%s" for comparisons' % filename
-    fo = open(filename)
-    s_input = fo.read()
-    fo.close()
-
-    for testfunc in [_test_file_object_smallread,
-                     _test_file_object_readall,
-                     _test_file_object_readline,
-                     _test_file_object_readlines]:
-        fo_input = cStringIO.StringIO(s_input)
-        fo_output = cStringIO.StringIO()
-        wrapper = URLGrabberFileObject(fo_input, None, 0)
-        print 'testing %-30s ' % testfunc.__name__,
-        testfunc(wrapper, fo_output)
-        s_output = fo_output.getvalue()
-        if s_output == s_input: print 'passed'
-        else: print 'FAILED'
-            
-def _test_file_object_smallread(wrapper, fo_output):
-    while 1:
-        s = wrapper.read(23)
-        fo_output.write(s)
-        if not s: return
-
-def _test_file_object_readall(wrapper, fo_output):
-    s = wrapper.read()
-    fo_output.write(s)
-
-def _test_file_object_readline(wrapper, fo_output):
-    while 1:
-        s = wrapper.readline()
-        fo_output.write(s)
-        if not s: return
-
-def _test_file_object_readlines(wrapper, fo_output):
-    li = wrapper.readlines()
-    fo_output.write(string.join(li, ''))
-
-if __name__ == '__main__':
-    _main_test()
-    _retry_test()
-    _file_object_test('test')
-    

=== removed file 'bzrlib/util/urlgrabber/keepalive.py'
--- bzrlib/util/urlgrabber/keepalive.py	2005-09-19 06:05:19 +0000
+++ bzrlib/util/urlgrabber/keepalive.py	1970-01-01 00:00:00 +0000
@@ -1,590 +0,0 @@
-#   This library is free software; you can redistribute it and/or
-#   modify it under the terms of the GNU Lesser General Public
-#   License as published by the Free Software Foundation; either
-#   version 2.1 of the License, or (at your option) any later version.
-#
-#   This library is distributed in the hope that it will be useful,
-#   but WITHOUT ANY WARRANTY; without even the implied warranty of
-#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#   Lesser General Public License for more details.
-#
-#   You should have received a copy of the GNU Lesser General Public
-#   License along with this library; if not, write to the 
-#      Free Software Foundation, Inc., 
-#      59 Temple Place, Suite 330, 
-#      Boston, MA  02111-1307  USA
-
-# This file is part of urlgrabber, a high-level cross-protocol url-grabber
-# Copyright 2002-2004 Michael D. Stenner, Ryan Tomayko
-
-"""An HTTP handler for urllib2 that supports HTTP 1.1 and keepalive.
-
->>> import urllib2
->>> from keepalive import HTTPHandler
->>> keepalive_handler = HTTPHandler()
->>> opener = urllib2.build_opener(keepalive_handler)
->>> urllib2.install_opener(opener)
->>> 
->>> fo = urllib2.urlopen('http://www.python.org')
-
-If a connection to a given host is requested, and all of the existing
-connections are still in use, another connection will be opened.  If
-the handler tries to use an existing connection but it fails in some
-way, it will be closed and removed from the pool.
-
-To remove the handler, simply re-run build_opener with no arguments, and
-install that opener.
-
-You can explicitly close connections by using the close_connection()
-method of the returned file-like object (described below) or you can
-use the handler methods:
-
-  close_connection(host)
-  close_all()
-  open_connections()
-
-NOTE: using the close_connection and close_all methods of the handler
-should be done with care when using multiple threads.
-  * there is nothing that prevents another thread from creating new
-    connections immediately after connections are closed
-  * no checks are done to prevent in-use connections from being closed
-
->>> keepalive_handler.close_all()
-
-EXTRA ATTRIBUTES AND METHODS
-
-  Upon a status of 200, the object returned has a few additional
-  attributes and methods, which should not be used if you want to
-  remain consistent with the normal urllib2-returned objects:
-
-    close_connection()  -  close the connection to the host
-    readlines()         -  you know, readlines()
-    status              -  the return status (ie 404)
-    reason              -  english translation of status (ie 'File not found')
-
-  If you want the best of both worlds, use this inside an
-  AttributeError-catching try:
-
-  >>> try: status = fo.status
-  >>> except AttributeError: status = None
-
-  Unfortunately, these are ONLY there if status == 200, so it's not
-  easy to distinguish between non-200 responses.  The reason is that
-  urllib2 tries to do clever things with error codes 301, 302, 401,
-  and 407, and it wraps the object upon return.
-
-  For python versions earlier than 2.4, you can avoid this fancy error
-  handling by setting the module-level global HANDLE_ERRORS to zero.
-  You see, prior to 2.4, it's the HTTP Handler's job to determine what
-  to handle specially, and what to just pass up.  HANDLE_ERRORS == 0
-  means "pass everything up".  In python 2.4, however, this job no
-  longer belongs to the HTTP Handler and is now done by a NEW handler,
-  HTTPErrorProcessor.  Here's the bottom line:
-
-    python version < 2.4
-        HANDLE_ERRORS == 1  (default) pass up 200, treat the rest as
-                            errors
-        HANDLE_ERRORS == 0  pass everything up, error processing is
-                            left to the calling code
-    python version >= 2.4
-        HANDLE_ERRORS == 1  pass up 200, treat the rest as errors
-        HANDLE_ERRORS == 0  (default) pass everything up, let the
-                            other handlers (specifically,
-                            HTTPErrorProcessor) decide what to do
-
-  In practice, setting the variable either way makes little difference
-  in python 2.4, so for the most consistent behavior across versions,
-  you probably just want to use the defaults, which will give you
-  exceptions on errors.
-
-"""
-
-# $Id: keepalive.py,v 1.9 2005/02/14 21:55:07 mstenner Exp $
-
-import urllib2
-import httplib
-import socket
-import thread
-
-DEBUG = 0
-def DBPRINT(*args): print ' '.join(args)
-
-import sys
-if hasattr(sys, 'version_info'):
-    _python_version = sys.version_info
-else:
-    _python_version = map(int, sys.version.split()[0].split('.'))
-if _python_version < [2, 4]: HANDLE_ERRORS = 1
-else: HANDLE_ERRORS = 0
-    
-class ConnectionManager:
-    """
-    The connection manager must be able to:
-      * keep track of all existing
-      """
-    def __init__(self):
-        self._lock = thread.allocate_lock()
-        self._hostmap = {} # map hosts to a list of connections
-        self._connmap = {} # map connections to host
-        self._readymap = {} # map connection to ready state
-
-    def add(self, host, connection, ready):
-        self._lock.acquire()
-        try:
-            if not self._hostmap.has_key(host): self._hostmap[host] = []
-            self._hostmap[host].append(connection)
-            self._connmap[connection] = host
-            self._readymap[connection] = ready
-        finally:
-            self._lock.release()
-
-    def remove(self, connection):
-        self._lock.acquire()
-        try:
-            try:
-                host = self._connmap[connection]
-            except KeyError:
-                pass
-            else:
-                del self._connmap[connection]
-                del self._readymap[connection]
-                self._hostmap[host].remove(connection)
-                if not self._hostmap[host]: del self._hostmap[host]
-        finally:
-            self._lock.release()
-
-    def set_ready(self, connection, ready):
-        try: self._readymap[connection] = ready
-        except KeyError: pass
-        
-    def get_ready_conn(self, host):
-        conn = None
-        self._lock.acquire()
-        try:
-            if self._hostmap.has_key(host):
-                for c in self._hostmap[host]:
-                    if self._readymap[c]:
-                        self._readymap[c] = 0
-                        conn = c
-                        break
-        finally:
-            self._lock.release()
-        return conn
-
-    def get_all(self, host=None):
-        if host:
-            return list(self._hostmap.get(host, []))
-        else:
-            return dict(self._hostmap)
-
-class HTTPHandler(urllib2.HTTPHandler):
-    def __init__(self):
-        self._cm = ConnectionManager()
-        
-    #### Connection Management
-    def open_connections(self):
-        """return a list of connected hosts and the number of connections
-        to each.  [('foo.com:80', 2), ('bar.org', 1)]"""
-        return [(host, len(li)) for (host, li) in self._cm.get_all().items()]
-
-    def close_connection(self, host):
-        """close connection(s) to <host>
-        host is the host:port spec, as in 'www.cnn.com:8080' as passed in.
-        no error occurs if there is no connection to that host."""
-        for h in self._cm.get_all(host):
-            self._cm.remove(h)
-            h.close()
-        
-    def close_all(self):
-        """close all open connections"""
-        for host, conns in self._cm.get_all().items():
-            for h in conns:
-                self._cm.remove(h)
-                h.close()
-        
-    def _request_closed(self, request, host, connection):
-        """tells us that this request is now closed and the the
-        connection is ready for another request"""
-        self._cm.set_ready(connection, 1)
-
-    def _remove_connection(self, host, connection, close=0):
-        if close: connection.close()
-        self._cm.remove(connection)
-        
-    #### Transaction Execution
-    def http_open(self, req):
-        return self.do_open(HTTPConnection, req)
-
-    def do_open(self, http_class, req):
-        host = req.get_host()
-        if not host:
-            raise urllib2.URLError('no host given')
-
-        try:
-            h = self._cm.get_ready_conn(host)
-            while h:
-                r = self._reuse_connection(h, req, host)
-
-                # if this response is non-None, then it worked and we're
-                # done.  Break out, skipping the else block.
-                if r: break
-
-                # connection is bad - possibly closed by server
-                # discard it and ask for the next free connection
-                h.close()
-                self._cm.remove(h)
-                h = self._cm.get_ready_conn(host)
-            else:
-                # no (working) free connections were found.  Create a new one.
-                h = http_class(host)
-                if DEBUG: DBPRINT("creating new connection to %s (%d)" % \
-                                  (host, id(h)))
-                self._cm.add(host, h, 0)
-                self._start_transaction(h, req)
-                r = h.getresponse()
-        except (socket.error, httplib.HTTPException), err:
-            raise urllib2.URLError(err)
-            
-        # if not a persistent connection, don't try to reuse it
-        if r.will_close: self._cm.remove(h)
-
-        if DEBUG: DBPRINT("STATUS: %s, %s" % (r.status, r.reason))
-        r._handler = self
-        r._host = host
-        r._url = req.get_full_url()
-        r._connection = h
-        r.code = r.status
-        
-        if r.status == 200 or not HANDLE_ERRORS:
-            return r
-        else:
-            return self.parent.error('http', req, r, r.status, r.reason, r.msg)
-
-
-    def _reuse_connection(self, h, req, host):
-        """start the transaction with a re-used connection
-        return a response object (r) upon success or None on failure.
-        This DOES not close or remove bad connections in cases where
-        it returns.  However, if an unexpected exception occurs, it
-        will close and remove the connection before re-raising.
-        """
-        try:
-            self._start_transaction(h, req)
-            r = h.getresponse()
-            # note: just because we got something back doesn't mean it
-            # worked.  We'll check the version below, too.
-        except (socket.error, httplib.HTTPException):
-            r = None
-        except:
-            # adding this block just in case we've missed
-            # something we will still raise the exception, but
-            # lets try and close the connection and remove it
-            # first.  We previously got into a nasty loop
-            # where an exception was uncaught, and so the
-            # connection stayed open.  On the next try, the
-            # same exception was raised, etc.  The tradeoff is
-            # that it's now possible this call will raise
-            # a DIFFERENT exception
-            if DEBUG: DBPRINT("unexpected exception - " \
-                              "closing connection to %s (%d)" % (host, id(h)))
-            self._cm.remove(h)
-            h.close()
-            raise
-                    
-        if r is None or r.version == 9:
-            # httplib falls back to assuming HTTP 0.9 if it gets a
-            # bad header back.  This is most likely to happen if
-            # the socket has been closed by the server since we
-            # last used the connection.
-            if DEBUG: DBPRINT("failed to re-use connection to %s (%d)" \
-                              % (host, id(h)))
-            r = None
-        else:
-            if DEBUG: DBPRINT("re-using connection to %s (%d)" % (host, id(h)))
-
-        return r
-
-    def _start_transaction(self, h, req):
-        try:
-            if req.has_data():
-                data = req.get_data()
-                h.putrequest('POST', req.get_selector())
-                if not req.headers.has_key('Content-type'):
-                    h.putheader('Content-type',
-                                'application/x-www-form-urlencoded')
-                if not req.headers.has_key('Content-length'):
-                    h.putheader('Content-length', '%d' % len(data))
-            else:
-                h.putrequest('GET', req.get_selector())
-        except (socket.error, httplib.HTTPException), err:
-            raise urllib2.URLError(err)
-
-        for args in self.parent.addheaders:
-            h.putheader(*args)
-        for k, v in req.headers.items():
-            h.putheader(k, v)
-        h.endheaders()
-        if req.has_data():
-            h.send(data)
-
-class HTTPResponse(httplib.HTTPResponse):
-    # we need to subclass HTTPResponse in order to
-    # 1) add readline() and readlines() methods
-    # 2) add close_connection() methods
-    # 3) add info() and geturl() methods
-
-    # in order to add readline(), read must be modified to deal with a
-    # buffer.  example: readline must read a buffer and then spit back
-    # one line at a time.  The only real alternative is to read one
-    # BYTE at a time (ick).  Once something has been read, it can't be
-    # put back (ok, maybe it can, but that's even uglier than this),
-    # so if you THEN do a normal read, you must first take stuff from
-    # the buffer.
-
-    # the read method wraps the original to accomodate buffering,
-    # although read() never adds to the buffer.
-    # Both readline and readlines have been stolen with almost no
-    # modification from socket.py
-    
-
-    def __init__(self, sock, debuglevel=0, strict=0, method=None):
-        if method: # the httplib in python 2.3 uses the method arg
-            httplib.HTTPResponse.__init__(self, sock, debuglevel, method)
-        else: # 2.2 doesn't
-            httplib.HTTPResponse.__init__(self, sock, debuglevel)
-        self.fileno = sock.fileno
-        self.code = None
-        self._rbuf = ''
-        self._rbufsize = 8096
-        self._handler = None # inserted by the handler later
-        self._host = None    # (same)
-        self._url = None     # (same)
-        self._connection = None # (same)
-
-    _raw_read = httplib.HTTPResponse.read
-
-    def close(self):
-        if self.fp:
-            self.fp.close()
-            self.fp = None
-            if self._handler:
-                self._handler._request_closed(self, self._host,
-                                              self._connection)
-
-    def close_connection(self):
-        self._handler._remove_connection(self._host, self._connection, close=1)
-        self.close()
-        
-    def info(self):
-        return self.msg
-
-    def geturl(self):
-        return self._url
-
-    def read(self, amt=None):
-        # the _rbuf test is only in this first if for speed.  It's not
-        # logically necessary
-        if self._rbuf and not amt is None:
-            L = len(self._rbuf)
-            if amt > L:
-                amt -= L
-            else:
-                s = self._rbuf[:amt]
-                self._rbuf = self._rbuf[amt:]
-                return s
-
-        s = self._rbuf + self._raw_read(amt)
-        self._rbuf = ''
-        return s
-
-    def readline(self, limit=-1):
-        data = ""
-        i = self._rbuf.find('\n')
-        while i < 0 and not (0 < limit <= len(self._rbuf)):
-            new = self._raw_read(self._rbufsize)
-            if not new: break
-            i = new.find('\n')
-            if i >= 0: i = i + len(self._rbuf)
-            self._rbuf = self._rbuf + new
-        if i < 0: i = len(self._rbuf)
-        else: i = i+1
-        if 0 <= limit < len(self._rbuf): i = limit
-        data, self._rbuf = self._rbuf[:i], self._rbuf[i:]
-        return data
-
-    def readlines(self, sizehint = 0):
-        total = 0
-        list = []
-        while 1:
-            line = self.readline()
-            if not line: break
-            list.append(line)
-            total += len(line)
-            if sizehint and total >= sizehint:
-                break
-        return list
-
-
-class HTTPConnection(httplib.HTTPConnection):
-    # use the modified response class
-    response_class = HTTPResponse
-    
-#########################################################################
-#####   TEST FUNCTIONS
-#########################################################################
-
-def error_handler(url):
-    global HANDLE_ERRORS
-    orig = HANDLE_ERRORS
-    keepalive_handler = HTTPHandler()
-    opener = urllib2.build_opener(keepalive_handler)
-    urllib2.install_opener(opener)
-    pos = {0: 'off', 1: 'on'}
-    for i in (0, 1):
-        print "  fancy error handling %s (HANDLE_ERRORS = %i)" % (pos[i], i)
-        HANDLE_ERRORS = i
-        try:
-            fo = urllib2.urlopen(url)
-            foo = fo.read()
-            fo.close()
-            try: status, reason = fo.status, fo.reason
-            except AttributeError: status, reason = None, None
-        except IOError, e:
-            print "  EXCEPTION: %s" % e
-            raise
-        else:
-            print "  status = %s, reason = %s" % (status, reason)
-    HANDLE_ERRORS = orig
-    hosts = keepalive_handler.open_connections()
-    print "open connections:", hosts
-    keepalive_handler.close_all()
-
-def continuity(url):
-    import md5
-    format = '%25s: %s'
-    
-    # first fetch the file with the normal http handler
-    opener = urllib2.build_opener()
-    urllib2.install_opener(opener)
-    fo = urllib2.urlopen(url)
-    foo = fo.read()
-    fo.close()
-    m = md5.new(foo)
-    print format % ('normal urllib', m.hexdigest())
-
-    # now install the keepalive handler and try again
-    opener = urllib2.build_opener(HTTPHandler())
-    urllib2.install_opener(opener)
-
-    fo = urllib2.urlopen(url)
-    foo = fo.read()
-    fo.close()
-    m = md5.new(foo)
-    print format % ('keepalive read', m.hexdigest())
-
-    fo = urllib2.urlopen(url)
-    foo = ''
-    while 1:
-        f = fo.readline()
-        if f: foo = foo + f
-        else: break
-    fo.close()
-    m = md5.new(foo)
-    print format % ('keepalive readline', m.hexdigest())
-
-def comp(N, url):
-    print '  making %i connections to:\n  %s' % (N, url)
-
-    sys.stdout.write('  first using the normal urllib handlers')
-    # first use normal opener
-    opener = urllib2.build_opener()
-    urllib2.install_opener(opener)
-    t1 = fetch(N, url)
-    print '  TIME: %.3f s' % t1
-
-    sys.stdout.write('  now using the keepalive handler       ')
-    # now install the keepalive handler and try again
-    opener = urllib2.build_opener(HTTPHandler())
-    urllib2.install_opener(opener)
-    t2 = fetch(N, url)
-    print '  TIME: %.3f s' % t2
-    print '  improvement factor: %.2f' % (t1/t2, )
-    
-def fetch(N, url, delay=0):
-    lens = []
-    starttime = time.time()
-    for i in range(N):
-        if delay and i > 0: time.sleep(delay)
-        fo = urllib2.urlopen(url)
-        foo = fo.read()
-        fo.close()
-        lens.append(len(foo))
-    diff = time.time() - starttime
-
-    j = 0
-    for i in lens[1:]:
-        j = j + 1
-        if not i == lens[0]:
-            print "WARNING: inconsistent length on read %i: %i" % (j, i)
-
-    return diff
-
-def test_timeout(url):
-    global DEBUG, DBPRINT
-    dbp = DBPRINT
-    def DBPRINT(*args): print '    ' + ' '.join(args)
-    DEBUG=1
-    print "  fetching the file to establish a connection"
-    fo = urllib2.urlopen(url)
-    data1 = fo.read()
-    fo.close()
- 
-    i = 20
-    print "  waiting %i seconds for the server to close the connection" % i
-    while i > 0:
-        sys.stdout.write('\r  %2i' % i)
-        sys.stdout.flush()
-        time.sleep(1)
-        i -= 1
-    sys.stderr.write('\r')
-
-    print "  fetching the file a second time"
-    fo = urllib2.urlopen(url)
-    data2 = fo.read()
-    fo.close()
-
-    if data1 == data2:
-        print '  data are identical'
-    else:
-        print '  ERROR: DATA DIFFER'
-
-    DEBUG=0
-    DBPRINT = dbp
-
-    
-def test(url, N=10):
-    print "checking error hander (do this on a non-200)"
-    try: error_handler(url)
-    except IOError, e:
-        print "exiting - exception will prevent further tests"
-        sys.exit()
-    print
-    print "performing continuity test (making sure stuff isn't corrupted)"
-    continuity(url)
-    print
-    print "performing speed comparison"
-    comp(N, url)
-    print
-    print "performing dropped-connection check"
-    test_timeout(url)
-    
-if __name__ == '__main__':
-    import time
-    import sys
-    try:
-        N = int(sys.argv[1])
-        url = sys.argv[2]
-    except:
-        print "%s <integer> <url>" % sys.argv[0]
-    else:
-        test(url, N)

=== removed file 'bzrlib/util/urlgrabber/mirror.py'
--- bzrlib/util/urlgrabber/mirror.py	2005-09-19 06:05:19 +0000
+++ bzrlib/util/urlgrabber/mirror.py	1970-01-01 00:00:00 +0000
@@ -1,461 +0,0 @@
-#   This library is free software; you can redistribute it and/or
-#   modify it under the terms of the GNU Lesser General Public
-#   License as published by the Free Software Foundation; either
-#   version 2.1 of the License, or (at your option) any later version.
-#
-#   This library is distributed in the hope that it will be useful,
-#   but WITHOUT ANY WARRANTY; without even the implied warranty of
-#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#   Lesser General Public License for more details.
-#
-#   You should have received a copy of the GNU Lesser General Public
-#   License along with this library; if not, write to the 
-#      Free Software Foundation, Inc., 
-#      59 Temple Place, Suite 330, 
-#      Boston, MA  02111-1307  USA
-
-# This file is part of urlgrabber, a high-level cross-protocol url-grabber
-# Copyright 2002-2004 Michael D. Stenner, Ryan Tomayko
-
-"""Module for downloading files from a pool of mirrors
-
-DESCRIPTION
-
-  This module provides support for downloading files from a pool of
-  mirrors with configurable failover policies.  To a large extent, the
-  failover policy is chosen by using different classes derived from
-  the main class, MirrorGroup.
-
-  Instances of MirrorGroup (and cousins) act very much like URLGrabber
-  instances in that they have urlread, urlgrab, and urlopen methods.
-  They can therefore, be used in very similar ways.
-
-    from urlgrabber.grabber import URLGrabber
-    from urlgrabber.mirror import MirrorGroup
-    gr = URLGrabber()
-    mg = MirrorGroup(gr, ['http://foo.com/some/directory/',
-                          'http://bar.org/maybe/somewhere/else/',
-                          'ftp://baz.net/some/other/place/entirely/']
-    mg.urlgrab('relative/path.zip')
-
-  The assumption is that all mirrors are identical AFTER the base urls
-  specified, so that any mirror can be used to fetch any file.
-
-FAILOVER
-
-  The failover mechanism is designed to be customized by subclassing
-  from MirrorGroup to change the details of the behavior.  In general,
-  the classes maintain a master mirror list and a "current mirror"
-  index.  When a download is initiated, a copy of this list and index
-  is created for that download only.  The specific failover policy
-  depends on the class used, and so is documented in the class
-  documentation.  Note that ANY behavior of the class can be
-  overridden, so any failover policy at all is possible (although
-  you may need to change the interface in extreme cases).
-
-CUSTOMIZATION
-
-  Most customization of a MirrorGroup object is done at instantiation
-  time (or via subclassing).  There are four major types of
-  customization:
-
-    1) Pass in a custom urlgrabber - The passed in urlgrabber will be
-       used (by default... see #2) for the grabs, so options to it
-       apply for the url-fetching
-
-    2) Custom mirror list - Mirror lists can simply be a list of
-       stings mirrors (as shown in the example above) but each can
-       also be a dict, allowing for more options.  For example, the
-       first mirror in the list above could also have been:
-
-         {'mirror': 'http://foo.com/some/directory/',
-          'grabber': <a custom grabber to be used for this mirror>,
-          'kwargs': { <a dict of arguments passed to the grabber> }}
-
-       All mirrors are converted to this format internally.  If
-       'grabber' is omitted, the default grabber will be used.  If
-       kwargs are omitted, then (duh) they will not be used.
-
-    3) Pass keyword arguments when instantiating the mirror group.
-       See, for example, the failure_callback argument.
-
-    4) Finally, any kwargs passed in for the specific file (to the
-       urlgrab method, for example) will be folded in.  The options
-       passed into the grabber's urlXXX methods will override any
-       options specified in a custom mirror dict.
-
-"""
-
-# $Id: mirror.py,v 1.12 2004/09/07 21:19:54 mstenner Exp $
-
-import random
-import thread  # needed for locking to make this threadsafe
-
-from grabber import URLGrabError, CallbackObject
-
-DEBUG=0
-def DBPRINT(*args): print ' '.join(args)
-
-try:
-    from i18n import _
-except ImportError, msg:
-    def _(st): return st
-
-class GrabRequest:
-    """This is a dummy class used to hold information about the specific
-    request.  For example, a single file.  By maintaining this information
-    separately, we can accomplish two things:
-
-      1) make it a little easier to be threadsafe
-      2) have request-specific parameters
-    """
-    pass
-
-class MirrorGroup:
-    """Base Mirror class
-
-    Instances of this class are built with a grabber object and a list
-    of mirrors.  Then all calls to urlXXX should be passed relative urls.
-    The requested file will be searched for on the first mirror.  If the
-    grabber raises an exception (possibly after some retries) then that
-    mirror will be removed from the list, and the next will be attempted.
-    If all mirrors are exhausted, then an exception will be raised.
-
-    MirrorGroup has the following failover policy:
-
-      * downloads begin with the first mirror
-
-      * by default (see default_action below) a failure (after retries)
-        causes it to increment the local AND master indices.  Also,
-        the current mirror is removed from the local list (but NOT the
-        master list - the mirror can potentially be used for other
-        files)
-
-      * if the local list is ever exhausted, a URLGrabError will be
-        raised (errno=256, no more mirrors)
-
-    OPTIONS
-
-      In addition to the required arguments "grabber" and "mirrors",
-      MirrorGroup also takes the following optional arguments:
-      
-      default_action
-
-        A dict that describes the actions to be taken upon failure
-        (after retries).  default_action can contain any of the
-        following keys (shown here with their default values):
-
-          default_action = {'increment': 1,
-                            'increment_master': 1,
-                            'remove': 1,
-                            'remove_master': 0,
-                            'fail': 0}
-
-        In this context, 'increment' means "use the next mirror" and
-        'remove' means "never use this mirror again".  The two
-        'master' values refer to the instance-level mirror list (used
-        for all files), whereas the non-master values refer to the
-        current download only.
-
-        The 'fail' option will cause immediate failure by re-raising
-        the exception and no further attempts to get the current
-        download.
-
-        This dict can be set at instantiation time,
-          mg = MirrorGroup(grabber, mirrors, default_action={'fail':1})
-        at method-execution time (only applies to current fetch),
-          filename = mg.urlgrab(url, default_action={'increment': 0})
-        or by returning an action dict from the failure_callback
-          return {'fail':0}
-        in increasing precedence.
-        
-        If all three of these were done, the net result would be:
-              {'increment': 0,         # set in method
-               'increment_master': 1,  # class default
-               'remove': 1,            # class default
-               'remove_master': 0,     # class default
-               'fail': 0}              # set at instantiation, reset
-                                       # from callback
-
-      failure_callback
-
-        this is a callback that will be called when a mirror "fails",
-        meaning the grabber raises some URLGrabError.  If this is a
-        tuple, it is interpreted to be of the form (cb, args, kwargs)
-        where cb is the actual callable object (function, method,
-        etc).  Otherwise, it is assumed to be the callable object
-        itself.  The callback will be passed a grabber.CallbackObject
-        instance along with args and kwargs (if present).  The following
-        attributes are defined withing the instance:
-
-           obj.exception    = < exception that was raised >
-           obj.mirror       = < the mirror that was tried >
-           obj.relative_url = < url relative to the mirror >
-           obj.url          = < full url that failed >
-                              # .url is just the combination of .mirror
-                              # and .relative_url
-
-        The failure callback can return an action dict, as described
-        above.
-
-        Like default_action, the failure_callback can be set at
-        instantiation time or when the urlXXX method is called.  In
-        the latter case, it applies only for that fetch.
-
-        The callback can re-raise the exception quite easily.  For
-        example, this is a perfectly adequate callback function:
-
-          def callback(obj): raise obj.exception
-
-        WARNING: do not save the exception object (or the
-        CallbackObject instance).  As they contain stack frame
-        references, they can lead to circular references.
-
-    Notes:
-      * The behavior can be customized by deriving and overriding the
-        'CONFIGURATION METHODS'
-      * The 'grabber' instance is kept as a reference, not copied.
-        Therefore, the grabber instance can be modified externally
-        and changes will take effect immediately.
-    """
-
-    # notes on thread-safety:
-
-    #   A GrabRequest should never be shared by multiple threads because
-    #   it's never saved inside the MG object and never returned outside it.
-    #   therefore, it should be safe to access/modify grabrequest data
-    #   without a lock.  However, accessing the mirrors and _next attributes
-    #   of the MG itself must be done when locked to prevent (for example)
-    #   removal of the wrong mirror.
-
-    ##############################################################
-    #  CONFIGURATION METHODS  -  intended to be overridden to
-    #                            customize behavior
-    def __init__(self, grabber, mirrors, **kwargs):
-        """Initialize the MirrorGroup object.
-
-        REQUIRED ARGUMENTS
-
-          grabber  - URLGrabber instance
-          mirrors  - a list of mirrors
-
-        OPTIONAL ARGUMENTS
-
-          failure_callback  - callback to be used when a mirror fails
-          default_action    - dict of failure actions
-
-        See the module-level and class level documentation for more
-        details.
-        """
-
-        # OVERRIDE IDEAS:
-        #   shuffle the list to randomize order
-        self.grabber = grabber
-        self.mirrors = self._parse_mirrors(mirrors)
-        self._next = 0
-        self._lock = thread.allocate_lock()
-        self.default_action = None
-        self._process_kwargs(kwargs)
-
-    # if these values are found in **kwargs passed to one of the urlXXX
-    # methods, they will be stripped before getting passed on to the
-    # grabber
-    options = ['default_action', 'failure_callback']
-    
-    def _process_kwargs(self, kwargs):
-        self.failure_callback = kwargs.get('failure_callback')
-        self.default_action   = kwargs.get('default_action')
-       
-    def _parse_mirrors(self, mirrors):
-        parsed_mirrors = []
-        for m in mirrors:
-            if type(m) == type(''): m = {'mirror': m}
-            parsed_mirrors.append(m)
-        return parsed_mirrors
-    
-    def _load_gr(self, gr):
-        # OVERRIDE IDEAS:
-        #   shuffle gr list
-        self._lock.acquire()
-        gr.mirrors = list(self.mirrors)
-        gr._next = self._next
-        self._lock.release()
-
-    def _get_mirror(self, gr):
-        # OVERRIDE IDEAS:
-        #   return a random mirror so that multiple mirrors get used
-        #   even without failures.
-        if not gr.mirrors:
-            raise URLGrabError(256, _('No more mirrors to try.'))
-        return gr.mirrors[gr._next]
-
-    def _failure(self, gr, cb_obj):
-        # OVERRIDE IDEAS:
-        #   inspect the error - remove=1 for 404, remove=2 for connection
-        #                       refused, etc. (this can also be done via
-        #                       the callback)
-        cb = gr.kw.get('failure_callback') or self.failure_callback
-        if cb:
-            if type(cb) == type( () ):
-                cb, args, kwargs = cb
-            else:
-                args, kwargs = (), {}
-            action = cb(cb_obj, *args, **kwargs) or {}
-        else:
-            action = {}
-        # XXXX - decide - there are two ways to do this
-        # the first is action-overriding as a whole - use the entire action
-        # or fall back on module level defaults
-        #action = action or gr.kw.get('default_action') or self.default_action
-        # the other is to fall through for each element in the action dict
-        a = dict(self.default_action or {})
-        a.update(gr.kw.get('default_action', {}))
-        a.update(action)
-        action = a
-        self.increment_mirror(gr, action)
-        if action and action.get('fail', 0): raise
-
-    def increment_mirror(self, gr, action={}):
-        """Tell the mirror object increment the mirror index
-
-        This increments the mirror index, which amounts to telling the
-        mirror object to use a different mirror (for this and future
-        downloads).
-
-        This is a SEMI-public method.  It will be called internally,
-        and you may never need to call it.  However, it is provided
-        (and is made public) so that the calling program can increment
-        the mirror choice for methods like urlopen.  For example, with
-        urlopen, there's no good way for the mirror group to know that
-        an error occurs mid-download (it's already returned and given
-        you the file object).
-        
-        remove  ---  can have several values
-           0   do not remove the mirror from the list
-           1   remove the mirror for this download only
-           2   remove the mirror permanently
-
-        beware of remove=0 as it can lead to infinite loops
-        """
-        badmirror = gr.mirrors[gr._next]
-
-        self._lock.acquire()
-        try:
-            ind = self.mirrors.index(badmirror)
-        except ValueError:
-            pass
-        else:
-            if action.get('remove_master', 0):
-                del self.mirrors[ind]
-            elif self._next == ind and action.get('increment_master', 1):
-                self._next += 1
-                if self._next >= len(self.mirrors): self._next = 0
-        self._lock.release()
-        
-        if action.get('remove', 1):
-            del gr.mirrors[gr._next]
-        elif action.get('increment', 1):
-            gr._next += 1
-            if gr._next >= len(gr.mirrors): gr._next = 0
-
-        if DEBUG:
-            grm = [m['mirror'] for m in gr.mirrors]
-            DBPRINT('GR   mirrors: [%s] %i' % (' '.join(grm), gr._next))
-            selfm = [m['mirror'] for m in self.mirrors]
-            DBPRINT('MAIN mirrors: [%s] %i' % (' '.join(selfm), self._next))
-
-    #####################################################################
-    # NON-CONFIGURATION METHODS
-    # these methods are designed to be largely workhorse methods that
-    # are not intended to be overridden.  That doesn't mean you can't;
-    # if you want to, feel free, but most things can be done by
-    # by overriding the configuration methods :)
-
-    def _join_url(self, base_url, rel_url):
-        if base_url.endswith('/') or rel_url.startswith('/'):
-            return base_url + rel_url
-        else:
-            return base_url + '/' + rel_url
-        
-    def _mirror_try(self, func, url, kw):
-        gr = GrabRequest()
-        gr.func = func
-        gr.url  = url
-        gr.kw   = dict(kw)
-        self._load_gr(gr)
-
-        for k in self.options:
-            try: del kw[k]
-            except KeyError: pass
-
-        while 1:
-            mirrorchoice = self._get_mirror(gr)
-            fullurl = self._join_url(mirrorchoice['mirror'], gr.url)
-            kwargs = dict(mirrorchoice.get('kwargs', {}))
-            kwargs.update(kw)
-            grabber = mirrorchoice.get('grabber') or self.grabber
-            func_ref = getattr(grabber, func)
-            if DEBUG: DBPRINT('MIRROR: trying %s -> %s' % (url, fullurl))
-            try:
-                return func_ref( *(fullurl,), **kwargs )
-            except URLGrabError, e:
-                if DEBUG: DBPRINT('MIRROR: failed')
-                obj = CallbackObject()
-                obj.exception = e
-                obj.mirror = mirrorchoice['mirror']
-                obj.relative_url = gr.url
-                obj.url = fullurl
-                self._failure(gr, obj)
-
-    def urlgrab(self, url, filename=None, **kwargs):
-        kw = dict(kwargs)
-        kw['filename'] = filename
-        func = 'urlgrab'
-        return self._mirror_try(func, url, kw)
-    
-    def urlopen(self, url, **kwargs):
-        kw = dict(kwargs)
-        func = 'urlopen'
-        return self._mirror_try(func, url, kw)
-
-    def urlread(self, url, limit=None, **kwargs):
-        kw = dict(kwargs)
-        kw['limit'] = limit
-        func = 'urlread'
-        return self._mirror_try(func, url, kw)
-            
-
-class MGRandomStart(MirrorGroup):
-    """A mirror group that starts at a random mirror in the list.
-
-    This behavior of this class is identical to MirrorGroup, except that
-    it starts at a random location in the mirror list.
-    """
-
-    def __init__(self, grabber, mirrors, **kwargs):
-        """Initialize the object
-
-        The arguments for intialization are the same as for MirrorGroup
-        """
-        MirrorGroup.__init__(self, grabber, mirrors, **kwargs)
-        self._next = random.randrange(len(mirrors))
-
-class MGRandomOrder(MirrorGroup):
-    """A mirror group that uses mirrors in a random order.
-
-    This behavior of this class is identical to MirrorGroup, except that
-    it uses the mirrors in a random order.  Note that the order is set at
-    initialization time and fixed thereafter.  That is, it does not pick a
-    random mirror after each failure.
-    """
-
-    def __init__(self, grabber, mirrors, **kwargs):
-        """Initialize the object
-
-        The arguments for intialization are the same as for MirrorGroup
-        """
-        MirrorGroup.__init__(self, grabber, mirrors, **kwargs)
-        random.shuffle(self.mirrors)
-
-if __name__ == '__main__':
-    pass

=== removed file 'bzrlib/util/urlgrabber/progress.py'
--- bzrlib/util/urlgrabber/progress.py	2005-09-19 06:05:19 +0000
+++ bzrlib/util/urlgrabber/progress.py	1970-01-01 00:00:00 +0000
@@ -1,531 +0,0 @@
-#   This library is free software; you can redistribute it and/or
-#   modify it under the terms of the GNU Lesser General Public
-#   License as published by the Free Software Foundation; either
-#   version 2.1 of the License, or (at your option) any later version.
-#
-#   This library is distributed in the hope that it will be useful,
-#   but WITHOUT ANY WARRANTY; without even the implied warranty of
-#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#   Lesser General Public License for more details.
-#
-#   You should have received a copy of the GNU Lesser General Public
-#   License along with this library; if not, write to the 
-#      Free Software Foundation, Inc., 
-#      59 Temple Place, Suite 330, 
-#      Boston, MA  02111-1307  USA
-
-# This file is part of urlgrabber, a high-level cross-protocol url-grabber
-# Copyright 2002-2004 Michael D. Stenner, Ryan Tomayko
-
-# $Id: progress.py,v 1.5 2005/01/14 18:21:41 rtomayko Exp $
-
-import sys
-import time
-import math
-import thread
-    
-class BaseMeter:
-    def __init__(self):
-        self.update_period = 0.3 # seconds
-
-        self.filename   = None
-        self.url        = None
-        self.basename   = None
-        self.text       = None
-        self.size       = None
-        self.start_time = None
-        self.last_amount_read = 0
-        self.last_update_time = None
-        self.re = RateEstimator()
-        
-    def start(self, filename=None, url=None, basename=None,
-              size=None, now=None, text=None):
-        self.filename = filename
-        self.url      = url
-        self.basename = basename
-        self.text     = text
-
-        #size = None #########  TESTING
-        self.size = size
-        if not size is None: self.fsize = format_number(size) + 'B'
-
-        if now is None: now = time.time()
-        self.start_time = now
-        self.re.start(size, now)
-        self.last_amount_read = 0
-        self.last_update_time = now
-        self._do_start(now)
-        
-    def _do_start(self, now=None):
-        pass
-
-    def update(self, amount_read, now=None):
-        # for a real gui, you probably want to override and put a call
-        # to your mainloop iteration function here
-        if now is None: now = time.time()
-        if (now >= self.last_update_time + self.update_period) or \
-               not self.last_update_time:
-            self.re.update(amount_read, now)
-            self.last_amount_read = amount_read
-            self.last_update_time = now
-            self._do_update(amount_read, now)
-
-    def _do_update(self, amount_read, now=None):
-        pass
-
-    def end(self, amount_read, now=None):
-        if now is None: now = time.time()
-        self.re.update(amount_read, now)
-        self.last_amount_read = amount_read
-        self.last_update_time = now
-        self._do_end(amount_read, now)
-
-    def _do_end(self, amount_read, now=None):
-        pass
-        
-class TextMeter(BaseMeter):
-    def __init__(self, fo=sys.stderr):
-        BaseMeter.__init__(self)
-        self.fo = fo
-
-    def _do_update(self, amount_read, now=None):
-        etime = self.re.elapsed_time()
-        fetime = format_time(etime)
-        fread = format_number(amount_read)
-        #self.size = None
-        if self.text is not None:
-            text = self.text
-        else:
-            text = self.basename
-        if self.size is None:
-            out = '\r%-60.60s    %5sB %s ' % \
-                  (text, fread, fetime)
-        else:
-            rtime = self.re.remaining_time()
-            frtime = format_time(rtime)
-            frac = self.re.fraction_read()
-            bar = '='*int(25 * frac)
-
-            out = '\r%-25.25s %3i%% |%-25.25s| %5sB %8s ETA ' % \
-                  (text, frac*100, bar, fread, frtime)
-
-        self.fo.write(out)
-        self.fo.flush()
-
-    def _do_end(self, amount_read, now=None):
-        total_time = format_time(self.re.elapsed_time())
-        total_size = format_number(amount_read)
-        if self.text is not None:
-            text = self.text
-        else:
-            text = self.basename
-        if self.size is None:
-            out = '\r%-60.60s    %5sB %s ' % \
-                  (text, total_size, total_time)
-        else:
-            bar = '='*25
-            out = '\r%-25.25s %3i%% |%-25.25s| %5sB %8s     ' % \
-                  (text, 100, bar, total_size, total_time)
-        self.fo.write(out + '\n')
-        self.fo.flush()
-
-text_progress_meter = TextMeter
-
-class MultiFileHelper(BaseMeter):
-    def __init__(self, master):
-        BaseMeter.__init__(self)
-        self.master = master
-
-    def _do_start(self, now):
-        self.master.start_meter(self, now)
-
-    def _do_update(self, amount_read, now):
-        # elapsed time since last update
-        self.master.update_meter(self, now)
-
-    def _do_end(self, amount_read, now):
-        self.ftotal_time = format_time(now - self.start_time)
-        self.ftotal_size = format_number(self.last_amount_read)
-        self.master.end_meter(self, now)
-
-    def failure(self, message, now=None):
-        self.master.failure_meter(self, message, now)
-
-    def message(self, message):
-        self.master.message_meter(self, message)
-
-class MultiFileMeter:
-    helperclass = MultiFileHelper
-    def __init__(self):
-        self.meters = []
-        self.in_progress_meters = []
-        self._lock = thread.allocate_lock()
-        self.update_period = 0.3 # seconds
-        
-        self.numfiles         = None
-        self.finished_files   = 0
-        self.failed_files     = 0
-        self.open_files       = 0
-        self.total_size       = None
-        self.failed_size      = 0
-        self.start_time       = None
-        self.finished_file_size = 0
-        self.last_update_time = None
-        self.re = RateEstimator()
-
-    def start(self, numfiles=None, total_size=None, now=None):
-        if now is None: now = time.time()
-        self.numfiles         = numfiles
-        self.finished_files   = 0
-        self.failed_files     = 0
-        self.open_files       = 0
-        self.total_size       = total_size
-        self.failed_size      = 0
-        self.start_time       = now
-        self.finished_file_size = 0
-        self.last_update_time = now
-        self.re.start(total_size, now)
-        self._do_start(now)
-
-    def _do_start(self, now):
-        pass
-
-    def end(self, now=None):
-        if now is None: now = time.time()
-        self._do_end(now)
-        
-    def _do_end(self, now):
-        pass
-
-    def lock(self): self._lock.acquire()
-    def unlock(self): self._lock.release()
-
-    ###########################################################
-    # child meter creation and destruction
-    def newMeter(self):
-        newmeter = self.helperclass(self)
-        self.meters.append(newmeter)
-        return newmeter
-    
-    def removeMeter(self, meter):
-        self.meters.remove(meter)
-        
-    ###########################################################
-    # child functions - these should only be called by helpers
-    def start_meter(self, meter, now):
-        if not meter in self.meters:
-            raise ValueError('attempt to use orphaned meter')
-        self._lock.acquire()
-        try:
-            if not meter in self.in_progress_meters:
-                self.in_progress_meters.append(meter)
-                self.open_files += 1
-        finally:
-            self._lock.release()
-        self._do_start_meter(meter, now)
-        
-    def _do_start_meter(self, meter, now):
-        pass
-        
-    def update_meter(self, meter, now):
-        if not meter in self.meters:
-            raise ValueError('attempt to use orphaned meter')
-        if (now >= self.last_update_time + self.update_period) or \
-               not self.last_update_time:
-            self.re.update(self._amount_read(), now)
-            self.last_update_time = now
-            self._do_update_meter(meter, now)
-
-    def _do_update_meter(self, meter, now):
-        pass
-
-    def end_meter(self, meter, now):
-        if not meter in self.meters:
-            raise ValueError('attempt to use orphaned meter')
-        self._lock.acquire()
-        try:
-            try: self.in_progress_meters.remove(meter)
-            except ValueError: pass
-            self.open_files     -= 1
-            self.finished_files += 1
-            self.finished_file_size += meter.last_amount_read
-        finally:
-            self._lock.release()
-        self._do_end_meter(meter, now)
-
-    def _do_end_meter(self, meter, now):
-        pass
-
-    def failure_meter(self, meter, message, now):
-        if not meter in self.meters:
-            raise ValueError('attempt to use orphaned meter')
-        self._lock.acquire()
-        try:
-            try: self.in_progress_meters.remove(meter)
-            except ValueError: pass
-            self.open_files     -= 1
-            self.failed_files   += 1
-            if meter.size and self.failed_size is not None:
-                self.failed_size += meter.size
-            else:
-                self.failed_size = None
-        finally:
-            self._lock.release()
-        self._do_failure_meter(meter, message, now)
-
-    def _do_failure_meter(self, meter, message, now):
-        pass
-
-    def message_meter(self, meter, message):
-        pass
-
-    ########################################################
-    # internal functions
-    def _amount_read(self):
-        tot = self.finished_file_size
-        for m in self.in_progress_meters:
-            tot += m.last_amount_read
-        return tot
-
-
-class TextMultiFileMeter(MultiFileMeter):
-    def __init__(self, fo=sys.stderr):
-        self.fo = fo
-        MultiFileMeter.__init__(self)
-
-    # files: ###/### ###%  data: ######/###### ###%  time: ##:##:##/##:##:##
-    def _do_update_meter(self, meter, now):
-        self._lock.acquire()
-        try:
-            format = "files: %3i/%-3i %3i%%   data: %6.6s/%-6.6s %3i%%   " \
-                     "time: %8.8s/%8.8s"
-            df = self.finished_files
-            tf = self.numfiles or 1
-            pf = 100 * float(df)/tf + 0.49
-            dd = self.re.last_amount_read
-            td = self.total_size
-            pd = 100 * (self.re.fraction_read() or 0) + 0.49
-            dt = self.re.elapsed_time()
-            rt = self.re.remaining_time()
-            if rt is None: tt = None
-            else: tt = dt + rt
-
-            fdd = format_number(dd) + 'B'
-            ftd = format_number(td) + 'B'
-            fdt = format_time(dt, 1)
-            ftt = format_time(tt, 1)
-            
-            out = '%-79.79s' % (format % (df, tf, pf, fdd, ftd, pd, fdt, ftt))
-            self.fo.write('\r' + out)
-            self.fo.flush()
-        finally:
-            self._lock.release()
-
-    def _do_end_meter(self, meter, now):
-        self._lock.acquire()
-        try:
-            format = "%-30.30s %6.6s    %8.8s    %9.9s"
-            fn = meter.basename
-            size = meter.last_amount_read
-            fsize = format_number(size) + 'B'
-            et = meter.re.elapsed_time()
-            fet = format_time(et, 1)
-            frate = format_number(size / et) + 'B/s'
-            
-            out = '%-79.79s' % (format % (fn, fsize, fet, frate))
-            self.fo.write('\r' + out + '\n')
-        finally:
-            self._lock.release()
-        self._do_update_meter(meter, now)
-
-    def _do_failure_meter(self, meter, message, now):
-        self._lock.acquire()
-        try:
-            format = "%-30.30s %6.6s %s"
-            fn = meter.basename
-            if type(message) in (type(''), type(u'')):
-                message = message.splitlines()
-            if not message: message = ['']
-            out = '%-79s' % (format % (fn, 'FAILED', message[0] or ''))
-            self.fo.write('\r' + out + '\n')
-            for m in message[1:]: self.fo.write('  ' + m + '\n')
-            self._lock.release()
-        finally:
-            self._do_update_meter(meter, now)
-
-    def message_meter(self, meter, message):
-        self._lock.acquire()
-        try:
-            pass
-        finally:
-            self._lock.release()
-
-    def _do_end(self, now):
-        self._do_update_meter(None, now)
-        self._lock.acquire()
-        try:
-            self.fo.write('\n')
-            self.fo.flush()
-        finally:
-            self._lock.release()
-        
-######################################################################
-# support classes and functions
-
-class RateEstimator:
-    def __init__(self, timescale=5.0):
-        self.timescale = timescale
-
-    def start(self, total=None, now=None):
-        if now is None: now = time.time()
-        self.total = total
-        self.start_time = now
-        self.last_update_time = now
-        self.last_amount_read = 0
-        self.ave_rate = None
-        
-    def update(self, amount_read, now=None):
-        if now is None: now = time.time()
-        if amount_read == 0:
-            # if we just started this file, all bets are off
-            self.last_update_time = now
-            self.last_amount_read = 0
-            self.ave_rate = None
-            return
-
-        #print 'times', now, self.last_update_time
-        time_diff = now         - self.last_update_time
-        read_diff = amount_read - self.last_amount_read
-        self.last_update_time = now
-        self.last_amount_read = amount_read
-        self.ave_rate = self._temporal_rolling_ave(\
-            time_diff, read_diff, self.ave_rate, self.timescale)
-        #print 'results', time_diff, read_diff, self.ave_rate
-        
-    #####################################################################
-    # result methods
-    def average_rate(self):
-        "get the average transfer rate (in bytes/second)"
-        return self.ave_rate
-
-    def elapsed_time(self):
-        "the time between the start of the transfer and the most recent update"
-        return self.last_update_time - self.start_time
-
-    def remaining_time(self):
-        "estimated time remaining"
-        if not self.ave_rate or not self.total: return None
-        return (self.total - self.last_amount_read) / self.ave_rate
-
-    def fraction_read(self):
-        """the fraction of the data that has been read
-        (can be None for unknown transfer size)"""
-        if self.total is None: return None
-        elif self.total == 0: return 1.0
-        else: return float(self.last_amount_read)/self.total
-
-    #########################################################################
-    # support methods
-    def _temporal_rolling_ave(self, time_diff, read_diff, last_ave, timescale):
-        """a temporal rolling average performs smooth averaging even when
-        updates come at irregular intervals.  This is performed by scaling
-        the "epsilon" according to the time since the last update.
-        Specifically, epsilon = time_diff / timescale
-
-        As a general rule, the average will take on a completely new value
-        after 'timescale' seconds."""
-        epsilon = time_diff / timescale
-        if epsilon > 1: epsilon = 1.0
-        return self._rolling_ave(time_diff, read_diff, last_ave, epsilon)
-    
-    def _rolling_ave(self, time_diff, read_diff, last_ave, epsilon):
-        """perform a "rolling average" iteration
-        a rolling average "folds" new data into an existing average with
-        some weight, epsilon.  epsilon must be between 0.0 and 1.0 (inclusive)
-        a value of 0.0 means only the old value (initial value) counts,
-        and a value of 1.0 means only the newest value is considered."""
-        
-        try:
-            recent_rate = read_diff / time_diff
-        except ZeroDivisionError:
-            recent_rate = None
-        if last_ave is None: return recent_rate
-        elif recent_rate is None: return last_ave
-
-        # at this point, both last_ave and recent_rate are numbers
-        return epsilon * recent_rate  +  (1 - epsilon) * last_ave
-
-    def _round_remaining_time(self, rt, start_time=15.0):
-        """round the remaining time, depending on its size
-        If rt is between n*start_time and (n+1)*start_time round downward
-        to the nearest multiple of n (for any counting number n).
-        If rt < start_time, round down to the nearest 1.
-        For example (for start_time = 15.0):
-         2.7  -> 2.0
-         25.2 -> 25.0
-         26.4 -> 26.0
-         35.3 -> 34.0
-         63.6 -> 60.0
-        """
-
-        if rt < 0: return 0.0
-        shift = int(math.log(rt/start_time)/math.log(2))
-        rt = int(rt)
-        if shift <= 0: return rt
-        return float(int(rt) >> shift << shift)
-        
-
-def format_time(seconds, use_hours=0):
-    if seconds is None or seconds < 0:
-        if use_hours: return '--:--:--'
-        else:         return '--:--'
-    else:
-        seconds = int(seconds)
-        minutes = seconds / 60
-        seconds = seconds % 60
-        if use_hours:
-            hours = minutes / 60
-            minutes = minutes % 60
-            return '%02i:%02i:%02i' % (hours, minutes, seconds)
-        else:
-            return '%02i:%02i' % (minutes, seconds)
-            
-def format_number(number, SI=0, space=' '):
-    """Turn numbers into human-readable metric-like numbers"""
-    symbols = ['',  # (none)
-               'k', # kilo
-               'M', # mega
-               'G', # giga
-               'T', # tera
-               'P', # peta
-               'E', # exa
-               'Z', # zetta
-               'Y'] # yotta
-    
-    if SI: step = 1000.0
-    else: step = 1024.0
-
-    thresh = 999
-    depth = 0
-    
-    # we want numbers between 
-    while number > thresh:
-        depth  = depth + 1
-        number = number / step
-        
-    # just in case someone needs more than 1000 yottabytes!
-    diff = depth - len(symbols) + 1
-    if diff > 0:
-        depth = depth - diff
-        number = number * thresh**depth
-
-    if type(number) == type(1) or type(number) == type(1L):
-        format = '%i%s%s'
-    elif number < 9.95:
-        # must use 9.95 for proper sizing.  For example, 9.99 will be
-        # rounded to 10.0 with the .1f format string (which is too long)
-        format = '%.1f%s%s'
-    else:
-        format = '%.0f%s%s'
-        
-    return(format % (float(number or 0), space, symbols[depth]))

=== added file 'bzrlib/version.py'
--- bzrlib/version.py	1970-01-01 00:00:00 +0000
+++ bzrlib/version.py	2006-08-16 02:53:40 +0000
@@ -0,0 +1,67 @@
+# Copyright (C) 2004, 2005, 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Report on version of bzrlib"""
+
+import os
+import sys
+
+import bzrlib
+from bzrlib import errors, osutils
+from bzrlib.branch import Branch
+
+
+def show_version():
+    print "Bazaar (bzr) %s" % bzrlib.__version__
+    # is bzrlib itself in a branch?
+    src_tree = _get_bzr_source_tree()
+    if src_tree:
+        src_revision_id = src_tree.last_revision()
+        revno = src_tree.branch.revision_id_to_revno(src_revision_id)
+        print "  from bzr checkout", src_tree.basedir
+        print "    revision:", revno
+        print "    revid:", src_revision_id
+        print "    branch nick:", src_tree.branch.nick
+    print "Using python interpreter:", sys.executable
+    import site
+    print "Using python standard library:", os.path.dirname(site.__file__)
+    print "Using bzrlib:",
+    if len(bzrlib.__path__) > 1:
+        # print repr, which is a good enough way of making it clear it's
+        # more than one element (eg ['/foo/bar', '/foo/bzr'])
+        print repr(bzrlib.__path__)
+    else:
+        print bzrlib.__path__[0]
+
+    print
+    print bzrlib.__copyright__
+    print "http://bazaar-vcs.org/"
+    print
+    print "bzr comes with ABSOLUTELY NO WARRANTY.  bzr is free software, and"
+    print "you may use, modify and redistribute it under the terms of the GNU"
+    print "General Public License version 2 or later."
+
+
+def _get_bzr_source_tree():
+    """Return the WorkingTree for bzr source, if any.
+
+    If bzr is not being run from its working tree, returns None.
+    """
+    try:
+        from bzrlib.workingtree import WorkingTree
+        return WorkingTree.open_containing(__file__)[0]
+    except errors.NotBranchError:
+        return None

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-08-17 14:16:46 +0000
+++ bzrlib/workingtree.py	2006-08-22 10:50:58 +0000
@@ -394,7 +394,11 @@
         return pathjoin(self.basedir, filename)
     
     def basis_tree(self):
-        """Return RevisionTree for the current last revision."""
+        """Return RevisionTree for the current last revision.
+        
+        If the left most parent is a ghost then the returned tree will be an
+        empty tree - one obtained by calling repository.revision_tree(None).
+        """
         revision_id = self.last_revision()
         if revision_id is not None:
             try:
@@ -415,7 +419,7 @@
             # its a ghost.
             if self.branch.repository.has_revision(revision_id):
                 raise
-            # the basis tree is a ghost
+            # the basis tree is a ghost so return an empty tree.
             return self.branch.repository.revision_tree(None)
 
     @staticmethod
@@ -661,7 +665,9 @@
         and setting the list to its value plus revision_id.
 
         :param revision_id: The revision id to add to the parent list. It may
-        be a ghost revision.
+        be a ghost revision as long as its not the first parent to be added,
+        or the allow_leftmost_as_ghost parameter is set True.
+        :param allow_leftmost_as_ghost: Allow the first parent to be a ghost.
         """
         self.set_parent_ids(self.get_parent_ids() + [revision_id],
             allow_leftmost_as_ghost=allow_leftmost_as_ghost)
@@ -676,7 +682,9 @@
         simpler to use that api. If you have the parent already available, using
         this api is preferred.
 
-        :param parent_tuple: The (revision id, tree) to add to the parent list.             If the revision_id is a ghost, pass None for the tree.
+        :param parent_tuple: The (revision id, tree) to add to the parent list.
+            If the revision_id is a ghost, pass None for the tree.
+        :param allow_leftmost_as_ghost: Allow the first parent to be a ghost.
         """
         self.set_parent_ids(self.get_parent_ids() + [parent_tuple[0]],
             allow_leftmost_as_ghost=allow_leftmost_as_ghost)
@@ -734,7 +742,6 @@
                     (rev_id, self.branch.repository.revision_tree(rev_id)))
             except errors.RevisionNotPresent:
                 trees.append((rev_id, None))
-                pass
         self.set_parent_trees(trees,
             allow_leftmost_as_ghost=allow_leftmost_as_ghost)
 
@@ -746,12 +753,12 @@
             If tree is None, then that element is treated as an unreachable
             parent tree - i.e. a ghost.
         """
-        parent = parents_list[:1]
-        if len(parent):
+        if len(parents_list) > 0:
+            leftmost_id = parents_list[0][0]
             if (not allow_leftmost_as_ghost and not
-                self.branch.repository.has_revision(parent[0][0])):
-                raise errors.GhostRevision(parent[0][0])
-            self.set_last_revision(parent[0][0])
+                self.branch.repository.has_revision(leftmost_id)):
+                raise errors.GhostRevisionUnusableHere(leftmost_id)
+            self.set_last_revision(leftmost_id)
         else:
             self.set_last_revision(None)
         merges = parents_list[1:]
@@ -1504,50 +1511,66 @@
         Do a 'normal' merge of the old branch basis if it is relevant.
         """
         old_tip = self.branch.update()
-        try:
-            result = 0
-            if self.last_revision() != self.branch.last_revision():
-                # merge tree state up to new branch tip.
-                basis = self.basis_tree()
-                to_tree = self.branch.basis_tree()
-                result += merge_inner(self.branch,
-                                      to_tree,
-                                      basis,
-                                      this_tree=self)
-                self.set_last_revision(self.branch.last_revision())
-                # TODO - dedup parents list with things merged by pull ?
-                # reuse the tree we've updated to to set the basis:
-                parent_trees = [(self.branch.last_revision(), to_tree)]
-                merges = self.get_parent_ids()[1:]
-                # pull the other trees out of the repository. This could be
-                # better expressed - for instance by inserting a parent, and
-                # that would remove duplication.
-                parent_trees.extend([
-                    (parent, self.branch.repository.revision_tree(parent)) for
-                     parent in merges])
-                self.set_parent_trees(parent_trees)
-            if old_tip and old_tip != self.last_revision():
-                # our last revision was not the prior branch last revision
-                # and we have converted that last revision to a pending merge.
-                # base is somewhere between the branch tip now
-                # and the now pending merge
-                from bzrlib.revision import common_ancestor
-                try:
-                    base_rev_id = common_ancestor(self.branch.last_revision(),
-                                                  old_tip,
-                                                  self.branch.repository)
-                except errors.NoCommonAncestor:
-                    base_rev_id = None
-                base_tree = self.branch.repository.revision_tree(base_rev_id)
-                other_tree = self.branch.repository.revision_tree(old_tip)
-                result += merge_inner(self.branch,
-                                      other_tree,
-                                      base_tree,
-                                      this_tree=self)
-            return result
-        finally:
+        # here if old_tip is not None, it is the old tip of the branch before
+        # it was updated from the master branch. This should become a pending
+        # merge in the working tree to preserve the user existing work.  we
+        # cant set that until we update the working trees last revision to be
+        # one from the new branch, because it will just get absorbed by the
+        # parent de-duplication logic.
+        # 
+        # We MUST save it even if an error occurs, because otherwise the users
+        # local work is unreferenced and will appear to have been lost.
+        # 
+        result = 0
+        if self.last_revision() != self.branch.last_revision():
+            # merge tree state up to new branch tip.
+            basis = self.basis_tree()
+            to_tree = self.branch.basis_tree()
+            result += merge_inner(self.branch,
+                                  to_tree,
+                                  basis,
+                                  this_tree=self)
+            # TODO - dedup parents list with things merged by pull ?
+            # reuse the tree we've updated to to set the basis:
+            parent_trees = [(self.branch.last_revision(), to_tree)]
+            merges = self.get_parent_ids()[1:]
+            # Ideally we ask the tree for the trees here, that way the working
+            # tree can decide whether to give us teh entire tree or give us a
+            # lazy initialised tree. dirstate for instance will have the trees
+            # in ram already, whereas a last-revision + basis-inventory tree
+            # will not, but also does not need them when setting parents.
+            for parent in merges:
+                parent_trees.append(
+                    (parent, self.branch.repository.revision_tree(parent)))
+            if old_tip is not None:
+                parent_trees.append(
+                    (old_tip, self.branch.repository.revision_tree(old_tip)))
+            self.set_parent_trees(parent_trees)
+        else:
+            # the working tree had the same last-revision as the master
+            # branch did. We may still have pivot local work from the local
+            # branch into old_tip:
             if old_tip is not None:
                 self.add_pending_merge(old_tip)
+        if old_tip and old_tip != self.last_revision():
+            # our last revision was not the prior branch last revision
+            # and we have converted that last revision to a pending merge.
+            # base is somewhere between the branch tip now
+            # and the now pending merge
+            from bzrlib.revision import common_ancestor
+            try:
+                base_rev_id = common_ancestor(self.branch.last_revision(),
+                                              old_tip,
+                                              self.branch.repository)
+            except errors.NoCommonAncestor:
+                base_rev_id = None
+            base_tree = self.branch.repository.revision_tree(base_rev_id)
+            other_tree = self.branch.repository.revision_tree(old_tip)
+            result += merge_inner(self.branch,
+                                  other_tree,
+                                  base_tree,
+                                  this_tree=self)
+        return result
 
     @needs_write_lock
     def _write_inventory(self, inv):

=== modified file 'bzrlib/xml5.py'
--- bzrlib/xml5.py	2006-08-10 01:30:06 +0000
+++ bzrlib/xml5.py	2006-08-17 14:00:59 +0000
@@ -14,17 +14,81 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import cStringIO
+import re
 
 from bzrlib import (
     cache_utf8,
+    inventory,
     )
 from bzrlib.xml_serializer import SubElement, Element, Serializer
 from bzrlib.inventory import ROOT_ID, Inventory, InventoryEntry
-import bzrlib.inventory as inventory
 from bzrlib.revision import Revision
 from bzrlib.errors import BzrError
 
 
+_utf8_re = None
+_utf8_escape_map = {
+    "&":'&amp;',
+    "'":"&apos;", # FIXME: overkill
+    "\"":"&quot;",
+    "<":"&lt;",
+    ">":"&gt;",
+    }
+
+
+def _ensure_utf8_re():
+    """Make sure the _utf8_re regex has been compiled"""
+    global _utf8_re
+    if _utf8_re is not None:
+        return
+    _utf8_re = re.compile(u'[&<>\'\"\u0080-\uffff]')
+
+
+def _utf8_escape_replace(match, _map=_utf8_escape_map):
+    """Replace a string of non-ascii, non XML safe characters with their escape
+
+    This will escape both Standard XML escapes, like <>"', etc.
+    As well as escaping non ascii characters, because ElementTree did.
+    This helps us remain compatible to older versions of bzr. We may change
+    our policy in the future, though.
+    """
+    # jam 20060816 Benchmarks show that try/KeyError is faster if you
+    # expect the entity to rarely miss. There is about a 10% difference
+    # in overall time. But if you miss frequently, then if None is much
+    # faster. For our use case, we *rarely* have a revision id, file id
+    # or path name that is unicode. So use try/KeyError.
+    try:
+        return _map[match.group()]
+    except KeyError:
+        return "&#%d;" % ord(match.group())
+
+
+_unicode_to_escaped_map = {}
+
+def _encode_and_escape(unicode_str, _map=_unicode_to_escaped_map):
+    """Encode the string into utf8, and escape invalid XML characters"""
+    # We frequently get entities we have not seen before, so it is better
+    # to check if None, rather than try/KeyError
+    text = _map.get(unicode_str)
+    if text is None:
+        # The alternative policy is to do a regular UTF8 encoding
+        # and then escape only XML meta characters.
+        # Performance is equivalent once you use cache_utf8. *However*
+        # this makes the serialized texts incompatible with old versions
+        # of bzr. So no net gain. (Perhaps the read code would handle utf8
+        # better than entity escapes, but cElementTree seems to do just fine
+        # either way)
+        text = str(_utf8_re.sub(_utf8_escape_replace, unicode_str)) + '"'
+        _map[unicode_str] = text
+    return text
+
+
+def _clear_cache():
+    """Clean out the unicode => escaped map"""
+    _unicode_to_escaped_map.clear()
+
+
 class Serializer_v5(Serializer):
     """Version 5 serializer
 
@@ -32,50 +96,78 @@
     """
     
     __slots__ = []
-    
-    def _pack_inventory(self, inv):
-        """Convert to XML Element"""
+
+    def write_inventory_to_string(self, inv):
+        """Just call write_inventory with a StringIO and return the value"""
+        sio = cStringIO.StringIO()
+        self.write_inventory(inv, sio)
+        return sio.getvalue()
+
+    def write_inventory(self, inv, f):
+        """Write inventory to a file.
+        
+        :param inv: the inventory to write.
+        :param f: the file to write.
+        """
+        _ensure_utf8_re()
+        output = []
+        append = output.append
+        self._append_inventory_root(append, inv)
         entries = inv.iter_entries()
-        e = Element('inventory',
-                    format='5')
-        e.text = '\n'
-        path, root = entries.next()
-        if root.file_id not in (None, ROOT_ID):
-            e.set('file_id', root.file_id)
+        # Skip the root
+        root_path, root_ie = entries.next()
+        for path, ie in entries:
+            self._append_entry(append, ie)
+        append('</inventory>\n')
+        f.writelines(output)
+        # Just to keep the cache from growing without bounds
+        # but we may actually not want to do clear the cache
+        #_clear_cache()
+
+    def _append_inventory_root(self, append, inv):
+        """Append the inventory root to output."""
+        append('<inventory')
+        if inv.root.file_id not in (None, ROOT_ID):
+            append(' file_id="')
+            append(_encode_and_escape(inv.root.file_id))
+        append(' format="5"')
         if inv.revision_id is not None:
-            e.set('revision_id', inv.revision_id)
-        for path, ie in entries:
-            e.append(self._pack_entry(ie))
-        return e
-
-    def _pack_entry(self, ie):
-        """Convert InventoryEntry to XML element"""
+            append(' revision_id="')
+            append(_encode_and_escape(inv.revision_id))
+        append('>\n')
+        
+    def _append_entry(self, append, ie):
+        """Convert InventoryEntry to XML element and append to output."""
         # TODO: should just be a plain assertion
-        if not InventoryEntry.versionable_kind(ie.kind):
-            raise AssertionError('unsupported entry kind %s' % ie.kind)
-        e = Element(ie.kind)
-        e.set('name', ie.name)
-        e.set('file_id', ie.file_id)
-
-        if ie.text_size != None:
-            e.set('text_size', '%d' % ie.text_size)
-
-        for f in ['text_sha1', 'revision', 'symlink_target']:
-            v = getattr(ie, f)
-            if v != None:
-                e.set(f, v)
-
+        assert InventoryEntry.versionable_kind(ie.kind), \
+            'unsupported entry kind %s' % ie.kind
+
+        append("<")
+        append(ie.kind)
         if ie.executable:
-            e.set('executable', 'yes')
-
-        # to be conservative, we don't externalize the root pointers
-        # for now, leaving them as null in the xml form.  in a future
-        # version it will be implied by nested elements.
+            append(' executable="yes"')
+        append(' file_id="')
+        append(_encode_and_escape(ie.file_id))
+        append(' name="')
+        append(_encode_and_escape(ie.name))
         if ie.parent_id != ROOT_ID:
             assert isinstance(ie.parent_id, basestring)
-            e.set('parent_id', ie.parent_id)
-        e.tail = '\n'
-        return e
+            append(' parent_id="')
+            append(_encode_and_escape(ie.parent_id))
+        if ie.revision is not None:
+            append(' revision="')
+            append(_encode_and_escape(ie.revision))
+        if ie.symlink_target is not None:
+            append(' symlink_target="')
+            append(_encode_and_escape(ie.symlink_target))
+        if ie.text_sha1 is not None:
+            append(' text_sha1="')
+            append(ie.text_sha1)
+            append('"')
+        if ie.text_size is not None:
+            append(' text_size="%d"' % ie.text_size)
+        append(" />\n")
+        return
 
     def _pack_revision(self, rev):
         """Revision object -> xml tree"""
@@ -86,7 +178,7 @@
                        inventory_sha1 = rev.inventory_sha1,
                        format='5',
                        )
-        if rev.timezone:
+        if rev.timezone is not None:
             root.set('timezone', str(rev.timezone))
         root.text = '\n'
         msg = SubElement(root, 'message')
@@ -104,7 +196,6 @@
             self._pack_revision_properties(rev, root)
         return root
 
-
     def _pack_revision_properties(self, rev, under_element):
         top_elt = SubElement(under_element, 'properties')
         for prop_name, prop_value in sorted(rev.properties.items()):
@@ -116,7 +207,6 @@
             prop_elt.tail = '\n'
         top_elt.tail = '\n'
 
-
     def _unpack_inventory(self, elt):
         """Construct from XML Element
         """
@@ -138,7 +228,6 @@
             inv.add(ie)
         return inv
 
-
     def _unpack_entry(self, elt):
         kind = elt.tag
         if not InventoryEntry.versionable_kind(kind):
@@ -149,8 +238,15 @@
         parent_id = elt.get('parent_id')
         if parent_id == None:
             parent_id = ROOT_ID
-        parent_id = get_cached(parent_id)
-        file_id = get_cached(elt.get('file_id'))
+        # TODO: jam 20060817 At present, caching file ids costs us too 
+        #       much time. It slows down overall read performances from
+        #       approx 500ms to 700ms. And doesn't improve future reads.
+        #       it might be because revision ids and file ids are mixing.
+        #       Consider caching *just* the file ids, for a limited period
+        #       of time.
+        #parent_id = get_cached(parent_id)
+        #file_id = get_cached(elt.get('file_id'))
+        file_id = elt.get('file_id')
 
         if kind == 'directory':
             ie = inventory.InventoryDirectory(file_id,
@@ -179,7 +275,6 @@
 
         return ie
 
-
     def _unpack_revision(self, elt):
         """XML Element -> Revision object"""
         assert elt.tag == 'revision'
@@ -201,11 +296,13 @@
             rev.parent_ids.append(get_cached(p.get('revision_id')))
         self._unpack_revision_properties(elt, rev)
         v = elt.get('timezone')
-        rev.timezone = v and int(v)
+        if v is None:
+            rev.timezone = 0
+        else:
+            rev.timezone = int(v)
         rev.message = elt.findtext('message') # text of <message>
         return rev
 
-
     def _unpack_revision_properties(self, elt, rev):
         """Unpack properties onto a revision."""
         props_elt = elt.find('properties')

=== modified file 'doc/tutorial.txt'
--- doc/tutorial.txt	2006-07-30 07:23:36 +0000
+++ doc/tutorial.txt	2006-08-16 13:54:49 +0000
@@ -528,3 +528,26 @@
 
 * The rspush plugin that comes with BzrTools
 
+
+Moving changes between trees 
+============================
+
+It happens to the best of us: sometimes you'll make changes in the wrong
+tree.  Maybe because you've accidentally started work in the wrong directory,
+maybe because as you're working, the change turns out to be bigger than you
+expected, so you start a new branch for it.
+
+To move your changes from one tree to another, use
+
+::
+
+  % cd NEWDIR
+  % bzr merge --uncommitted OLDDIR
+
+This will apply all of the uncommitted changes you made in OLDDIR to NEWDIR.
+It will not apply committed changes, even if they could be applied to NEWDIR
+with a regular merge.  The changes will remain in OLDDIR, but you can use **bzr
+revert OLDDIR** to remove them, once you're satisfied with NEWDIR.
+
+NEWDIR does not have to be a copy of OLDDIR, but they should be related.
+The more different they are, the greater the chance of conflicts.

=== modified file 'setup.py'
--- setup.py	2006-08-11 07:10:09 +0000
+++ setup.py	2006-08-17 13:53:36 +0000
@@ -6,6 +6,9 @@
  './setup.py --help' for more options
 """
 
+import os
+import sys
+
 import bzrlib
 
 ##
@@ -20,38 +23,9 @@
              'license':      'GNU GPL v2',
             }
 
-BZRLIB = {'packages': ['bzrlib',
-                       'bzrlib.benchmarks',
-                       'bzrlib.bundle',
-                       'bzrlib.bundle.serializer',
-                       'bzrlib.doc',
-                       'bzrlib.doc.api',
-                       'bzrlib.export',
-                       'bzrlib.plugins',
-                       'bzrlib.plugins.launchpad',
-                       'bzrlib.store',
-                       'bzrlib.store.revision',
-                       'bzrlib.store.versioned',
-                       'bzrlib.tests',
-                       'bzrlib.tests.blackbox',
-                       'bzrlib.tests.branch_implementations',
-                       'bzrlib.tests.bzrdir_implementations',
-                       'bzrlib.tests.interrepository_implementations',
-                       'bzrlib.tests.intertree_implementations',
-                       'bzrlib.tests.interversionedfile_implementations',
-                       'bzrlib.tests.repository_implementations',
-                       'bzrlib.tests.revisionstore_implementations',
-                       'bzrlib.tests.tree_implementations',
-                       'bzrlib.tests.workingtree_implementations',
-                       'bzrlib.transport',
-                       'bzrlib.transport.http',
-                       'bzrlib.ui',
-                       'bzrlib.util',
-                       'bzrlib.util.configobj',
-                       'bzrlib.util.effbot.org',
-                       'bzrlib.util.elementtree',
-                      ],
-         }
+# The list of packages is automatically generated later. Add other things
+# that are part of BZRLIB here.
+BZRLIB = {}
 
 PKG_DATA = {# install files from selftest suite
             'package_data': {'bzrlib': ['doc/api/*.txt',
@@ -63,9 +37,6 @@
 # Reinvocation stolen from bzr, we need python2.4 by virtue of bzr_man
 # including bzrlib.help
 
-import os
-import sys
-
 try:
     version_info = sys.version_info
 except AttributeError:
@@ -91,6 +62,30 @@
     os.unsetenv(REINVOKE)
 
 
+def get_bzrlib_packages():
+    """Recurse through the bzrlib directory, and extract the package names"""
+
+    packages = []
+    base_path = os.path.dirname(os.path.abspath(bzrlib.__file__))
+    for root, dirs, files in os.walk(base_path):
+        if '__init__.py' in files:
+            assert root.startswith(base_path)
+            # Get just the path below bzrlib
+            package_path = root[len(base_path):]
+            # Remove leading and trailing slashes
+            package_path = package_path.strip('\\/')
+            if not package_path:
+                package_name = 'bzrlib'
+            else:
+                package_name = ('bzrlib.' +
+                            package_path.replace('/', '.').replace('\\', '.'))
+            packages.append(package_name)
+    return sorted(packages)
+
+
+BZRLIB['packages'] = get_bzrlib_packages()
+
+
 from distutils.core import setup
 from distutils.command.install_scripts import install_scripts
 from distutils.command.build import build

=== modified file 'tools/doc_generate/autodoc_man.py'
--- tools/doc_generate/autodoc_man.py	2006-08-06 23:35:09 +0000
+++ tools/doc_generate/autodoc_man.py	2006-08-16 04:04:30 +0000
@@ -110,19 +110,19 @@
     if options:
         option_str = "\nOptions:\n"
         for option_name, option in sorted(options.items()):
-            l = '    --' + option_name
-            if option.type is not None:
-                l += ' ' + option.argname.upper()
-            short_name = option.short_name()
-            if short_name:
-                assert len(short_name) == 1
-                l += ', -' + short_name
-            l += (30 - len(l)) * ' ' + option.help
-            # TODO: Split help over multiple lines with
-            # correct indenting and wrapping.
-            wrapped = textwrap.fill(l, initial_indent='',
-                                    subsequent_indent=30*' ')
-            option_str = option_str + wrapped + '\n'       
+            for name, short_name, argname, help in option.iter_switches():
+                l = '    --' + name
+                if argname is not None:
+                    l += ' ' + argname
+                if short_name:
+                    assert len(short_name) == 1
+                    l += ', -' + short_name
+                l += (30 - len(l)) * ' ' + help
+                # TODO: Split help over multiple lines with
+                # correct indenting and wrapping.
+                wrapped = textwrap.fill(l, initial_indent='',
+                                        subsequent_indent=30*' ')
+                option_str = option_str + wrapped + '\n'       
 
     aliases_str = ""
     if cmd.aliases:

=== added file 'tools/rst2html.py'
--- tools/rst2html.py	1970-01-01 00:00:00 +0000
+++ tools/rst2html.py	2006-08-17 12:14:57 +0000
@@ -0,0 +1,25 @@
+#!/usr/bin/env python
+
+# Author: David Goodger
+# Contact: goodger@python.org
+# Revision: $Revision: 3901 $
+# Date: $Date: 2005-09-25 17:49:54 +0200 (Sun, 25 Sep 2005) $
+# Copyright: This module has been placed in the public domain.
+
+"""
+A minimal front end to the Docutils Publisher, producing HTML.
+"""
+
+try:
+    import locale
+    locale.setlocale(locale.LC_ALL, '')
+except:
+    pass
+
+from docutils.core import publish_cmdline, default_description
+
+
+description = ('Generates (X)HTML documents from standalone reStructuredText '
+               'sources.  ' + default_description)
+
+publish_cmdline(writer_name='html', description=description)

